//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue;

import figue.path.PathInterface;
import figue.resource.FontMetricInterface;
import figue.resource.FontInterface;
import figue.resource.PortableFontInterface;
import figue.resource.ColorInterface;
import figue.resource.WrongResourceException;
import figue.geometry.Rectangle;
import figue.geometry.Dimension;
import figue.error.Warning;
import figue.error.ShouldNotBeCalledError;
import figue.multithread.StoppedFacadeException;
import figue.box.LightweightInterface;
import figue.box.LightweightComponentAdapter;
import figue.box.GlyphInterface;
import figue.internal.AvailableVersion;
import figue.internal.ContextInterface;
import figue.internal.Cursor;
import figue.internal.Globals;
import figue.internal.GlyphViewerInterface;
import figue.internal.Context;

import java.util.Hashtable;
import java.io.IOException;
import java.awt.Graphics;
import java.io.Writer;
import java.io.InputStream;
import antlr.ParserException;
import java.io.IOException;

/**
 * [PENDING: update the comments] ASCIIFacade.
 */

public final class ASCIIFacade extends Facade {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/ASCIIFacade.java,v $ $revision$ $Date: 1998/11/02 15:54:27 $ Copyright 1998 INRIA.";

    public ASCIIFacade(FacadeInitializer anInitializer) {
	super(AvailableVersion.TEXTUAL.clone(null),anInitializer);
	internalInit(anInitializer);
    }
  
    public ASCIIFacade() {
	this(new FacadeInitializer());
    }

    public ASCIIFacade(InputStream anInput) throws ParserException, IOException {
	this(new FacadeInitializer(),anInput);
    }

    public ASCIIFacade(FacadeInitializer anInitializer,
		       InputStream anInput) throws ParserException, IOException {
	super(AvailableVersion.TEXTUAL.clone(null),anInitializer,anInput);
	internalInit(anInitializer);
    }

    public ASCIIFacade(FacadeInitializer anInitializer, FacadeInterface aFacade) throws WrongResourceException, CloneNotSupportedException {
	super(AvailableVersion.TEXTUAL.clone(null),anInitializer,aFacade);
	internalInit(anInitializer);
    }

    private final void internalInit(FacadeInitializer anInitializer) {
	_context = new ASCIIContext(this);
	getVersion().setContext(_context);
	_rectangleCacheForBoundingBox = new Rectangle();
	_rectangleCacheForRefreshZone = new Rectangle();
	init();
    }

    final GlyphViewerInterface getViewer() {
	reportError(new ShouldNotBeCalledError());
	return null;
    }

    public final ContextInterface getContext() {
	return _context;
    }

    public final void update(Writer anOutputStream) {
	if ( Globals.DEBUG_MULTI_THREAD ) {
	    System.out.println("-> update");
	}
	try {
	    format();
	    synchronized ( getInactive() ) {
		while ( true ) {
		    synchronized ( this ) {
			if ( getThreadState().isInactive() ) {
			    break;
			}
		    }
		    getInactive().await();
		}
	    }
// 	    _context.awaitClean(getRootGlyph());
	    if ( anOutputStream != null ) {
		final Dimension theSize = getRootGlyph().getSizeValue();
		final int theHeight = theSize.getHeight();
		final int theWidth = theSize.getWidth();
		final int x = 0;
		int y = 0;

		synchronized ( getDrawable() ) {
		main_loop: while ( true ) {
		    synchronized ( this ) {
			if ( getThreadState().isDrawable() ) {
			    getThreadState().startDraw();
			    while ( true ) {
				_context.setOutput(anOutputStream);
				if ( ( y + Globals.DEFAULT_FORMATTING_HEIGHT ) <= theHeight ) {
				    _context.setClipBounds(x,y,theWidth,Globals.DEFAULT_FORMATTING_HEIGHT);
				}
				else {
				    _context.setClipBounds(x,y,theWidth,theHeight - y);
				}
				executeDrawTask();
				y+=Globals.DEFAULT_FORMATTING_HEIGHT;
				if ( y >= theHeight ) {
				    break main_loop;
				}
			    }
			}
		    }
		    getDrawable().await();
		}
		}
		endDraw();
	    }
	}
	catch ( StoppedFacadeException anException ) {
	    return;
	}
	if ( Globals.DEBUG_MULTI_THREAD ) {
	    System.out.println("<- update");
	}
    }

    protected final void executeDrawTask() {
	if ( Globals.DEBUG_MULTI_THREAD ) {
	    System.out.println("\nbegin draw\n");
	}
	GlyphInterface theRoot = getRootGlyph();
	_rectangleCacheForRefreshZone.setBounds(_context.getClipBounds());
	_rectangleCacheForBoundingBox.setBounds(0,0,0,0);
	getVersion().draw(theRoot,null,getSelectionPath(),
			  _rectangleCacheForRefreshZone,_rectangleCacheForBoundingBox);
	_context.setOutput(null);
	if ( Globals.DEBUG_MULTI_THREAD ) {
	    System.out.println("\nend draw\n");
	}
    }

    private final void format() throws StoppedFacadeException {
	if ( Globals.DEBUG_MULTI_THREAD ) {
	    System.out.println("-> format");
	}
	boolean theFormattingFlag = false;
	synchronized ( getFormatable() ) {
	    while ( true ) {
		synchronized ( this ) {
		    if ( ( getRootGlyph().numberOfChildren() == 0 ) || 
			 ( ( ! getRootGlyph().isDirty() ) ) && ( ! getRootGlyph().isUnstable() ) ) {
			break;
		    }
		    if ( getThreadState().isFormatting() ) {
			theFormattingFlag = true;
			break;
		    }
		    if ( getThreadState().isFormattable() ) {
			theFormattingFlag = true;
			getThreadState().startFormat();
			break;
		    }
		}
		getFormatable().await();
	    }
	}
	if ( theFormattingFlag ) {
	    awakeFormatting();
	}
	if ( Globals.DEBUG_MULTI_THREAD ) {
	    System.out.println("<- format");
	}
    }

    public final void refresh(Rectangle aRectangle) {
	reportError(new ShouldNotBeCalledError());
    }

    public final void refresh() {
	reportError(new ShouldNotBeCalledError());
    }

    protected final void insureVisibleWhenInactive(PathInterface aPath) {
	reportError(new ShouldNotBeCalledError());
    }

    protected final void init() {
	initDone();
    }

    protected final void dispose() {
	_context.dispose();
	_context = null;
	_rectangleCacheForBoundingBox = null;
	_rectangleCacheForRefreshZone = null;
	super.dispose();
    }

    private ASCIIContext _context;
    private Rectangle _rectangleCacheForBoundingBox;
    private Rectangle _rectangleCacheForRefreshZone;

}

final class ASCIIContext extends Context {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/ASCIIFacade.java,v $ $revision$ $Date: 1998/11/02 15:54:27 $ Copyright 1998 INRIA.";
  
    public ASCIIContext(FacadeInterface aFacade) {
	super(aFacade,Constants.DEFAULT_FONT,Constants.DEFAULT_COLOR,Constants.DEFAULT_BACKGROUND);
	_output = null;
	_offBuffer = null;
	_clipBounds = new Rectangle();
	_registryFontMetrics = new Hashtable(Globals.AVERAGE_FONT_NUMBER,
					     Globals.FAST_ACCESS);
    }

    public final void init() {
    }

    public final void setCursor(Cursor aCursor) {
    }

   public final void setAwtContext(Graphics aGraphics) {
        reportError(new ShouldNotBeCalledError());
    }

    public final FontMetricInterface getFontMetric(FontInterface aFont) {
	final String theKey = aFont.getKey();
	FontMetricInterface theFontMetric = 
	    (FontMetricInterface)(_registryFontMetrics.get(theKey));
	if ( theFontMetric != null ) {
	    return theFontMetric;
	}
	theFontMetric = new ASCIIFontMetric(aFont); 
	_registryFontMetrics.put(theKey,theFontMetric);
	return theFontMetric;
    }

    public final void setOutput(Writer anOutputStream) {
	_output = anOutputStream;
    }

    public final void drawBytes(byte[] aByteArray, int x, int y) {
	--y;
	int theY = y - _clipBounds.getY();
	if ( ( theY < 0 ) || ( theY >= _offBuffer.length ) ) {
	    return;
	}
	for ( int i = 0; i < aByteArray.length; i++ ) {
	    _offBuffer[theY][x++] = aByteArray[i];
	}
    }

    public final void drawString(String aString, int x, int y) {
	drawBytes(aString.getBytes(),x,y);
    }

    public final void drawSymbol(PortableFontInterface aFont, String aString, int x, int y) {
	drawString(aString,x,y);
    }
  
    public final void drawChars(char[] aCharArray, int x, int y) {
	drawBytes((new String(aCharArray)).getBytes(),x,y);
    }
  
    public final void drawChar(char aChar, int x, int y) {
	--y;
	int theY = y - _clipBounds.getY();
	if ( ( theY < 0 ) || ( theY >= _offBuffer.length ) ) {
	    return;
	}
	byte[] theBytes = (new Character(aChar)).toString().getBytes();
	for ( int i = 0; i < theBytes.length; i++ ) {
	    _offBuffer[theY][x++] = theBytes[i];
	}
    }

    public final void fillRectangle(int x, int y, int width, int height) {
	;
    }

    public final void drawRectangle(int x, int y, int width, int height, ColorInterface aColor) {
	;
    }

    public final void drawLine(int x1, int y1, int x2, int y2) {
	;
    }

    public final void drawLightweightComponent(LightweightInterface aLightweight, int x, int y) {
	;
    }

    public final boolean isComplientWith(LightweightComponentAdapter aLightweight) {
	return false;
    }

    public final void applyFontToLightweightComponent(FontInterface aFont,
						      LightweightInterface aLightweight) {
	aLightweight.setFont(null);
    }
  
    public final Rectangle getRefreshZone() {
	reportError(new ShouldNotBeCalledError());
	return null;
    }

    public final Rectangle getClipBounds() {
	return _clipBounds;
    }
  
    public final void setClipBounds(int x, int y, int width,int height) {
	if ( ( _offBuffer == null ) || 
	     ( width > _clipBounds.getWidth() ) || ( height > _clipBounds.getHeight() ) ) {
	    _offBuffer = new byte[height + 1][width];
	}
	_clipBounds.setBounds(x,y,width,height);
    }

    public final void setRefreshZone(int x, int y, int width,int height) {
	reportError(new ShouldNotBeCalledError());
    }

    public final void flush() {
	try {
	    final int theHeight = _clipBounds.getHeight();
	    final int theWidth = _clipBounds.getWidth();
	    int theEndOfLine;
	    for ( int i = 0; i < theHeight; i++ ) {
		theEndOfLine = theWidth;
		while ( --theEndOfLine > 0 ) {
		    if ( _offBuffer[i][theEndOfLine] != 0 ) {
			break;
		    }
		}
		for ( int j = 0; j <= theEndOfLine; j++ ) {
		    if ( _offBuffer[i][j] != 0 ) {
			_output.write(_offBuffer[i][j]);
		    }
		    else {
			_output.write(' ');
		    }
		    _offBuffer[i][j] = 0;
		}
		_output.write('\n');
	    }
	    _output.flush();
	}
	catch ( IOException anError ) {
	    reportError(new Warning("IO error"));
	}
    }

    public final void touch() {
	;
    }

    public final void dispose() {
	_output = null;
	_offBuffer = null;
	_clipBounds = null;
	_registryFontMetrics.clear();
	_registryFontMetrics = null;
	super.dispose();
    }

    private Writer _output;
    private byte[][] _offBuffer;
    private Rectangle _clipBounds;
    private Hashtable _registryFontMetrics;

}

class ASCIIFontMetric implements FontMetricInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/ASCIIFacade.java,v $ $revision$ $Date: 1998/11/02 15:54:27 $ Copyright 1998 INRIA.";
  
    public ASCIIFontMetric(FontInterface aFont) {
	_font = aFont;
    }
  
    public final int getAscent() {
	return 1;
    }
  
    public final int getDescent() {
	return 0;
    }
  
    public final int bytesWidth(byte[] aByteArray) {
	return aByteArray.length;
    }

    public final int stringWidth(String aString) {
	return aString.length();
    }
  
    public final int getHeight() {
	return 1;
    }

    public final FontInterface getFont() {
	return _font;
    }

    public final boolean equals(Object anObject) {
	if ( this == anObject ) {
	    return true;
	}
	if ( getClass() != anObject.getClass() ) {
	    return false;
	}
	return _font.equals(((ASCIIFontMetric)anObject).getFont());
    }
  
    private FontInterface _font;
  
}

