//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.path.SelectionPathInterface;
import figue.resource.FontMetricInterface;
import figue.resource.StyleOwnerInterface;
import figue.resource.ColorInterface;
import figue.resource.FontInterface;
import figue.resource.StyleInterface;
import figue.resource.StyleDefered;
import figue.resource.StyleInitializer;
import figue.geometry.Rectangle;
import figue.geometry.Dimension;
import figue.geometry.Transformer;
import figue.format.DistanceInterface;
import figue.format.AlternativeInterface;
import figue.format.ModificationInterface;
import figue.format.StrategyReadyToFormat;
import figue.format.StrategyImproveSizeEstimation;
import figue.format.StrategySetOrigin;
import figue.format.StrategySetSize;
import figue.format.StopFormattingSonsException;
import figue.error.DefaultErrorHandler;
import figue.error.ShouldNotBeCalledError;
import figue.error.Warning;
import figue.internal.Globals;
import figue.internal.ContextInterface;
import figue.internal.ContextMementoInterface;

import java.util.Hashtable;
import java.io.Writer;
import java.io.IOException;
import figue.io.Utility;

/**
 * [PENDING: update the comments] Atom.
 */

public abstract class Atom {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/Atom.java,v $ $revision$ $Date: 1998/11/02 15:54:34 $ Copyright 1998 INRIA.";

    protected Atom() {
	super();
    }
 
    public final static AtomAbstract newAtom(String aString) {
	if ( Globals.ROBUST ) {
	    return new UnsharedAtom(aString);
	}
	else {
	    return new SharedAtom(aString);
	}
    }
  
    public final static AtomAbstract newAtom(char aChar) {
	if ( Globals.ROBUST ) {
	    return new UnsharedAtom(aChar);
	}
	else {
	    return new SharedAtom(aChar);
	}
    }

    public final static AtomAbstract newAtom(String aString, String aStyleName) {
	return new KeyWord(aString,aStyleName);
    }

    abstract FlyWeightChar newFlyWeightChar(char aChar, FontMetricInterface aFontMetric);
    abstract FlyWeightChar newFlyWeightChar(char aChar);
    abstract FlyWeightString newFlyWeightString(String aString);
    abstract FlyWeightString newFlyWeightString(String aString, FontMetricInterface aFontMetric);
    abstract FlyWeightKeyWord newFlyWeightKeyWord(String aText,FontMetricInterface aFontMetric);
    abstract FlyWeightKeyWord newFlyWeightKeyWord(String aText);
  
    static Atom PATCH; /// TEMPO PATCH FOR LINUX

    static {
	/// PATCH DRAWBYTES BUG UNDER LINUX
	try {
	    if ( System.getProperty("os.name").trim().toUpperCase().equals("LINUX") ) {
		DefaultErrorHandler.INSTANCE.reportError(new Warning("Using patch for drawByte bug under linux in Atom.java."));
		PATCH = new LinuxPatchAtom();
	    }
	    else {
		PATCH = new StandardAtom();
	    }
	}
	catch ( SecurityException  anException ) {
	    ;
	}
    }
  
}

final class StandardAtom extends Atom {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/Atom.java,v $ $revision$ $Date: 1998/11/02 15:54:34 $ Copyright 1998 INRIA.";

    FlyWeightChar newFlyWeightChar(char aChar) {
	return new FlyWeightChar(aChar);
    }
    FlyWeightChar newFlyWeightChar(char aChar, FontMetricInterface aFontMetric) {
	return new FlyWeightChar(aChar,aFontMetric);
    }
    FlyWeightString newFlyWeightString(String aString) {
	return new FlyWeightString(aString);
    }
    FlyWeightString newFlyWeightString(String aString, FontMetricInterface aFontMetric) {
	return new FlyWeightString(aString,aFontMetric);
    }
    FlyWeightKeyWord newFlyWeightKeyWord(String aText,FontMetricInterface aFontMetric) {
	return new FlyWeightKeyWord(aText,aFontMetric);
    }
    FlyWeightKeyWord newFlyWeightKeyWord(String aText) {
	return new FlyWeightKeyWord(aText);
    }
}

final class LinuxPatchAtom extends Atom {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/Atom.java,v $ $revision$ $Date: 1998/11/02 15:54:34 $ Copyright 1998 INRIA.";

    FlyWeightChar newFlyWeightChar(char aChar) {
	return new LinuxPatchFlyWeightChar(aChar);
    }
    FlyWeightChar newFlyWeightChar(char aChar, FontMetricInterface aFontMetric) {
	return new LinuxPatchFlyWeightChar(aChar,aFontMetric);
    }
    FlyWeightString newFlyWeightString(String aString) {
	return new LinuxPatchFlyWeightString(aString);
    }
    FlyWeightString newFlyWeightString(String aString, FontMetricInterface aFontMetric) {
	return new LinuxPatchFlyWeightString(aString,aFontMetric);
    }
    FlyWeightKeyWord newFlyWeightKeyWord(String aText,FontMetricInterface aFontMetric) {
	return new LinuxPatchFlyWeightKeyWord(aText,aFontMetric);
    }
    FlyWeightKeyWord newFlyWeightKeyWord(String aText) {
	return new LinuxPatchFlyWeightKeyWord(aText);
    }
}

final class SharedAtom extends AtomAbstract {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/Atom.java,v $ $revision$ $Date: 1998/11/02 15:54:34 $ Copyright 1998 INRIA.";
  
    public SharedAtom(String aString) {
	super();
	//     _flyWeight = new FlyWeightString(aString);
	_flyWeight = Atom.PATCH.newFlyWeightString(aString); /// TEMPO PATCH FOR LINUX
    }

    public SharedAtom(char aChar) {
	super();
	//     _flyWeight = new FlyWeightChar(aChar);
	_flyWeight = Atom.PATCH.newFlyWeightChar(aChar); /// TEMPO PATCH FOR LINUX
    }

    protected final void finalize() {
	if ( _flyWeight != null ) {
	    _flyWeight.dispose();
	}
    }
  
    protected final String getText() {
	return _flyWeight.getText();
    }

    public final Dimension getSizeValue() {
	return _flyWeight.getSizeValue();
    }

    public final void setSize(Dimension aSize) {
	reportError(new ShouldNotBeCalledError());
    }

    public final void resetSize(Dimension aSize, ContextInterface aContext) {
	reportError(new ShouldNotBeCalledError());
    }

    public final void format2D(ContextInterface aContext,
			       FontMetricInterface aFontMetric,
			       AlternativeInterface anAlternative,
			       DistanceInterface aDistance,
			       StrategyReadyToFormat aReadyToFormatStrategy,
			       StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
			       
			       StrategySetOrigin aSetOriginStrategy,
			       StrategySetSize aSetSizeStrategy,
			       boolean aMustFormatFlag)
	throws StopFormattingSonsException {
	_flyWeight = _flyWeight.format2D(aContext,aFontMetric,aDistance);
	aDistance.checkAfter(this,anAlternative);
    }

    public final boolean draw2D(ContextInterface aContext,
				Transformer aTransformer,
				SelectionPathInterface aSelectionPath,
				Rectangle aRefreshZone,
				Rectangle aBoundingBox) {
	if ( isClean() ) {
	    draw2DClean(aContext,aTransformer,aSelectionPath,aRefreshZone,aBoundingBox);
	    return true;
	}
	else {
	    return false;
	}
    }
  
    protected final void draw2DClean(ContextInterface aContext,
				     Transformer aTransformer,
				     SelectionPathInterface aSelectionPath,
				     Rectangle aRefreshZone,
				     Rectangle aBoundingBox) {
	_flyWeight.draw2DClean(this,aContext,aTransformer,aSelectionPath);
    }

    protected final boolean draw2D(ContextInterface aContext,
				   Transformer aTransformer,
				   Rectangle aRefreshZone,
				   Rectangle aBoundingBox) {
	if ( isClean() ) {
	    draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
	    return true;
	}
	else {
	    return false;
	}
    }

    protected final void draw2DClean(ContextInterface aContext,
				     Transformer aTransformer,
				     Rectangle aRefreshZone,
				     Rectangle aBoundingBox) {
	_flyWeight.draw2DClean(this,aContext,aTransformer);
    }

    private FlyWeightAtomInterface _flyWeight; 
  
}

class FlyWeightString extends FlyWeightAtom { /// TEMPO NOT FINAL FOR LINUX PATCH

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/Atom.java,v $ $revision$ $Date: 1998/11/02 15:54:34 $ Copyright 1998 INRIA.";
  
    public FlyWeightString(String aText) {
	this(aText,null);
    }
  
    public FlyWeightString(String aText,
			   FontMetricInterface aFontMetric) {
	super(aFontMetric);
	_text = aText;
	_bytes = null;
    }

    protected final void insertInRegistry(ContextInterface aContext) {
	_bytes = _text.getBytes();
	final Hashtable _registry = aContext.getFlyWeigthRegistryForString();
	Hashtable theFontRegistry = (Hashtable)_registry.get(_text);
	if ( theFontRegistry == null ) {
	    theFontRegistry = new Hashtable(Globals.AVERAGE_FLY_WEIGHT_ATOM_FONT_NUMBER,
					    Globals.FAST_ACCESS);
	    _registry.put(_text,theFontRegistry);
	}
	theFontRegistry.put(getFontMetric().getFont().getKey(),this);
    }

    protected final void removeFromRegistry(ContextInterface aContext) {
	final Hashtable _registry = aContext.getFlyWeigthRegistryForString();
	Hashtable theFontRegistry = (Hashtable)_registry.get(_text);
	theFontRegistry.remove(getFontMetric().getFont().getKey());
	if ( theFontRegistry.isEmpty() ) {
	    _registry.remove(_text);
	}
    }
  
    public final String getText() {
	return _text;
    }

    public final int computeWidth(FontMetricInterface aFontMetric) {
	return aFontMetric.bytesWidth(_bytes);
    }

    protected final FlyWeightAtomInterface updateRegistry(ContextInterface aContext,
							  FontMetricInterface aFontMetric) {
	FlyWeightString theFlyWeight = this;
	final Hashtable _registry = aContext.getFlyWeigthRegistryForString();
	if ( theFlyWeight.getFontMetric() == null ) {
	    theFlyWeight.setFontMetric(aFontMetric);
	    final String theText = theFlyWeight._text;
	    theFlyWeight = null;
	    Hashtable theFontRegistry = (Hashtable)_registry.get(theText);
	    if ( theFontRegistry != null ) {
		theFlyWeight = (FlyWeightString)(theFontRegistry.get(aFontMetric.getFont().getKey()));
	    }
	    if ( theFlyWeight == null ) {
		// 	theFlyWeight = new FlyWeightString(theText,aFontMetric);
		theFlyWeight = Atom.PATCH.newFlyWeightString(theText,aFontMetric); /// TEMPO PATCH FOR LINUX
	    }
	    theFlyWeight.incrReferenceCounter(aContext);
	}
	else {
	    if ( ! theFlyWeight.getFontMetric().equals(aFontMetric) ) {
		final String theText = theFlyWeight._text;
		theFlyWeight.decrReferenceCounter(aContext);
		theFlyWeight = null;
		Hashtable theFontRegistry = (Hashtable)_registry.get(theText);
		if ( theFontRegistry != null ) {
		    theFlyWeight = (FlyWeightString)(theFontRegistry.get(aFontMetric.getFont().getKey()));
		}
		if ( theFlyWeight == null ) {
		    // 	  theFlyWeight = new FlyWeightString(theText,aFontMetric);
		    theFlyWeight = Atom.PATCH.newFlyWeightString(theText,aFontMetric); /// TEMPO PATCH FOR LINUX
		}
		theFlyWeight.incrReferenceCounter(aContext);
	    }
	    else {
		theFlyWeight.incrCounter();
	    }
	}   
	return theFlyWeight;
    }
  
    public void draw2DClean(GlyphInterface anOwner, /// TEMPO NOT FINAL FOR LINUX PATCH
			    ContextInterface aContext,
			    Transformer aTransformer) {
	final int theOriginX = anOwner.getOriginX();
	final int theOriginY = anOwner.getOriginY();
	if ( aContext.getFont().isUnderlined() ) {
	    final Dimension theSize = anOwner.getSizeValue();
	    final int theY = aTransformer.applyToY(theOriginY) + Globals.PIXEL_OFFSET_FOR_UNDERLINE;
	    aContext.drawLine(aTransformer.applyToX(theOriginX) + theSize.getUpperLeftX(),
			      theY,
			      aTransformer.applyToX(theOriginX) + theSize.getBottomRightX(),
			      theY);
	}
	aContext.drawBytes(_bytes,
			   aTransformer.applyToX(theOriginX),
			   aTransformer.applyToY(theOriginY));
    }

    String _text;/// TEMPO NOT PRIVATE FOR LINUX PATCH
private byte[] _bytes;

}

final class LinuxPatchFlyWeightString extends FlyWeightString { /// TEMPO FOR LINUX PATCH

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/Atom.java,v $ $revision$ $Date: 1998/11/02 15:54:34 $ Copyright 1998 INRIA.";
  
    public LinuxPatchFlyWeightString(String aText) {
	super(aText);
    }
  
    public LinuxPatchFlyWeightString(String aText,
				     FontMetricInterface aFontMetric) {
	super(aText,aFontMetric);
    }
    public void draw2DClean(GlyphInterface anOwner, /// TEMPO NOT FINAL FOR LINUX PATCH
			    ContextInterface aContext,
			    Transformer aTransformer) {
	final int theOriginX = anOwner.getOriginX();
	final int theOriginY = anOwner.getOriginY();
	if ( aContext.getFont().isUnderlined() ) {
	    final Dimension theSize = anOwner.getSizeValue();
	    final int theY = aTransformer.applyToY(theOriginY) + Globals.PIXEL_OFFSET_FOR_UNDERLINE;
	    aContext.drawLine(aTransformer.applyToX(theOriginX) + theSize.getUpperLeftX(),
			      theY,
			      aTransformer.applyToX(theOriginX) + theSize.getBottomRightX(),
			      theY);
	}
	aContext.drawString(_text,
			    aTransformer.applyToX(theOriginX),
			    aTransformer.applyToY(theOriginY));
    }
}

class LinuxPatchFlyWeightChar extends FlyWeightChar { /// TEMPO FOR LINUX PATCH

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/Atom.java,v $ $revision$ $Date: 1998/11/02 15:54:34 $ Copyright 1998 INRIA.";

    public LinuxPatchFlyWeightChar(char aChar) {
	super(aChar);
    }
    public LinuxPatchFlyWeightChar(char aChar,FontMetricInterface aFontMetric) {
	super(aChar,aFontMetric);
    }
    public void draw2DClean(GlyphInterface anOwner,
			    ContextInterface aContext,
			    Transformer aTransformer) {
	final int theOriginX = anOwner.getOriginX();
	final int theOriginY = anOwner.getOriginY();
	if ( aContext.getFont().isUnderlined() ) {
	    final Dimension theSize = anOwner.getSizeValue();
	    final int theY = aTransformer.applyToY(theOriginY) + Globals.PIXEL_OFFSET_FOR_UNDERLINE;
	    aContext.drawLine(aTransformer.applyToX(theOriginX) + theSize.getUpperLeftX(),
			      theY,
			      aTransformer.applyToX(theOriginX) + theSize.getBottomRightX(),
			      theY);
	}
	aContext.drawChar(_char,
			  aTransformer.applyToX(theOriginX),
			  aTransformer.applyToY(theOriginY));
    }
}


final class UnsharedAtom extends AtomAbstract {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/Atom.java,v $ $revision$ $Date: 1998/11/02 15:54:34 $ Copyright 1998 INRIA.";
  
    public UnsharedAtom(String aString) {
	super();
	_bytes = aString.getBytes();
	_size = null;
    }

    public UnsharedAtom(char aChar) {
	this((new Character(aChar)).toString());
    }
  
    protected final String getText() { 
	return new String(_bytes); 
    }

    public final Dimension getSizeValue() {
	return _size;
    }

    public final void setSize(Dimension aSize) {
	_size = aSize;
    }

    public final void resetSize(Dimension aSize, ContextInterface aContext) {
	if ( ( _size != null ) && ( ! _size.sameShape(aSize) ) ) {
	    ModificationInterface theModification = getModification();
	    if ( ! theModification.isModified() ) {
		theModification = aContext.newModification();
		setModification(theModification);
	    }
	    theModification.setSize(getSizeValue());
	}
	_size = aSize;
    }
  
    public final void format2D(ContextInterface aContext,
			       FontMetricInterface aFontMetric,
			       AlternativeInterface anAlternative,
			       DistanceInterface aDistance,
			       StrategyReadyToFormat aReadyToFormatStrategy,
			       StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
			       
			       StrategySetOrigin aSetOriginStrategy,
			       StrategySetSize aSetSizeStrategy,
			       boolean aMustFormatFlag)
	throws StopFormattingSonsException {
	aSetSizeStrategy.doAlgorithm(this,
				     new Dimension(aFontMetric.getAscent(),
						   aFontMetric.bytesWidth(_bytes),
						   aFontMetric.getDescent()));
	aDistance.checkAfter(this,anAlternative);
    }
  
    public final boolean draw2D(ContextInterface aContext,
				Transformer aTransformer,
				SelectionPathInterface aSelectionPath,
				Rectangle aRefreshZone,
				Rectangle aBoundingBox) {
	if ( isClean() ) {
	    draw2DClean(aContext,aTransformer,aSelectionPath,aRefreshZone,aBoundingBox);
	    return true;
	}
	else {
	    return false;
	}
    }

    protected final void draw2DClean(ContextInterface aContext,
				     Transformer aTransformer,
				     SelectionPathInterface aSelectionPath,
				     Rectangle aRefreshZone,
				     Rectangle aBoundingBox) {
	ContextMementoInterface theMemento = null;
	theMemento = draw2DSelections(aContext,aTransformer,aSelectionPath);
	underline(aContext,aTransformer);
	aContext.drawBytes(_bytes,
			   aTransformer.applyToX(getOriginX()),
			   aTransformer.applyToY(getOriginY()));
	if ( theMemento != null ) {
	    aContext.setState(theMemento);
	    aContext.disposeMemento(theMemento);
	}
    }

    protected final boolean draw2D(ContextInterface aContext,
				   Transformer aTransformer,
				   Rectangle aRefreshZone,
				   Rectangle aBoundingBox) {
	if ( isClean() ) {
	    draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
	    return true;
	}
	else {
	    return false;
	}
    }

    protected final void draw2DClean(ContextInterface aContext,
				     Transformer aTransformer,
				     Rectangle aRefreshZone,
				     Rectangle aBoundingBox) {
	underline(aContext,aTransformer);
	aContext.drawBytes(_bytes,
			   aTransformer.applyToX(getOriginX()),
			   aTransformer.applyToY(getOriginY()));
    }

    private final void underline(ContextInterface aContext,
				 Transformer aTransformer) {
	if ( aContext.getFont().isUnderlined() ) {
	    final Dimension theSize = getSizeValue();
	    final int theY = aTransformer.applyToY(getOriginY()) + Globals.PIXEL_OFFSET_FOR_UNDERLINE;
	    aContext.drawLine(aTransformer.applyToX(getOriginX()) + theSize.getUpperLeftX(),
			      theY,
			      aTransformer.applyToX(getOriginX()) + theSize.getBottomRightX(),
			      theY);
	}
    }

    private byte[] _bytes;
private Dimension _size;
  
}

final class KeyWord extends AtomAbstract implements  StyleOwnerInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/Atom.java,v $ $revision$ $Date: 1998/11/02 15:54:34 $ Copyright 1998 INRIA.";

    public KeyWord(String aString, String aStyleName) {
	super();
	_style = new StyleDefered(aStyleName,new StyleInitializer(this));
	//     _flyWeight = new FlyWeightKeyWord(aString);
	_flyWeight = Atom.PATCH.newFlyWeightKeyWord(aString);/// TEMPO PATCH FOR LINUX
    }

    public Object clone() throws CloneNotSupportedException {
	return Atom.newAtom(getText(),_style.getName());
    }

    protected final void finalize() {
	if ( _flyWeight != null ) {
	    _flyWeight.dispose();
	}
    }

    protected final String getText() {
	return _flyWeight.getText();
    }
  
    public final Dimension getSizeValue() {
	return _flyWeight.getSizeValue();
    }
  
    public final void setSize(Dimension aSize) {
	reportError(new ShouldNotBeCalledError());
    }
  
    public final void resetSize(Dimension aSize, ContextInterface aContext) {
	reportError(new ShouldNotBeCalledError());
    }

    public final void format2D(ContextInterface aContext,
			       FontMetricInterface aFontMetric,
			       AlternativeInterface anAlternative,
			       DistanceInterface aDistance,
			       StrategyReadyToFormat aReadyToFormatStrategy,
			       StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
			       
			       StrategySetOrigin aSetOriginStrategy,
			       StrategySetSize aSetSizeStrategy,
			       boolean aMustFormatFlag)
	throws StopFormattingSonsException {
	final FontInterface theFont = _style.getFont();
	FontMetricInterface theFontMetric;
	if ( theFont != null ) {
	    theFontMetric = aContext.getFontMetric(theFont);
	}
	else {
	    theFontMetric = aFontMetric;
	}
	_flyWeight = _flyWeight.format2D(aContext,theFontMetric,aDistance);
	aDistance.checkAfter(this,anAlternative);
    }

    public final boolean draw2D(ContextInterface aContext,
				Transformer aTransformer,
				SelectionPathInterface aSelectionPath,
				Rectangle aRefreshZone,
				Rectangle aBoundingBox) {
	if ( isClean() ) {
	    _flyWeight.draw2DClean(this,aContext,aTransformer,aSelectionPath);
	    return true;
	}
	else {
	    return false;
	}
    }
  
    protected final void draw2DClean(ContextInterface aContext,
				     Transformer aTransformer,
				     SelectionPathInterface aSelectionPath,
				     Rectangle aRefreshZone,
				     Rectangle aBoundingBox) {
	_flyWeight.draw2DClean(this,aContext,aTransformer,aSelectionPath);
    }

    protected final boolean draw2D(ContextInterface aContext,
				   Transformer aTransformer,
				   Rectangle aRefreshZone,
				   Rectangle aBoundingBox) {
	if ( isClean() ) {
	    _flyWeight.draw2DClean(this,aContext,aTransformer);
	    return true;
	}
	else {
	    return false;
	}
    }

    protected final void draw2DClean(ContextInterface aContext,
				     Transformer aTransformer,
				     Rectangle aRefreshZone,
				     Rectangle aBoundingBox) {
	_flyWeight.draw2DClean(this,aContext,aTransformer);
    }

    public final void setStyle(StyleInterface aStyle) {
	_style = aStyle;
    }

    public final StyleInterface getStyle() {
	return _style;
    }

    public void writeState(Writer anOutput) throws IOException {
	anOutput.write("<KeyWord Style=\"");
	Utility.writeStringAsXmlValue(anOutput,_style.getName());
	anOutput.write("\" Text=\"");
	Utility.writeStringAsXmlValueWithSpaces(anOutput,getText());
	anOutput.write("\"/>");
    }

    private StyleInterface _style;
    private FlyWeightKeyWord _flyWeight;

}

class FlyWeightKeyWord { /// TEMPO NOT FINAL FOR LINUX PATCH

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/Atom.java,v $ $revision$ $Date: 1998/11/02 15:54:34 $ Copyright 1998 INRIA.";

    public FlyWeightKeyWord(String aText,FontMetricInterface aFontMetric) {
	super();
	_text = aText;
	_bytes = null;
	_size = null;
	_counter = 0;
	_fontMetric = aFontMetric;
    }

    public FlyWeightKeyWord(String aText) {
	this(aText,null);
    }

    public final FontMetricInterface getFontMetric() {
	return _fontMetric;
    }

    public final void setFontMetric(FontMetricInterface aFontMetric) {
	_fontMetric = aFontMetric;
    }

    public final Dimension getSize() {
	return _size;
    }
  
    public final void setSize(Dimension aSize) {
	_size = aSize;
    }

    public final void dispose() {
	//     decrReferenceCounter(); ///// TEMPO SHOULD DECREMENT
    }
  
    protected final void incrCounter() {
	_counter++;
    }

    protected final void incrReferenceCounter(ContextInterface aContext) {
	if ( _counter++ == 0 ) {
	    insertInRegistry(aContext);
	}
    }

    protected final void decrReferenceCounter(ContextInterface aContext) {
	if ( --_counter == 0 ) {
	    removeFromRegistry(aContext);
	}
    }

    protected final void insertInRegistry(ContextInterface aContext) {
	_bytes = _text.getBytes();
	final Hashtable _registry = aContext.getFlyWeigthRegistryForKeyWord();
	Hashtable theFontRegistry = (Hashtable)_registry.get(_text);
	if ( theFontRegistry == null ) {
	    theFontRegistry = new Hashtable(Globals.AVERAGE_FLY_WEIGHT_ATOM_FONT_NUMBER,
					    Globals.FAST_ACCESS);
	    _registry.put(_text,theFontRegistry);
	}
	theFontRegistry.put(getFontMetric().getFont().getKey(),this);
    }

    protected final void removeFromRegistry(ContextInterface aContext) {
	final Hashtable _registry = aContext.getFlyWeigthRegistryForKeyWord();
	Hashtable theFontRegistry = (Hashtable)_registry.get(_text);
	theFontRegistry.remove(getFontMetric().getFont().getKey());
	if ( theFontRegistry.isEmpty() ) {
	    _registry.remove(_text);
	}
    }

    public final String getText() {
	return _text;
    }

    public final int computeWidth(FontMetricInterface aFontMetric) {
	return aFontMetric.bytesWidth(_bytes);
    }

    private final FlyWeightKeyWord updateRegistry(ContextInterface aContext,
						  FontMetricInterface aFontMetric) {
	FlyWeightKeyWord theFlyWeight = this;
	final Hashtable _registry = aContext.getFlyWeigthRegistryForKeyWord();
	if ( theFlyWeight.getFontMetric() == null ) {
	    theFlyWeight.setFontMetric(aFontMetric);
	    final String theText = theFlyWeight._text;
	    theFlyWeight = null;
	    Hashtable theFontRegistry = (Hashtable)_registry.get(theText);
	    if ( theFontRegistry != null ) {
		theFlyWeight = (FlyWeightKeyWord)(theFontRegistry.get(aFontMetric.getFont().getKey()));
	    }
	    if ( theFlyWeight == null ) {
		// 	theFlyWeight = new FlyWeightKeyWord(theText,aFontMetric);
		theFlyWeight = Atom.PATCH.newFlyWeightKeyWord(theText,aFontMetric); /// TEMPO PATCH FOR LINUX
	    }
	    theFlyWeight.incrReferenceCounter(aContext);
	}
	else {
	    if ( ! theFlyWeight.getFontMetric().equals(aFontMetric) ) {
		final String theText = theFlyWeight._text;
		theFlyWeight.decrReferenceCounter(aContext);
		theFlyWeight = null;
		Hashtable theFontRegistry = (Hashtable)_registry.get(theText);
		if ( theFontRegistry != null ) {
		    theFlyWeight = (FlyWeightKeyWord)(theFontRegistry.get(aFontMetric.getFont().getKey()));
		}
		if ( theFlyWeight == null ) {
		    // 	  theFlyWeight = new FlyWeightKeyWord(theText,aFontMetric);
		    theFlyWeight = Atom.PATCH.newFlyWeightKeyWord(theText,aFontMetric); /// TEMPO PATCH FOR LINUX
		}
		theFlyWeight.incrReferenceCounter(aContext);
	    }
	    else {
		theFlyWeight.incrCounter();
	    }
	}   
	return theFlyWeight;
    }

    public void draw2DClean(KeyWord anOwner, /// TEMPO NOT FINAL FOR LINUX PATCH
			    ContextInterface aContext,
			    Transformer aTransformer,
			    SelectionPathInterface aSelectionPath) {
	final StyleInterface theStyle = anOwner.getStyle();
	ContextMementoInterface theOldState = aContext.set(theStyle.getFont(),
							   theStyle.getColor(),
							   theStyle.getBackground());
	final int theOriginX = anOwner.getOriginX();
	final int theOriginY = anOwner.getOriginY();
	if ( theStyle.getBackground() != null ) {
	    final Dimension theSize = getSizeValue();
	    aContext.fillRectangle( 
				   aTransformer.applyToX(theOriginX) - theSize.getUpperLeftX(),
				   aTransformer.applyToY(theOriginY) - theSize.getUpperLeftY(),
				   theSize.getWidth(), 
				   theSize.getHeight());
	}
	ContextMementoInterface theMemento = anOwner.draw2DSelections(aContext,aTransformer,aSelectionPath);
	underline(anOwner,aContext,aTransformer);
	aContext.drawBytes(_bytes,
			   aTransformer.applyToX(theOriginX),
			   aTransformer.applyToY(theOriginY));
	if ( theMemento != null ) {
	    aContext.setState(theMemento);
	    aContext.disposeMemento(theMemento);
	}
	aContext.setState(theOldState); 
	aContext.disposeMemento(theOldState);
    }

    public void draw2DClean(KeyWord anOwner, /// TEMPO NOT FINAL FOR LINUX PATCH
			    ContextInterface aContext,
			    Transformer aTransformer) {
	final StyleInterface theStyle = anOwner.getStyle();
	ColorInterface theBackground = theStyle.getBackground();
	ContextMementoInterface theOldState = aContext.set(theStyle.getFont(),
							   theStyle.getColor(),
							   theBackground);
	final int x = anOwner.getOriginX();
	final int y = anOwner.getOriginY();
	if ( theBackground != null ) {
	    final Dimension theSize = getSizeValue();
	    aContext.fillRectangle( 
				   aTransformer.applyToX(x) - theSize.getUpperLeftX(),
				   aTransformer.applyToY(y) - theSize.getUpperLeftY(),
				   theSize.getWidth(), 
				   theSize.getHeight());
	}
	underline(anOwner,aContext,aTransformer);
	aContext.drawBytes(_bytes,
			   aTransformer.applyToX(x),
			   aTransformer.applyToY(y));
	aContext.setState(theOldState);
	aContext.disposeMemento(theOldState);
    }

protected final void underline(KeyWord anOwner, /// TEMPO NOT PRIVATE FOR LINUX PATCH
			       ContextInterface aContext,
			       Transformer aTransformer) {
    if ( aContext.getFont().isUnderlined() ) {
	final Dimension theSize = anOwner.getSizeValue();
	final int theOriginX = anOwner.getOriginX();
	final int theY = aTransformer.applyToY(anOwner.getOriginY()) + Globals.PIXEL_OFFSET_FOR_UNDERLINE;
	aContext.drawLine(aTransformer.applyToX(theOriginX) + theSize.getUpperLeftX(),
			  theY,
			  aTransformer.applyToX(theOriginX) + theSize.getBottomRightX(),
			  theY);
    }
}

public final FlyWeightKeyWord format2D(ContextInterface aContext,
				       FontMetricInterface aFontMetric,
				       DistanceInterface aDistance) {
    if ( ( _fontMetric != null ) && ( aFontMetric.equals(_fontMetric) ) ) {
	return this;
    }
    FlyWeightKeyWord theFlyWeight = updateRegistry(aContext,aFontMetric);
    final FontMetricInterface theFontMetric = theFlyWeight.getFontMetric();
    if ( theFlyWeight.getSize() == null ) {
	theFlyWeight.setSize(new Dimension(theFontMetric.getAscent(),
					   theFlyWeight.computeWidth(theFontMetric),
					   theFontMetric.getDescent()));
    }
    return theFlyWeight;
}

    public final Dimension getSizeValue() {
	return _size;
    }

String _text; /// TEMPO NOT PRIVATE FOR LINUX PATCH
private byte[] _bytes;
private Dimension _size;
private int _counter;
private FontMetricInterface _fontMetric;

}

class LinuxPatchFlyWeightKeyWord extends FlyWeightKeyWord { /// TEMPO FOR LINUX PATCH

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/Atom.java,v $ $revision$ $Date: 1998/11/02 15:54:34 $ Copyright 1998 INRIA.";

    public LinuxPatchFlyWeightKeyWord(String aText,FontMetricInterface aFontMetric) {
	super(aText,aFontMetric);
    }

    public LinuxPatchFlyWeightKeyWord(String aText) {
	super(aText);
    }

    public void draw2DClean(KeyWord anOwner, /// TEMPO NOT FINAL FOR LINUX PATCH
			    ContextInterface aContext,
			    Transformer aTransformer,
			    SelectionPathInterface aSelectionPath) {
	final StyleInterface theStyle = anOwner.getStyle();
	ContextMementoInterface theOldState = aContext.set(theStyle.getFont(),
							   theStyle.getColor(),
							   theStyle.getBackground());
	final int theOriginX = anOwner.getOriginX();
	final int theOriginY = anOwner.getOriginY();
	if ( theStyle.getBackground() != null ) {
	    final Dimension theSize = getSizeValue();
	    aContext.fillRectangle( 
				   aTransformer.applyToX(theOriginX) - theSize.getUpperLeftX(),
				   aTransformer.applyToY(theOriginY) - theSize.getUpperLeftY(),
				   theSize.getWidth(), 
				   theSize.getHeight());
	}
	ContextMementoInterface theMemento = anOwner.draw2DSelections(aContext,aTransformer,aSelectionPath);
	underline(anOwner,aContext,aTransformer);
	aContext.drawString(_text,
			    aTransformer.applyToX(theOriginX),
			    aTransformer.applyToY(theOriginY));
	if ( theMemento != null ) {
	    aContext.setState(theMemento);
	    aContext.disposeMemento(theMemento);
	}
	aContext.setState(theOldState); 
	aContext.disposeMemento(theOldState);
    }

    public void draw2DClean(KeyWord anOwner, /// TEMPO NOT FINAL FOR LINUX PATCH
			    ContextInterface aContext,
			    Transformer aTransformer) {
	final StyleInterface theStyle = anOwner.getStyle();
	ColorInterface theBackground = theStyle.getBackground();
	ContextMementoInterface theOldState = aContext.set(theStyle.getFont(),
							   theStyle.getColor(),
							   theBackground);
	final int x = anOwner.getOriginX();
	final int y = anOwner.getOriginY();
	if ( theBackground != null ) {
	    final Dimension theSize = getSizeValue();
	    aContext.fillRectangle( 
				   aTransformer.applyToX(x) - theSize.getUpperLeftX(),
				   aTransformer.applyToY(y) - theSize.getUpperLeftY(),
				   theSize.getWidth(), 
				   theSize.getHeight());
	}
	underline(anOwner,aContext,aTransformer);
	aContext.drawString(_text,
			    aTransformer.applyToX(x),
			    aTransformer.applyToY(y));
	aContext.setState(theOldState);
	aContext.disposeMemento(theOldState);
    }

}

