//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.geometry.Rectangle;
import figue.geometry.Dimension;
import figue.geometry.Transformer;
import figue.format.DistanceInterface;
import figue.format.DecalageInterface;
import figue.format.AlternativeInterface;
import figue.format.SonModificationStatus;
import figue.format.StrategyImproveSizeEstimation;
import figue.format.StrategySetOrigin;
import figue.format.StrategyComputeSizeTill;
import figue.format.ActionNewLine;
import figue.error.ShouldNotBeCalledError;
import figue.internal.Globals;
import figue.internal.PairGlyphPath;
import figue.internal.ContextInterface;

import java.io.IOException;
import java.io.Writer;
import figue.io.Utility;

/**
 * [PENDING: update the comments] AtomAbstract.
 * @level advanced
 */

public abstract class AtomAbstract extends Glyph {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/AtomAbstract.java,v $ $revision$ $Date: 1998/11/02 15:54:35 $ Copyright 1998 INRIA.";
  
    protected AtomAbstract() {
	super();
    }

    public Object clone() throws CloneNotSupportedException {
 	return Atom.newAtom(getText());
    }
  
    protected abstract String getText();

    public final PairGlyphPath getDeepestContains(int x, 
						  int y,
						  Rectangle aBoundingBox,
						  ContextInterface aContext) { 
	computeBoundingBox(aBoundingBox);
	if ( aBoundingBox.contains(x,y) ) {
	    return aContext.newPairGlyphPath(this);
	}
	return null;
    }
    
    public final void update() {
	;
    }
    
    public final void updateEager() {
	;
    }

    protected Dimension computeSize(ContextInterface aContext,
				    StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
				  
				    StrategySetOrigin aSetOriginStrategy,
				    StrategyComputeSizeTill aComputeSizeTillStrategy) {
	reportError(new ShouldNotBeCalledError());
	return null;
    }

    protected final StrategyImproveSizeEstimation chooseAlgorithm(StrategyImproveSizeEstimation aStrategy) {
	return StrategyImproveSizeEstimation.INACTIVE;
    }

    public final int computeDecalage2D(GlyphInterface aGlyph,
				       DecalageInterface aDecalage) {
	return 0;
    }

    public final int computeInitialDecalage2D(DecalageInterface aDecalage) {
	return 0;
    }
  
    public final int computeDistance2D(DistanceInterface aDistance,
				       DecalageInterface aDecalage) {
	return 0;
    }

    public final void improveSizeEstimationAndUpdateParents(ContextInterface aContext,
							    int aRank) {
	reportError(new ShouldNotBeCalledError());
    }

    public final void tryAlternative(AlternativeInterface anAlternative) {
	return;
    }

    public final void tryAlternativeForInitial(AlternativeInterface anAlternative) {
	return;
    }
  
    public final void tryAlternativeForDistance(DistanceInterface aDistance,
						AlternativeInterface anAlternative) {
	return;
    }
  
    public final void tryAlternativeForDecalage(DecalageInterface aDecalage,
						AlternativeInterface anAlternative) {
	return;
    }

    protected final void doTranslate(Rectangle aRectangle) {
	reportError(new ShouldNotBeCalledError());
    }
  
    protected final void undoTranslate(Rectangle aRectangle) {
	reportError(new ShouldNotBeCalledError());
    }

    public final void doTranslatesZones(Rectangle aRefreshZone,
					Rectangle aVisibleZone,
					int aRank) {
	reportError(new ShouldNotBeCalledError());
    }
    public final void undoTranslateZones(Rectangle aRefreshZone,
					 Rectangle aVisibleZone,
					 int aRank) {
	reportError(new ShouldNotBeCalledError());
    }

    protected final boolean sonModified(GlyphInterface aGlyph,
					SonModificationStatus aStatus,
					ContextInterface aContext) {
	reportError(new ShouldNotBeCalledError());
	return true;
    }

    public final void computeCuttingCosts(AlternativeInterface anAlternative,
					  Rectangle aRectangle,
					  Transformer aTransformer) {
	computeBoundingBox(aRectangle,aTransformer);
	final int theStart = aRectangle.getY() + 1;
	final int theEnd = aRectangle.getY() + aRectangle.getHeight() - 1;
	anAlternative.preComputeActionCostNewLine(new ActionNewLine(theStart - 1,Globals.GOOD));
	anAlternative.preComputeActionCostNewLine(new ActionNewLine(theEnd,Globals.BEST));
	for ( int theRank = theStart; theRank < theEnd; theRank++ ) {
	    anAlternative.preComputeActionCostNewLine(new ActionNewLine(theRank,Globals.FORBIDDEN));
	}
    }

    public final String toString() {
	StringBuffer theResult = new StringBuffer("Atom<");
	theResult.append("`").append(getText()).append("':");
	theResult.append(super.toString());
	theResult.append('>');
	return theResult.toString();
    }

    protected String getName() {
	return null;
    }

    public void writeState(Writer anOutput) throws IOException {
	Utility.writeStringAsXmlValueWithSpaces(anOutput,getText());
    }

}

