//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.path.PathInterface;
import figue.path.LinearPath;
import figue.path.SelectionPathInterface;
import figue.resource.FontInterface;
import figue.resource.FontInterface;
import figue.resource.FontMetricInterface;
import figue.resource.ColorInterface;
import figue.geometry.Rectangle;
import figue.geometry.Dimension;
import figue.geometry.Transformer;
import figue.geometry.Size;
import figue.format.DistanceInterface;
import figue.format.DecalageInterface;
import figue.format.AlternativeInterface;
import figue.format.SonModificationStatus;
import figue.format.StrategyReadyToFormat;
import figue.format.StrategyImproveSizeEstimation;
import figue.format.StrategySetOrigin;
import figue.format.StrategyComputeSizeTill;
import figue.format.StrategySetSize;
import figue.format.StopFormattingSonsException;
import figue.error.ShouldNotBeCalledError;
import figue.internal.ContextInterface;
import figue.internal.PairGlyphPath;
import figue.internal.ContextMementoInterface;

import java.io.Writer;
import java.io.IOException;

/**
 * [PENDING: update the comments] ChangeGraphicalContextAbstract.
 * @level advanced
 */

public abstract class ChangeGraphicalContextAbstract extends CompositeGlyphArity1 {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/ChangeGraphicalContextAbstract.java,v $ $revision$ $Date: 1998/11/02 15:54:35 $ Copyright 1998 INRIA.";
  
    protected ChangeGraphicalContextAbstract() {
	super();
	_oldState = null;
    }

    public abstract FontInterface getFont();
  
    public abstract ColorInterface getColor();

    public abstract ColorInterface getBackground();

    public final void setOrigin(int x, int y) {
	super.setOrigin(x,y);
	if ( numberOfChildren() != 0 ) {
	    getChild(0).setOrigin(x,y);
	}
    }

    public final void resetOrigin(int x, int y, ContextInterface aContext) {
	super.resetOrigin(x,y,aContext);
	if ( numberOfChildren() != 0 ) {
	    getChild(0).resetOrigin(x,y,aContext);
	}
    }

    protected final void doTranslate(Rectangle aRectangle) {
	;
    }
  
    protected final void undoTranslate(Rectangle aRectangle) {
	;
    }

    public final void doTranslatesZones(Rectangle aRefreshZone,
					Rectangle aVisibleZone,
					int aRank) {
	;
    }
    public final void undoTranslateZones(Rectangle aRefreshZone,
					 Rectangle aVisibleZone,
					 int aRank) {
	;
    }

    public final PairGlyphPath getDeepestContains(int x, 
						  int y, 
						  Rectangle aBoundingBox,
						  ContextInterface aContext) {
	if ( numberOfChildren() == 0 ) {
	    return null;
	}
	PairGlyphPath theResult
	    = getChild(0).getDeepestContains(x,y,aBoundingBox,aContext);
	if ( theResult != null ) {
	    PathInterface theExtendedPath = new LinearPath(0);
	    if ( theResult.getPath() != null ) {
		theExtendedPath.addChild(theResult.getPath());
	    }
	    theResult.setPath(theExtendedPath);
	}
	return theResult;
    }

    protected final void computeBoundingBox(Rectangle aRectangle,
					    Transformer aTransformer) {
	if ( numberOfChildren() != 0 ) {
	    getChild(0).computeBoundingBox(aRectangle,aTransformer);
	}
	else {
	    computeEmptyBoundingBox(aRectangle,aTransformer);
	}
    }
  
    public final void computeBoundingBox(Rectangle aRectangle) {
	if ( numberOfChildren() != 0 ) {
	    getChild(0).computeBoundingBox(aRectangle);
	}
	else {
	    computeEmptyBoundingBox(aRectangle);
	}
    }

    protected final void computeEstimatedBoundingBox(Rectangle aRectangle,
						     Transformer aTransformer) {
	if ( numberOfChildren() != 0 ) {
	    getChild(0).computeEstimatedBoundingBox(aRectangle,aTransformer);
	}
	else {
	    computeEmptyBoundingBox(aRectangle,aTransformer);
	}
    }
  
    protected final void computeEstimatedBoundingBox(Rectangle aRectangle) {
	if ( numberOfChildren() != 0 ) {
	    getChild(0).computeEstimatedBoundingBox(aRectangle);
	}
	else {
	    computeEmptyBoundingBox(aRectangle);
	}
    }

    private final void computeEmptyBoundingBox(Rectangle aRectangle,
					       Transformer aTransformer) {
	aRectangle.setBounds(getSizeValue());
    }

    private final void computeEmptyBoundingBox(Rectangle aRectangle) {
	aRectangle.setBounds(getSizeValue());
	aRectangle.translate(getOriginX(),getOriginY());
    }

    public final void format2D(ContextInterface aContext,
			       FontMetricInterface aFontMetric,
			       AlternativeInterface anAlternative,
			       DistanceInterface aDistance,
			       StrategyReadyToFormat aReadyToFormatStrategy,
			       StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
			       
			       StrategySetOrigin aSetOriginStrategy,
			       StrategySetSize aSetSizeStrategy,
			       boolean aMustFormatFlag)
	throws StopFormattingSonsException {
	setProceed();
	if ( numberOfChildren() == 0 ) {
	    aSetSizeStrategy.doAlgorithm(this,new Size());
	    return;
	}
	GlyphInterface theChild = getChild(0);
	aReadyToFormatStrategy.doAlgorithm(aContext,theChild);
	FontInterface theFont = getFont();
	final boolean theMustFormatFlag = aMustFormatFlag || ( ! theChild.isClean() );
	if ( theMustFormatFlag ) {
	    if ( theFont == null ) {
		theChild.format2D(aContext,aFontMetric,anAlternative,aDistance,aReadyToFormatStrategy,anImproveSizeEstimationStrategy,aSetOriginStrategy,aSetSizeStrategy,aMustFormatFlag);
	    }
	    else {
		theChild.format2D(aContext,aContext.getFontMetric(theFont),anAlternative,aDistance,aReadyToFormatStrategy,anImproveSizeEstimationStrategy,aSetOriginStrategy,aSetSizeStrategy,aMustFormatFlag);
	    }
	    theChild.setClean();
	    aSetSizeStrategy.doAlgorithm(this,new Dimension(theChild.getSizeValue()));
	}
	else {
	    System.out.println("already clean change context");
	}
    }

    public final void update() {
	;
    }
    
    public final void updateEager() {
	;
    }

    protected Dimension computeSize(ContextInterface aContext,
				    StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
				  
				    StrategySetOrigin aSetOriginStrategy,
				    StrategyComputeSizeTill aComputeSizeTillStrategy) {
	if ( numberOfChildren() != 0 ) {
	    return aComputeSizeTillStrategy.getExactOrEstimatedSize(getChild(0),0);
	}
	else {
	    return new Dimension();
	}
    }

    public final void improveSizeEstimationAndUpdateParents(ContextInterface aContext,
							    int aRank) {
	if ( ! isProceed() ) {
	    setProceed();
	}
	StrategyComputeSizeTill.THE_RANK.setRank(0);
	setSizeEstimation(computeSize(aContext,
				      StrategyImproveSizeEstimation.INACTIVE,
				      StrategySetOrigin.NO_INCREMENTAL,
				      StrategyComputeSizeTill.THE_RANK));
	getParent().improveSizeEstimationAndUpdateParents(aContext,getParent().indexOf(this));
    }

  
    public final int computeDistance2D(DistanceInterface aDistance,
				       DecalageInterface aDecalage) {
	if ( numberOfChildren() != 0 ) {
	    return getChild(0).computeDistance2D(aDistance,aDecalage);
	}
	else {
	    return 0;
	}
    }
  
    public final int computeDecalage2D(GlyphInterface aGlyph,
				       DecalageInterface aDecalage) {
	if ( numberOfChildren() != 0 ) {
	    return getChild(0).computeDecalage2D(aGlyph,aDecalage);
	}
	else {
	    return 0;
	}
    }
  
    public final int computeInitialDecalage2D(DecalageInterface aDecalage) {
	if ( numberOfChildren() != 0 ) {
	    return getChild(0).computeInitialDecalage2D(aDecalage);
	}
	else {
	    return 0;
	}
    }
  
    public final void tryAlternative(AlternativeInterface anAlternative) {
	if ( numberOfChildren() != 0 ) {
	    getChild(0).tryAlternative(anAlternative);
	}
    }
  
    public final void tryAlternativeForInitial(AlternativeInterface anAlternative) {
	if ( numberOfChildren() != 0 ) {
	    getChild(0).tryAlternativeForInitial(anAlternative);
	}
    }
  
    public final void tryAlternativeForDistance(DistanceInterface aDistance,
						AlternativeInterface anAlternative) {
	if ( numberOfChildren() != 0 ) {
	    getChild(0).tryAlternativeForDistance(aDistance,anAlternative);
	}
    }
  
    public final void tryAlternativeForDecalage(DecalageInterface aDecalage,
						AlternativeInterface anAlternative) {
	if ( numberOfChildren() != 0 ) {
	    getChild(0).tryAlternativeForDecalage(aDecalage,anAlternative);
	}
    }
  
    public boolean isActionShrinkComplient() {
	if ( numberOfChildren() != 0 ) {
	    return getChild(0).isActionShrinkComplient();
	}
	else {
	    return false;
	}
    }

    public void executeActionShrink() throws StopFormattingSonsException {
	if ( numberOfChildren() != 0 ) {
	    getChild(0).executeActionShrink();
	}
	else {
	    reportError(new ShouldNotBeCalledError());
	}
    }

    public boolean isActionShrinkSonComplient() {
	if ( numberOfChildren() != 0 ) {
	    return getChild(0).isActionShrinkSonComplient();
	}
	else {
	    return false;
	}
    }

    public void executeActionShrinkSon() {
	if ( numberOfChildren() != 0 ) {
	    getChild(0).executeActionShrinkSon();
	}
	else {
	    reportError(new ShouldNotBeCalledError());
	}
    }

    public final boolean draw2D(ContextInterface aContext,
				Transformer aTransformer,
				SelectionPathInterface aSelectionPath,
				Rectangle aRefreshZone,
				Rectangle aBoundingBox) {
	boolean theIsCleanFlag;
	if ( isClean() ) {
	    theIsCleanFlag = true;
	}
	else {
	    if ( isProceed() ) {
		theIsCleanFlag = false;
	    }
	    else {
		return false;
	    }
	}
	if ( numberOfChildren() != 0 ) {
	    GlyphInterface theSon = getChild(0);
	    FontInterface theFont = getFont();
	    ColorInterface theColor = getColor();
	    ColorInterface theBackground = getBackground();
	    _oldState = aContext.set(theFont,theColor,theBackground);
	    if ( theBackground != null ) {
		if ( theIsCleanFlag ) {
		    final Dimension theSize = getSizeValue();
		    aContext.fillRectangle( 
					   aTransformer.applyToX(getOriginX()) - theSize.getUpperLeftX(),
					   aTransformer.applyToY(getOriginY()) - theSize.getUpperLeftY(),
					   theSize.getWidth(), 
					   theSize.getHeight());
		}
		else {
		    final Dimension theSize = getSizeEstimation();
		    aContext.fillRectangle( 
					   aTransformer.applyToX(getOriginX()) - theSize.getUpperLeftX(),
					   aTransformer.applyToY(getOriginY()) - theSize.getUpperLeftY(),
					   theSize.getWidth(), 
					   theSize.getHeight());
		}
	    } 
	    else {
		aRefreshZone.translate(-getOriginX(),-getOriginY());
		ContextMementoInterface theMemento = draw2DSelectionsWhenCleanOrEnoughProceed(aContext,aTransformer,aSelectionPath,aRefreshZone);
		if ( theMemento != null ) {
		    aContext.disposeMemento(theMemento);
		}
		aRefreshZone.translate(getOriginX(),getOriginY());
		theIsCleanFlag = isClean();
	    }
	    SelectionPathInterface theDownSelection = downInSelection(aSelectionPath);
	    if ( theIsCleanFlag ) {
		if ( theDownSelection != null ) {
		    theSon.draw2DClean(aContext,aTransformer,theDownSelection,aRefreshZone,aBoundingBox);
		}
		else {
		    theSon.draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
		}
	    }
	    else {
		if ( theDownSelection != null ) {
		    theSon.draw2D(aContext,aTransformer,theDownSelection,aRefreshZone,aBoundingBox);
		}
		else {
		    theSon.draw2D(aContext,aTransformer,aRefreshZone,aBoundingBox);
		}
	    }
	    aContext.setState(_oldState); 
	    aContext.disposeMemento(_oldState);
	}
	return theIsCleanFlag;
    }

    protected final void draw2DClean(ContextInterface aContext,
				     Transformer aTransformer,
				     SelectionPathInterface aSelectionPath,
				     Rectangle aRefreshZone,
				     Rectangle aBoundingBox) {
	if ( numberOfChildren() != 0 ) {
	    GlyphInterface theSon = getChild(0);
	    FontInterface theFont = getFont();
	    ColorInterface theColor = getColor();
	    ColorInterface theBackground = getBackground();
	    _oldState = aContext.set(theFont,theColor,theBackground);
	    if ( theBackground != null ) {
		final Dimension theSize = getSizeValue();
		aContext.fillRectangle( 
				       aTransformer.applyToX(getOriginX()) - theSize.getUpperLeftX(),
				       aTransformer.applyToY(getOriginY()) - theSize.getUpperLeftY(),
				       theSize.getWidth(), 
				       theSize.getHeight());
	    }
	    else {
		aRefreshZone.translate(-getOriginX(),-getOriginY());
		ContextMementoInterface theMemento = draw2DSelections(aContext,aTransformer,aSelectionPath);
		if ( theMemento != null ) {
		    aContext.disposeMemento(theMemento);
		}
		aRefreshZone.translate(getOriginX(),getOriginY());
	    }
	    SelectionPathInterface theDownSelection = downInSelection(aSelectionPath);
	    if ( theDownSelection != null ) {
		theSon.draw2DClean(aContext,aTransformer,theDownSelection,aRefreshZone,aBoundingBox);
	    }
	    else {
		theSon.draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
	    }
	    aContext.setState(_oldState); 
	    aContext.disposeMemento(_oldState);
	}
    }

    protected final boolean draw2D(ContextInterface aContext,
				   Transformer aTransformer,
				   Rectangle aRefreshZone,
				   Rectangle aBoundingBox) {
	final boolean theIsCleanFlag;
	if ( isClean() ) {
	    theIsCleanFlag = true;
	}
	else {
	    if ( isProceed() ) {
		theIsCleanFlag = false;
	    }
	    else {
		return false;
	    }
	}
	if ( numberOfChildren() != 0 ) {
	    GlyphInterface theSon = getChild(0);
	    FontInterface theFont = getFont();
	    ColorInterface theColor = getColor();
	    ColorInterface theBackground = getBackground();
	    _oldState = aContext.set(theFont,theColor,theBackground);
	    if ( theBackground != null ) {
		if ( theIsCleanFlag ) {
		    aContext.fillRectangle( 
					   aTransformer.applyToX(getOriginX()) - getSizeEstimation().getUpperLeftX(),
					   aTransformer.applyToY(getOriginY()) - getSizeEstimation().getUpperLeftY(),
					   getSizeEstimation().getWidth(), 
					   getSizeEstimation().getHeight()); ////////// =====> USE ESTIMATION FOR ALL NOT CLEAN
		}
		else {
		    Dimension theSize = getSizeEstimation();
		    aContext.fillRectangle( 
					   aTransformer.applyToX(getOriginX()) - theSize.getUpperLeftX(),
					   aTransformer.applyToY(getOriginY()) - theSize.getUpperLeftY(),
					   theSize.getWidth(), 
					   theSize.getHeight());	  
		}
	    }
	    if ( theIsCleanFlag ) { ////// BUG ??
		theSon.draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
	    }
	    else {
		theSon.draw2D(aContext,aTransformer,aRefreshZone,aBoundingBox);
	    }
	    aContext.setState(_oldState);
	    aContext.disposeMemento(_oldState);
	}
	return theIsCleanFlag;
    }

    protected final void draw2DClean(ContextInterface aContext,
				     Transformer aTransformer,
				     Rectangle aRefreshZone,
				     Rectangle aBoundingBox) {
	if ( numberOfChildren() != 0 ) {
	    GlyphInterface theSon = getChild(0);
	    FontInterface theFont = getFont();
	    ColorInterface theColor = getColor();
	    ColorInterface theBackground = getBackground();
	    _oldState = aContext.set(theFont,theColor,theBackground);
	    if ( theBackground != null ) {
		final Dimension theSize = getSizeValue();
		aContext.fillRectangle( 
				       aTransformer.applyToX(getOriginX()) - theSize.getUpperLeftX(),
				       aTransformer.applyToY(getOriginY()) - theSize.getUpperLeftY(),
				       theSize.getWidth(), 
				       theSize.getHeight());
	    }
	    theSon.draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
	    aContext.setState(_oldState);
	    aContext.disposeMemento(_oldState);
	}
    }

    protected final boolean sonModified(GlyphInterface aGlyph,
					SonModificationStatus aStatus,
					ContextInterface aContext) {
	return true;
    }

    public final void computeCuttingCosts(AlternativeInterface anAlternative,
					  Rectangle aRectangle,
					  Transformer aTransformer) {
	if ( numberOfChildren() != 0 ) {
	    getChild(0).computeCuttingCosts(anAlternative,aRectangle,aTransformer);
	}
    }

    protected String getName() {
	return "Context";
    }
  
    private ContextMementoInterface _oldState;
  
}

