//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.path.SelectionPathInterface;
import figue.path.PathInterface;
import figue.path.LinearPath;
import figue.resource.FontMetricInterface;
import figue.geometry.Rectangle;
import figue.geometry.Dimension;
import figue.geometry.Transformer;
import figue.format.DistanceInterface;
import figue.format.DecalageInterface;
import figue.format.AlternativeInterface;
import figue.format.Modification;
import figue.format.StrategyComputeSizeTill;
import figue.format.StrategyReadyToFormat;
import figue.format.StrategyImproveSizeEstimation;
import figue.format.StrategySetOrigin;
import figue.format.StrategySetSize;
import figue.format.StopFormattingSonsException;
import figue.internal.ContextInterface;
import figue.internal.ContextMementoInterface;
import figue.internal.PairGlyphPath;

import java.util.Hashtable;

/**
 * [PENDING: update the comments] Combinator.
 * @level advanced
 */

public abstract class Combinator extends CompositeGlyph {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/Combinator.java,v $ $revision$ $Date: 1998/11/02 15:54:36 $ Copyright 1998 INRIA.";

    protected Combinator() {
	super();
	_currentContext = null;
	_currentImproveSizeEstimationStrategy = null;
	_currentSetOriginStrategy = null;
    }

    protected abstract void preFormat2D(ContextInterface aContext,
					FontMetricInterface aFontMetric);

    public final void update() {
	if ( getSizeValue() == null ) {
	    Dimension theSize = computeSize(getCurrentContext(),
					    getCurrentImproveSizeEstimationStrategy(),
					    getCurrentSetOriginStrategy(),
					    StrategyComputeSizeTill.THE_END);
	    setSize(theSize);
	    notifyEagerChange();
	}
    }
    
    public final void updateEager() {
	final Dimension theSize = computeSize(getCurrentContext(),
					getCurrentImproveSizeEstimationStrategy(),
					getCurrentSetOriginStrategy(),
					StrategyComputeSizeTill.THE_END);
	final Dimension theOldSize = getSizeValue();
	if ( theOldSize == null ) {
	    setSize(theSize);
	}
	else {
	    if ( ! theOldSize.equals(theSize) ) {
		setSize(theSize);
	    }
	}
	notifyEagerChange();
    }
  
    public final void notifyEagerChange() {
	GlyphInterface theFather = getParent();
	if ( theFather != null ) {
	    theFather.updateEager();
	}
	super.notifyEagerChange();
    }

    public void format2D(ContextInterface aContext,
			 FontMetricInterface aFontMetric,
			 AlternativeInterface anAlternative,
			 DistanceInterface aDistance,
			 StrategyReadyToFormat aReadyToFormatStrategy,
			 StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
			 
			 StrategySetOrigin aSetOriginStrategy,
			 StrategySetSize aSetSizeStrategy,
			 boolean aMustFormatFlag)
	throws StopFormattingSonsException {
	final boolean theMustFormatFlag = aMustFormatFlag || isNew();
	setCurrentContext(aContext);
	setCurrentImproveSizeEstimationStrategy(anImproveSizeEstimationStrategy);
	setCurrentSetOriginStrategy(aSetOriginStrategy);
	preFormat2D(aContext,aFontMetric);
	DistanceInterface theNewDistance = aDistance.lowerLevel(aContext,this);
	GlyphInterface theCurrentSon = null;
	int theRank = 0;
	try { /// CLEAN THIS CODE !!!!
	    if ( theMustFormatFlag ) {
		while ( theRank < numberOfChildren() ) {
		    theCurrentSon = getChild(theRank);
		    theNewDistance.checkBefore(anAlternative);
		    aReadyToFormatStrategy.doAlgorithm(aContext,theCurrentSon);
		    theCurrentSon.format2D(aContext,aFontMetric,anAlternative,theNewDistance,aReadyToFormatStrategy,anImproveSizeEstimationStrategy,aSetOriginStrategy,aSetSizeStrategy,true);
		    theNewDistance.newBrother(aContext,theCurrentSon);
		    theCurrentSon.chooseAlgorithm(anImproveSizeEstimationStrategy).doAlgorithm(aContext,this,theRank++);
		}
	    } else {
		if ( getModification().isSonModified() && 
		     ( getModification().getSonModifiedRank() != Modification.NO_RANK_SPECIFIED ) ) {
		    final int theCleanRank = getModification().getSonModifiedRank();
		    while ( theRank < theCleanRank ) {
			theCurrentSon = getChild(theRank);
			theNewDistance.newBrother(aContext,theCurrentSon);
			theRank++;
		    }
		    ////////////////////////////////////////
		    if ( ! getModification().isSonClean() ) {
			theCurrentSon = getChild(theRank);
			theNewDistance.checkBefore(anAlternative);
			aReadyToFormatStrategy.doAlgorithm(aContext,theCurrentSon);
			theCurrentSon.format2D(aContext,aFontMetric,anAlternative,theNewDistance,aReadyToFormatStrategy,anImproveSizeEstimationStrategy,aSetOriginStrategy,aSetSizeStrategy,aMustFormatFlag); /// true
			theNewDistance.newBrother(aContext,theCurrentSon);
			theCurrentSon.chooseAlgorithm(anImproveSizeEstimationStrategy).doAlgorithm(aContext,this,theRank++);
		    }
		    ////////////////////////////////////////
		    while ( theRank < numberOfChildren() ) {
			theCurrentSon = getChild(theRank);
			theNewDistance.checkBefore(anAlternative);
			aReadyToFormatStrategy.doAlgorithm(aContext,theCurrentSon);
			theCurrentSon.format2D(aContext,aFontMetric,anAlternative,theNewDistance,aReadyToFormatStrategy,anImproveSizeEstimationStrategy,aSetOriginStrategy,aSetSizeStrategy,true);
			theNewDistance.newBrother(aContext,theCurrentSon);
			theCurrentSon.chooseAlgorithm(anImproveSizeEstimationStrategy).doAlgorithm(aContext,this,theRank++);
		    }
		}
		else {
		    while ( theRank < numberOfChildren() ) {
			theCurrentSon = getChild(theRank);
			theNewDistance.checkBefore(anAlternative);
			aReadyToFormatStrategy.doAlgorithm(aContext,theCurrentSon);
			if ( ! theCurrentSon.isClean() ) {
			    theCurrentSon.format2D(aContext,aFontMetric,anAlternative,theNewDistance,aReadyToFormatStrategy,anImproveSizeEstimationStrategy,aSetOriginStrategy,aSetSizeStrategy,aMustFormatFlag);
			}
			theNewDistance.newBrother(aContext,theCurrentSon);
			theCurrentSon.chooseAlgorithm(anImproveSizeEstimationStrategy).doAlgorithm(aContext,this,theRank++);
		    }
		}
	    }
	}
	catch ( StopFormattingSonsException anException ) {
	    if ( anException.getTarget() != this) {
		throw anException;
	    }
	}
	aSetSizeStrategy.doAlgorithm(this,
				     computeSize(getCurrentContext(),
						 getCurrentImproveSizeEstimationStrategy(),
						 getCurrentSetOriginStrategy(),
						 StrategyComputeSizeTill.THE_END));
	theNewDistance.upperLevel(aContext);
    }

    public final void improveSizeEstimationAndUpdateParents(ContextInterface aContext,
							    int aRank) {
	if ( ! isProceed() ) {
	    setProceed();
	}
	StrategyComputeSizeTill.THE_RANK.setRank(aRank);
	setSizeEstimation(computeSize(aContext,
				      StrategyImproveSizeEstimation.INACTIVE,
				      StrategySetOrigin.NO_INCREMENTAL,
				      StrategyComputeSizeTill.THE_RANK));
	getParent().improveSizeEstimationAndUpdateParents(aContext,getParent().indexOf(this));
    }
				   
    public void tryAlternative(AlternativeInterface anAlternative) {
	/// NOT YET IMPLEMENTED FOR SUB CLASSES
    }
				   
    public void tryAlternativeForInitial(AlternativeInterface anAlternative) {
	/// NOT YET IMPLEMENTED FOR SUB CLASSES
    }

    public void tryAlternativeForDistance(DistanceInterface aDistance,
					  AlternativeInterface anAlternative) {
	aDistance.getDecalage().tryAlternative(anAlternative);
	aDistance.getDistance().tryAlternative(anAlternative);
    }
				   
    public void tryAlternativeForDecalage(DecalageInterface aDecalage,
					  AlternativeInterface anAlternative) {
	/// NOT YET IMPLEMENTED FOR SUB CLASSES
    }

    public boolean draw2D(ContextInterface aContext,
			  Transformer aTransformer,
			  SelectionPathInterface aSelectionPath,
			  Rectangle aRefreshZone,
			  Rectangle aBoundingBox) {
	ContextMementoInterface theMemento = null;
	aRefreshZone.translate(-getOriginX(),-getOriginY());
	theMemento = draw2DSelectionsWhenCleanOrEnoughProceed(aContext,
							      aTransformer,
							      aSelectionPath,
							      aRefreshZone);
	GlyphInterface theCurrentSon;
	SelectionPathInterface theSelectionSon = null;
	aTransformer.translate(getOriginX(),getOriginY()); 
	int theRank = 0;
	while ( theRank < numberOfChildren() ) {
	    theCurrentSon = getChild(theRank);
	    theSelectionSon = zoomInSelection(aSelectionPath,theRank);
	    if ( theSelectionSon != null ) {
		if ( ! theCurrentSon.draw2D(aContext,aTransformer,theSelectionSon,aRefreshZone,aBoundingBox) ) {
		    break;
		}
	    }
	    else {
		if ( ! theCurrentSon.draw2D(aContext,aTransformer,aRefreshZone,aBoundingBox) ) {
		    break;
		}
	    }
	    theRank++;
	}
	aTransformer.translate(-getOriginX(),-getOriginY());
	aRefreshZone.translate(getOriginX(),getOriginY());
	if ( theMemento != null ) {
	    aContext.setState(theMemento);
	    aContext.disposeMemento(theMemento);
	}
	return isClean(); /// +++ TEMPO TEMPO
    }

    protected void draw2DClean(ContextInterface aContext,
			       Transformer aTransformer,
			       SelectionPathInterface aSelectionPath,
			       Rectangle aRefreshZone,
			       Rectangle aBoundingBox) {
	ContextMementoInterface theMemento = null;
	aRefreshZone.translate(-getOriginX(),-getOriginY());
	theMemento = draw2DSelections(aContext,aTransformer,aSelectionPath);
	final int theSize = numberOfChildren();
	GlyphInterface theCurrentSon;
	SelectionPathInterface theSelectionSon = null;
	aTransformer.translate(getOriginX(),getOriginY()); 
	for ( int theRank = 0; theRank < theSize; theRank++ ) {
	    theCurrentSon = getChild(theRank);
	    theSelectionSon = zoomInSelection(aSelectionPath,theRank);
	    if ( theSelectionSon != null ) {
		theCurrentSon.draw2DClean(aContext,aTransformer,theSelectionSon,aRefreshZone,aBoundingBox);
	    }
	    else {
		theCurrentSon.draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
	    }
	}
	aTransformer.translate(-getOriginX(),-getOriginY());
	aRefreshZone.translate(getOriginX(),getOriginY());
	if ( theMemento != null ) {
	    aContext.setState(theMemento);
	    aContext.disposeMemento(theMemento);
	}
    }

    protected boolean draw2D(ContextInterface aContext,
			     Transformer aTransformer,
			     Rectangle aRefreshZone,
			     Rectangle aBoundingBox) {
	if ( isClean() ) {
	    draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
	    return true;
	}
	else {
	    if ( isProceed() ) {
		aRefreshZone.translate(-getOriginX(),-getOriginY());
		aTransformer.translate(getOriginX(),getOriginY()); 
		GlyphInterface theCurrentSon;
		int theRank = 0;
		while ( theRank < numberOfChildren() ) {
		    theCurrentSon = getChild(theRank);
		    if ( ! theCurrentSon.draw2D(aContext,aTransformer,aRefreshZone,aBoundingBox) ) {
			break;
		    }
		    theRank++;
		}
		aTransformer.translate(-getOriginX(),-getOriginY()); 
		aRefreshZone.translate(getOriginX(),getOriginY());
	    }
	    return false;
	}
    }

    protected void draw2DClean(ContextInterface aContext,
			       Transformer aTransformer,
			       Rectangle aRefreshZone,
			       Rectangle aBoundingBox) {
	aRefreshZone.translate(-getOriginX(),-getOriginY());
	final int theSize = numberOfChildren();
	aTransformer.translate(getOriginX(),getOriginY()); 
	GlyphInterface theCurrentSon;
	for ( int theRank = 0; theRank < theSize; theRank++ ) {
	    theCurrentSon = getChild(theRank);
	    theCurrentSon.draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
	}
	aTransformer.translate(-getOriginX(),-getOriginY()); 
	aRefreshZone.translate(getOriginX(),getOriginY());
    }

    protected final void doTranslate(Rectangle aRectangle) {
	aRectangle.translate(-getOriginX(),-getOriginY());
    }
				   
    protected final void undoTranslate(Rectangle aRectangle) {
	aRectangle.translate(getOriginX(),getOriginY());
    }

    public void doTranslatesZones(Rectangle aRefreshZone,
				  Rectangle aVisibleZone,
				  int aRank) {
	final int theX = - getOriginX();
	final int theY = - getOriginY();
	aRefreshZone.translate(theX,theY);
	aVisibleZone.translate(theX,theY);
    }
     public void undoTranslateZones(Rectangle aRefreshZone,
				    Rectangle aVisibleZone,
				    int aRank) {
	 final int theX = getOriginX();
	final int theY = getOriginY();
	aRefreshZone.translate(theX,theY);
	aVisibleZone.translate(theX,theY);
    }

    public PairGlyphPath getDeepestContains(int x, 
					    int y, 
					    Rectangle aBoundingBox,
					    ContextInterface aContext) { 
	computeBoundingBox(aBoundingBox);
	if ( ! aBoundingBox.contains(x,y) ) {
	    return null;
	}
	final int theX = x - getOriginX();
	final int theY = y - getOriginY();
	GlyphInterface theCurrentSon;
	PairGlyphPath theResult;
	PathInterface theExtendedPath;
	PairGlyphPath thePreviousResult = null;
	final int theSize = numberOfChildren();
	for ( int theRank = 0; theRank < theSize; theRank++ ) {
	    theCurrentSon = getChild(theRank);
	    theResult 
		= theCurrentSon.getDeepestContains(theX,theY,aBoundingBox,aContext);
	    if ( theResult != null ) {
		theExtendedPath = new LinearPath(theRank);
		if ( theResult.getPath() != null ) {
		    theExtendedPath.addChild(theResult.getPath());
		}
		theResult.setPath(theExtendedPath);
		if ( theResult.getGlyph().isFinalTarget() ) {
		    return theResult;
		}
		else {
		    if ( thePreviousResult != null ) {
			thePreviousResult.dispose(aContext);
		    }
		    thePreviousResult = theResult;
		}
	    }
	}
	if ( thePreviousResult != null ) {
	    return thePreviousResult;
	}
	return aContext.newPairGlyphPath(this);
    }

    protected final ContextInterface getCurrentContext() {
	return _currentContext;
    }
				   
    protected final void setCurrentContext(ContextInterface aContext) {
	_currentContext = aContext;
    }

    protected final StrategyImproveSizeEstimation getCurrentImproveSizeEstimationStrategy() {
	return _currentImproveSizeEstimationStrategy;
    }
				   
    protected final void setCurrentImproveSizeEstimationStrategy(StrategyImproveSizeEstimation aStrategy) {
	_currentImproveSizeEstimationStrategy = aStrategy;
    }
				   
    protected final StrategySetOrigin getCurrentSetOriginStrategy() {
	return _currentSetOriginStrategy;
    }

    protected final void setCurrentSetOriginStrategy(StrategySetOrigin aStrategy) {
	_currentSetOriginStrategy = aStrategy;
    }

    public void computeCuttingCosts(AlternativeInterface anAlternative,
				    Rectangle aRectangle,
				    Transformer aTransformer) {
	aTransformer.translate(getOriginX(),getOriginY());
	int theRank = 0;
	final int theSize = numberOfChildren();
	while ( theRank < theSize ) {
	    getChild(theRank++).computeCuttingCosts(anAlternative,aRectangle,aTransformer);
	}
	aTransformer.translate(-getOriginX(),-getOriginY());
    }

    private ContextInterface _currentContext;
    private StrategyImproveSizeEstimation _currentImproveSizeEstimationStrategy;
    private StrategySetOrigin _currentSetOriginStrategy;

}

