//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.path.PathInterface;
import figue.path.LinearPath;
import figue.geometry.Dimension;
import figue.format.ModificationInterface;
import figue.internal.ContextInterface;

import java.io.IOException;
import java.io.Writer;

/**
 * [PENDING: update the comments] CompositeGlyph.
 * @level advanced
 */

public abstract class CompositeGlyph extends Glyph {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/CompositeGlyph.java,v $ $revision$ $Date: 1998/11/02 15:54:36 $ Copyright 1998 INRIA.";

    protected CompositeGlyph() {
	super();
	_size = null;
    }

    public final Dimension getSizeValue() {
	return _size;
    }

    public final void setSize(Dimension aSize) {
	_size = aSize;
    }
    
    public final void resetSize(Dimension aSize, ContextInterface aContext) {
	if ( ( _size != null ) && ( ! _size.sameShape(aSize) ) ) {
	    ModificationInterface theModification = getModification();
	    if ( ! theModification.isModified() ) {
		theModification = aContext.newModification();
		setModification(theModification);
	    }
	    theModification.setSize(getSizeValue());
	}
	_size = aSize;
    }

    public void insertChild(int aRank, GlyphInterface aGlyph) {
	aGlyph.setParent(this);
    }
  
    public void replaceChild(int aRank, GlyphInterface aGlyph) {
	getChild(aRank).clearParent();
	aGlyph.setParent(this);
    }
  
    public void removeChild(int aRank) {
	getChild(aRank).clearParent();
    }

    public int indexOf(GlyphInterface aGlyph) {
	final int theSize = numberOfChildren();
	for( int theRank = 0; theRank < theSize; theRank++ ) {
	    if ( getChild(theRank) == aGlyph ) {
		return theRank;
	    }
	}
	return -1;
    }

    protected PathInterface computePathTo(GlyphInterface aGlyph,PathInterface aPath) {
	final PathInterface theResult = new LinearPath(indexOf(aGlyph));
	if ( aPath != null ) {
	    theResult.addChild(aPath);
	}
	return getParent().computePathTo(this,theResult);
    }
  
    public String toString() {
	final StringBuffer theResult = new StringBuffer(super.toString());
	final int theLength = numberOfChildren();
	if ( theLength != 0 ) {
	    theResult.append("{");
	    boolean theFirstFlag = true;
	    for( int theRank = 0; theRank < theLength; theRank++ ) {
		if ( theFirstFlag ) { 
		    theFirstFlag = false;
		} 
		else {
		    theResult.append(",");
		}
		theResult.append(theRank).append(":")
		    .append(getChild(theRank).toString());
	    }
	    theResult.append("}");
	}
	return theResult.toString();
    }

    protected boolean writeExtrinsicState(Writer anOutput) throws IOException {
	boolean theNonEmptyFlag = super.writeExtrinsicState(anOutput);
	final int theLength = numberOfChildren();
	if ( theLength != 0) {
	    if ( ! theNonEmptyFlag ) {
		anOutput.write('>');
	    }
	    theNonEmptyFlag = true;
	    for ( int theRank = 0; theRank < theLength; theRank++ ) {
		if ( theRank != 0 ) {
		    anOutput.write(' ');
		}
		getChild(theRank).writeState(anOutput);
	    }
	}
	return theNonEmptyFlag;
    }

    private Dimension _size;
    
}
