//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.path.PathInterface;
import figue.path.SelectionPathInterface;
import figue.resource.ColorOwnerInterface;
import figue.resource.ColorDefered;
import figue.resource.ColorInitializer;
import figue.resource.ColorInterface;
import figue.geometry.Rectangle;
import figue.geometry.Transformer;
import figue.internal.ContextInterface;

import java.io.Writer;
import java.io.IOException;

/**
 * [PENDING: update the comments] DebugRefreshZone.
 * @level internal
 */

public final class DebugRefreshZone extends DecoratedGlyph implements ColorOwnerInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/DebugRefreshZone.java,v $ $revision$ $Date: 1998/11/02 15:54:37 $ Copyright 1998 INRIA.";

    public DebugRefreshZone(GlyphInterface aGlyph,
			    String aHightlightColor) {
	super(aGlyph);
	_highlightColor = new ColorDefered(aHightlightColor,
					   new ColorInitializer(this));
    }

    public Object clone() throws CloneNotSupportedException {
	final DebugRefreshZone theResult = (DebugRefreshZone) super.clone();
	theResult._highlightColor = new ColorDefered(_highlightColor.getName(),
						     new ColorInitializer(theResult));
	return theResult;
    }

    public final PathInterface computePathTo(GlyphInterface aGlyph,PathInterface aPath) {
	return getParent().computePathTo(this,aPath);
    }

    public final boolean draw2D(ContextInterface aContext,
				Transformer aTransformer,
				SelectionPathInterface aSelectionPath,
				Rectangle aRefreshZone,
				Rectangle aBoundingBox) {
	final boolean theResult = super.draw2D(aContext,aTransformer,aSelectionPath,aRefreshZone,aBoundingBox);
	highlightZone(aContext,aTransformer,aRefreshZone,aBoundingBox);
	return theResult;
    }

    public final void draw2DClean(ContextInterface aContext,
				  Transformer aTransformer,
				  SelectionPathInterface aSelectionPath,
				  Rectangle aRefreshZone,
				  Rectangle aBoundingBox) {
	super.draw2DClean(aContext,aTransformer,aSelectionPath,aRefreshZone,aBoundingBox);
	highlightZone(aContext,aTransformer,aRefreshZone,aBoundingBox);
    }

    public final boolean draw2D(ContextInterface aContext,
				Transformer aTransformer,
				Rectangle aRefreshZone,
				Rectangle aBoundingBox) {
	final boolean theResult = super.draw2D(aContext,aTransformer,aRefreshZone,aBoundingBox);
	highlightZone(aContext,aTransformer,aRefreshZone,aBoundingBox);
	return theResult;
    }

    public final void draw2DClean(ContextInterface aContext,
				  Transformer aTransformer,
				  Rectangle aRefreshZone,
				  Rectangle aBoundingBox) {
	super.draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
	highlightZone(aContext,aTransformer,aRefreshZone,aBoundingBox);
    }

    public final void setColor(ColorInterface aColor) {
	_highlightColor = aColor;
    }
  
    private final void highlightZone(ContextInterface aContext,
				     Transformer aTransformer,
				     Rectangle aRefreshZone,
				     Rectangle aRectangle) {
	aRectangle.setBounds(aRefreshZone);
	aTransformer.applyTo(aRectangle);
	aContext.drawRectangle(aRectangle.getX(),
			       aRectangle.getY(),
			       aRectangle.getWidth() - 1,
			       aRectangle.getHeight() - 1,
			       _highlightColor);
    }

    public final String toString() {
	final StringBuffer theResult = new StringBuffer("DebugRefreshZone<");
	theResult.append(super.toString());
	theResult.append('>');
	return theResult.toString();
    }

    public String getName() {
	return null;
    }

    public final void writeState(Writer anOutput) throws IOException {
	if ( getComponent() != null ) {
	    getComponent().writeState(anOutput);
	}
    }
    
    private ColorInterface _highlightColor;
  
}
