//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.resource.FontMetricInterface;
import figue.geometry.Dimension;
import figue.geometry.Transformer;
import figue.internal.Globals;
import figue.internal.ContextInterface;

import java.util.Hashtable;

/**
 * [PENDING: update the comments] FlyWeightChar.
 * @level advanced
 */

public class FlyWeightChar extends FlyWeightAtom { /// TEMPO NOT FINAL FOR LINUX PATCH

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/FlyWeightChar.java,v $ $revision$ $Date: 1998/11/02 15:54:38 $ Copyright 1998 INRIA.";
  
    public FlyWeightChar(char aChar) {
	this(aChar,null);
    }
  
    protected FlyWeightChar(char aChar,FontMetricInterface aFontMetric) { /// TEMPO NOT PRIVATE FOR LINUX PATCH
	super(aFontMetric);
	_char = aChar;
	_bytes = null;
    }

    protected final void insertInRegistry(ContextInterface aContext) {
	_bytes = (new Character(_char)).toString().getBytes();
	final Hashtable[] _registry = aContext.getFlyWeightRegistryForChar();
	Hashtable theFontRegistry = _registry[_char];
	if ( theFontRegistry == null ) {
	    theFontRegistry = new Hashtable(Globals.AVERAGE_FLY_WEIGHT_ATOM_FONT_NUMBER,
					    Globals.FAST_ACCESS);
	    _registry[_char] = theFontRegistry;
	}
	theFontRegistry.put(getFontMetric().getFont().getKey(),this);
    }

    protected final void removeFromRegistry(ContextInterface aContext) {
	final Hashtable[] _registry = aContext.getFlyWeightRegistryForChar();
	Hashtable theFontRegistry = _registry[_char];
	theFontRegistry.remove(getFontMetric().getFont().getKey());
	if ( theFontRegistry.isEmpty() ) {
	    _registry[_char] = null;
	}
    }
  
    public final String getText() {
	return (new Character(_char)).toString();
    }

    public final int computeWidth(FontMetricInterface aFontMetric) {
	return aFontMetric.bytesWidth(_bytes);
    }

    protected final FlyWeightAtomInterface updateRegistry(ContextInterface aContext,
							  FontMetricInterface aFontMetric) {
	FlyWeightChar theFlyWeight = this;
	if ( theFlyWeight.getFontMetric() == null ) {
	    theFlyWeight.setFontMetric(aFontMetric);
	    final char theChar = theFlyWeight._char;
	    theFlyWeight = null;
	    final Hashtable[] _registry = aContext.getFlyWeightRegistryForChar();
	    Hashtable theFontRegistry = _registry[theChar];
	    if ( theFontRegistry != null ) {
		theFlyWeight = (FlyWeightChar)(theFontRegistry.get(aFontMetric.getFont().getKey()));
	    }
	    if ( theFlyWeight == null ) {
		// 	theFlyWeight = new FlyWeightChar(theChar,aFontMetric);
		theFlyWeight = Atom.PATCH.newFlyWeightChar(theChar,aFontMetric);/// TEMPO PATCH FOR LINUX
	    }
	    theFlyWeight.incrReferenceCounter(aContext);
	}
	else {
	    if ( ! theFlyWeight.getFontMetric().equals(aFontMetric) ) {
		final char theChar = theFlyWeight._char;
		theFlyWeight.decrReferenceCounter(aContext);
		theFlyWeight = null;
		final Hashtable[] _registry = aContext.getFlyWeightRegistryForChar();
		Hashtable theFontRegistry = (Hashtable)_registry[theChar];
		if ( theFontRegistry != null ) {
		    theFlyWeight = (FlyWeightChar)(theFontRegistry.get(aFontMetric.getFont().getKey()));
		}
		if ( theFlyWeight == null ) {
		    // 	  theFlyWeight = new FlyWeightChar(theChar,aFontMetric);
		    theFlyWeight = Atom.PATCH.newFlyWeightChar(theChar,aFontMetric);/// TEMPO PATCH FOR LINUX
		}
		theFlyWeight.incrReferenceCounter(aContext);
	    }
	    else {
		theFlyWeight.incrCounter();
	    }
	}
	return theFlyWeight;
    }
  
    public void draw2DClean(GlyphInterface anOwner, /// TEMPO NOT FINAL FOR LINUX PATCH
			    ContextInterface aContext,
			    Transformer aTransformer) {
	final int theOriginX = anOwner.getOriginX();
	final int theOriginY = anOwner.getOriginY();
	if ( aContext.getFont().isUnderlined() ) {
	    final Dimension theSize = anOwner.getSizeValue();
	    final int theY = aTransformer.applyToY(theOriginY) + Globals.PIXEL_OFFSET_FOR_UNDERLINE;
	    aContext.drawLine(aTransformer.applyToX(theOriginX) + theSize.getUpperLeftX(),
			      theY,
			      aTransformer.applyToX(theOriginX) + theSize.getBottomRightX(),
			      theY);
	}
	aContext.drawBytes(_bytes,
			   aTransformer.applyToX(theOriginX),
			   aTransformer.applyToY(theOriginY));
    }

    char _char; /// TEMPO NOT PRIVATE FOR LINUX PATCH
    private byte[] _bytes;

}

