//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.path.PathInterface;
import figue.path.SelectionPathInterface;
import figue.selection.HighlighterInterface;
import figue.selection.SelectionInterface;
import figue.selection.DefaultHighlighter;
import figue.geometry.Point;
import figue.geometry.Rectangle;
import figue.geometry.Dimension;
import figue.geometry.Size;
import figue.geometry.Transformer;
import figue.format.ModificationInterface;
import figue.format.SonModificationStatus;
import figue.format.StrategyImproveSizeEstimation;
import figue.format.StopFormattingSonsException;
import figue.format.FormatStatusInterface;
import figue.format.FormatStatusDirty;
import figue.format.FormatStatusClean;
import figue.format.FormatStatusNew;
import figue.format.FormatStatusProceed;
import figue.format.FormatStatusUnstable;
import figue.format.NoModification;
import figue.format.ComputedValueInterface;
import figue.format.ActionNewLine;
import figue.format.ModificationStatus;
import figue.error.DefaultErrorHandler;
import figue.error.ErrorHandlerInterface;
import figue.error.Warning;
import figue.error.Error;
import figue.error.ShouldNotBeCalledError;
import figue.internal.ContextInterface;
import figue.internal.ContextMementoInterface;
import figue.internal.BooleanWrapper;
import figue.internal.Direction;
import figue.FacadeInterface;
import figue.InteractiveFacade;
import figue.internal.Cursor;

import java.util.Vector;
import java.io.Writer;
import java.io.IOException;

import java.awt.event.MouseEvent;

/**
 * [PENDING: update the comments] FlyWeightChar.
 * @level advanced
 */

public abstract class Glyph extends GlyphInterface implements ErrorHandlerInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/Glyph.java,v $ $revision$ $Date: 1998/11/02 15:54:39 $ Copyright 1998 INRIA.";
  
    protected Glyph() {
	super();
	_parent = null;
	_originX = 0;
	_originY = 0;
	_formatStatus = FormatStatusDirty.INSTANCE;
	_modification = NoModification.INSTANCE;
	_dependent = null;
    }

    public Object clone() throws CloneNotSupportedException {
 	final Glyph theResult = (Glyph) super.clone();
	theResult._parent = _parent;
	theResult._formatStatus = FormatStatusDirty.INSTANCE;
	theResult._modification = NoModification.INSTANCE;
 	return theResult;
    }

    public final int getOriginX() {
	return _originX;
    }
    
    public final int getOriginY() {
	return _originY;
    }

    public void setOrigin(int x, int y) {
	_originX = x;
	_originY = y;
    }

    public void resetOrigin(int x, int y, ContextInterface aContext) {
	if ( ( _originX != x ) || ( _originY != y ) ) {
	    ModificationInterface theModification = getModification();
	    if ( ! theModification.isModified() ) {
		theModification = aContext.newModification();
		setModification(theModification);
	    }
	    theModification.setOrigin(getOriginX(),getOriginY());
	}
	_originX = x;
	_originY = y;
    }

    protected final Dimension getSize() {
	update();
	return getSizeValue();
    }
  
    protected final Dimension getEagerSize() {
	updateEager();
	return getSizeValue();
    }

    protected void computeBoundingBox(Rectangle aRectangle,
				      Transformer aTransformer) {
	aRectangle.setBounds(getSizeValue());
	aRectangle.translate(getOriginX(),getOriginY());
	aTransformer.applyTo(aRectangle);
    }

    public void computeBoundingBox(Rectangle aRectangle) {
	aRectangle.setBounds(getSizeValue());
	aRectangle.translate(getOriginX(),getOriginY());
    }

    public void computeOldBoundingBox(Rectangle aRectangle,
				      int anOldX,
				      int anOldY,
				      Dimension anOldSize) {
	aRectangle.setBounds(anOldSize);
	aRectangle.translate(anOldX,anOldY);
    }

    protected void computeEstimatedBoundingBox(Rectangle aRectangle,
					       Transformer aTransformer) {
	aRectangle.setBounds(getSizeEstimation());
	aRectangle.translate(getOriginX(),getOriginY());
	aTransformer.applyTo(aRectangle);
    }

    protected void computeEstimatedBoundingBox(Rectangle aRectangle) {
	aRectangle.setBounds(getSizeEstimation());
	aRectangle.translate(getOriginX(),getOriginY());
    }
    
    public final GlyphInterface getParent() { 
	return _parent; 
    }
  
    protected final void setParent(GlyphInterface aGlyph) {
	_parent = aGlyph; 
    }
    
    protected final void clearParent() {
	_parent = null; 
    }
    
    public GlyphInterface getChild(int aRank) {
	reportError(new GlyphNotCompositeError(this));
	return null;
    }
    
    public final void addChild(GlyphInterface aGlyph) {
	insertChild(numberOfChildren(),aGlyph);
    }
    
    public void insertChild(int aRank, GlyphInterface aGlyph) {
	reportError(new GlyphNotCompositeError(this));
    }
    
    public void replaceChild(int aRank, GlyphInterface aGlyph) {
	reportError(new GlyphNotCompositeError(this));
    }
    
    public final void removeChild() {
	removeChild(numberOfChildren() - 1);
    }
    
    public void removeChild(int aRank) {
	reportError(new GlyphNotCompositeError(this));
    }
    
    public int indexOf(GlyphInterface aGlyph) {
	reportError(new GlyphNotCompositeError(this));
	return -1;
    }

    protected PathInterface computePathTo(GlyphInterface aGlyph,PathInterface aPath) {
	reportError(new GlyphNotCompositeError(this));
	return null;
    }
    
    public int numberOfChildren() { 
	return 0; 
    }

    public void notifyChange() {
	if ( _dependent != null ) {
	    _dependent.update();
	}
    }
  
    public void notifyEagerChange() {
	if ( _dependent != null ) {
	    _dependent.updateEager();
	}
    }
  
    public void addDependent(ComputedValueInterface aValue) {
	_dependent =  aValue;
    }

    public final ComputedValueInterface getDependent() {
	return _dependent;
    }
  
    public final void removeDependent() {
	_dependent = null;
    }
  
    protected final Direction checkRefreshZone(Rectangle aRefreshZone,
					       Rectangle aBoundingBox) {
	computeBoundingBox(aBoundingBox);
	return checkZone(aRefreshZone,aBoundingBox);
    }

    protected final Direction checkRefreshZoneAgainstEstimation(Rectangle aRefreshZone,
								Rectangle aBoundingBox) {
	computeEstimatedBoundingBox(aBoundingBox);
	return checkZone(aRefreshZone,aBoundingBox);
    }

    private final Direction checkZone(Rectangle aRefreshZone,
				      Rectangle aBoundingBox) {
	if ( aBoundingBox.intersects(aRefreshZone) ) {
	    return Direction.CENTER;
	}
	if ( aBoundingBox.getY() < aRefreshZone.getY() ) {
	    return Direction.NORTH;
	}
	if ( aBoundingBox.getY() > ( aRefreshZone.getY() + 
				     aRefreshZone.getHeight() ) ) {
	    return Direction.SOUTH;
	}
	return Direction.EAST_WEST;
    }

    protected void refresh() {
	;
    }

    public HighlighterInterface getHighlighter(SelectionPathInterface aSelectionPath) {
	HighlighterInterface theHighlighter = null;
	if ( aSelectionPath != null ) {
	    SelectionInterface theSelection;
	    HighlighterInterface theCurrentHighlighter;
	    int thePriority = -1;
	    final int theSelectionSize = aSelectionPath.numberOfSelections();
	    for( int theRank = 0; theRank < theSelectionSize; theRank++ ) {
		theSelection = aSelectionPath.getSelection(theRank);
		if ( theSelection.isVisible() ) {
		    theCurrentHighlighter = theSelection.getHighlighter();
		    if ( ( theHighlighter == null ) ||
			 ( theSelection.getPriority() > thePriority ) ) {
			theHighlighter = theCurrentHighlighter;
			thePriority = theSelection.getPriority();
		    }
		}
	    }
	}
	if ( theHighlighter != null ) {
	    return theHighlighter;
	}
	else {
	    return DefaultHighlighter.INSTANCE;
	}
    }

    protected final ContextMementoInterface draw2DSelections(ContextInterface aContext,
							     Transformer aTransformer,
							     SelectionPathInterface aSelectionPath) {
	return getHighlighter(aSelectionPath).draw2DClean(this,aContext,aTransformer,aSelectionPath);
    }

    public final ContextMementoInterface draw2DSelectionsWhenCleanOrEnoughProceed(ContextInterface aContext,
										  Transformer aTransformer,
										  SelectionPathInterface aSelectionPath,
										  Rectangle aRefreshZone) {
	return getHighlighter(aSelectionPath).draw2D(this,aContext,aTransformer,aSelectionPath);
    }
    
    protected SelectionPathInterface zoomInSelection(SelectionPathInterface aSelectionPath,
						     int aRank) {
	final int theSize = aSelectionPath.numberOfChildren();
	SelectionPathInterface theSelectionSon;
	for ( int theRank = 0; theRank < theSize; theRank++ ) {
	    theSelectionSon = aSelectionPath.getChild(theRank);
	    if ( theSelectionSon.getRank() == aRank ) {
		return theSelectionSon;
	    }
	    else {
		if ( theSelectionSon.getRank() > aRank ) {
		    return null;
		}
	    }
	}
	return null;
    }

    public Cursor getCursor() {
	return Cursor.TEXT;
    }
  
    protected void setCursor(Cursor aCursor) {
	reportError(new ShouldNotBeCalledError());
    }

    public boolean redispatchEvent(InteractiveFacade aFacade,int aMouseEventType,MouseEvent anEvent) {
	return false;
    }

    protected boolean isFinalTarget() {
	return true;
    }

    public final boolean isClean() {
	return _formatStatus.isClean();
    }
    
    public final void setClean() {
	_formatStatus = FormatStatusClean.INSTANCE;
    }
    
    public final boolean isDirty() {
	return _formatStatus.isDirty();
    }

    public final void setDirty() {
	_formatStatus = FormatStatusDirty.INSTANCE;
    }

    public final boolean isNew() {
	return _formatStatus.isNew();
    }

    public final void setNew() {
	_formatStatus = FormatStatusNew.INSTANCE;
    }
    
    public final boolean isProceed() {
	return _formatStatus.isProceed();
    }
    
    public final void setProceed() {
	_formatStatus = new FormatStatusProceed();
    }
  
    public final boolean isUnstable() {
	return _formatStatus.isUnstable();
    }
    
    public final void setUnstable() {
	_formatStatus = FormatStatusUnstable.INSTANCE;
    }

    public final void markAllBranchDirty(SonModificationStatus aStatus,
					 ContextInterface aContext) {
	GlyphInterface theCurrentGlyph = this;
	GlyphInterface theSon;
	SonModificationStatus theStatus = aStatus;
	boolean theEndFlag = true;
	while ( theEndFlag ) {
	    if ( theCurrentGlyph.isClean() ) {
		theCurrentGlyph.setDirty();
		if ( ! theCurrentGlyph.getModification().isModified() ) {
		    theCurrentGlyph.setModification(aContext.newModification(ModificationStatus.SON));
		}
	    }
	    theSon = theCurrentGlyph;
	    theCurrentGlyph = theCurrentGlyph.getParent();
	    theEndFlag = ( ( theCurrentGlyph != null ) &&
			   theCurrentGlyph.sonModified(theSon,theStatus,aContext) );
	    if ( theStatus == SonModificationStatus.KILL ) {
		theStatus = SonModificationStatus.SHRINK;
	    }
	} 
    }

    public final void cleanModificationAndUpdateRefreshZone(Rectangle aVisibleZone,
							    Rectangle aRefreshZone,
							    BooleanWrapper aRefreshZoneNotInitializedFlag,
							    Rectangle aRectangle,
							    ContextInterface aContext) {
	if ( updateRefreshZoneAndIterateOnSons(aVisibleZone,aRefreshZone,aRefreshZoneNotInitializedFlag,aRectangle,aContext) ) {
	    final int theSize = numberOfChildren();
	    doTranslate(aVisibleZone);
	    doTranslate(aRefreshZone);
	    GlyphInterface theCurrentSon;
	    for( int theRank = 0; theRank < theSize; theRank++ ) {
		theCurrentSon = getChild(theRank);
		if ( theCurrentSon.getModification().isModified() ) {
		    theCurrentSon.cleanModificationAndUpdateRefreshZone(aVisibleZone,
									aRefreshZone,
									aRefreshZoneNotInitializedFlag,
									aRectangle,
									aContext);
		}
		else {
		    theCurrentSon.cleanModification(aContext);
		}
	    }
	    undoTranslate(aVisibleZone);
	    undoTranslate(aRefreshZone);
	}
	else {
	    cleanModification(aContext);
	}
    }

    private final boolean updateRefreshZoneAndIterateOnSons(Rectangle aVisibleZone,
							    Rectangle aRefreshZone,
							    BooleanWrapper aRefreshZoneNotInitializedFlag,
							    Rectangle aRectangle,
							    ContextInterface aContext) {
	boolean theIterateOnSonFlag = true;
	ModificationInterface theModification = getModification();
	int x;
	int y;
	if ( theModification.isOriginModified() ) {
	    theIterateOnSonFlag = false;
	    Point thePoint = theModification.getOrigin();
	    x = thePoint.getX();
	    y = thePoint.getY();
	}
	else {
	    x = getOriginX();
	    y = getOriginY();
	}
	Dimension theSize;
	if ( theModification.isSizeModified() ) {
	    theIterateOnSonFlag = false;
	    theSize = theModification.getSize();
	}
	else {
	    theSize = getSizeValue();
	}
	if ( ! theIterateOnSonFlag ) {
	    computeBoundingBox(aRectangle);
	    aRectangle.setBounds(aRectangle.intersection(aVisibleZone));
	    if ( aRefreshZoneNotInitializedFlag.isTrue() ) {
		aRefreshZoneNotInitializedFlag.setFalse();
		aRefreshZone.setBounds(aRectangle);
	    }
	    else {
		aRefreshZone.setBounds(aRefreshZone.union(aRectangle));
	    }   
	    computeOldBoundingBox(aRectangle,x,y,theSize);
	    aRectangle.setBounds(aRectangle.intersection(aVisibleZone));
	    aRefreshZone.setBounds(aRefreshZone.union(aRectangle));  
	}
	if ( ! theIterateOnSonFlag ) {
	    theModification.dispose(aContext);
	    setModification(NoModification.INSTANCE);
	}
	else {
	    if ( theModification.isSonModified() ) {
		theModification.dispose(aContext);
		setModification(NoModification.INSTANCE);
	    }
	}
	return theIterateOnSonFlag;
    }

    protected final void cleanModification(ContextInterface aContext) {
	getModification().dispose(aContext);
	setModification(NoModification.INSTANCE);
	final int theSize = numberOfChildren();
	GlyphInterface theCurrentSon;
	for( int theRank = 0; theRank < theSize; theRank++ ) {
	    theCurrentSon = getChild(theRank);
	    if ( theCurrentSon.getModification().isModified() ) {
		theCurrentSon.cleanModification(aContext);
	    }
	}
    }

    public final boolean improveEstimatedHeightAndCheckAwake(int anHeight) {
	if ( ! isProceed() ) {
	    setProceed();
	}
	return _formatStatus.improveEstimatedHeightAndCheckAwake(anHeight);
    }

    protected StrategyImproveSizeEstimation chooseAlgorithm(StrategyImproveSizeEstimation aStrategy) {
	return aStrategy;
    }

    protected final void setSizeEstimation(Dimension aCurrentSize) {
	_formatStatus.setEstimatedSize(aCurrentSize);
    }

    public final Dimension getSizeEstimation() {
	return _formatStatus.getEstimatedSize(this);
    }

    public final Size getCurrentSize() {
	return _formatStatus.getCurrentSize();
    }
    
    public FacadeInterface getFacade() {
	return getParent().getFacade();
    }

    public final void reportError(Error anError) {
	if ( canHandleError() ) {
	    handleError(anError);
	}
	else {
	    if ( hasNextErrorHandler() ) {
		getNextErrorHandler().reportError(anError);
	    }
	    else {
		if ( DefaultErrorHandler.DEFAULT != null ) {
		    DefaultErrorHandler.DEFAULT.reportError(anError);
		}
		else {
		    DefaultErrorHandler.INSTANCE.reportError(anError);
		}
	    }      
	}
    }
  
    public final void handleError(Error anError) {
	anError.managedBy(this);
    }
  
    public final void manageFatalError(Error anError) {
	throw new ShouldNotBeCalledError();
    }
  
    public final void manageWarning(Warning aWarning) {
	throw new ShouldNotBeCalledError();
    }
  
    public final boolean canHandleError() {
	return false;
    }
    
    public final boolean hasNextErrorHandler() {
	return ( getFacade() != null );
    }
  
    public final ErrorHandlerInterface getNextErrorHandler() {
	return getFacade();
    }

    public boolean isActionNoneComplient() {
	return false;
    }
  
    public boolean isActionNewLineComplient(ActionNewLine anAction) {
	return false;
    }

    public void executeActionNewLine(GlyphInterface aGlyph) 
	throws StopFormattingSonsException {
	reportError(new ShouldNotBeCalledError());
    }
  
    public boolean isActionShrinkComplient() {
	return false;
    }

    public void executeActionShrink() throws StopFormattingSonsException {
	reportError(new ShouldNotBeCalledError());
    }

    public boolean isActionShrinkSonComplient() {
	return false;
    }

    public void executeActionShrinkSon() {
	reportError(new ShouldNotBeCalledError());
    }
  
    protected final ModificationInterface getModification() {
	return _modification;
    }
  
    protected final void setModification(ModificationInterface aModification) {
	_modification = aModification;
    }

    public String toString() {
	final StringBuffer theResult = new StringBuffer();
	if ( isDirty() ) {
	    theResult.append("dirty;");
	}
	else {
	    if ( isProceed() ) {
		theResult.append("proceed;");
	    }
	}
	theResult.append(getOriginX()).append("/").append(getOriginY()).append(";");
	if ( getSizeValue() != null ) {
	    theResult.append(getSizeValue().toString()).append(";");
	}
	if ( getModification().isModified() ) {
	    theResult.append(_modification).append(";");
	}
	return theResult.toString();
    }

    public final void writeAsXML(Writer anOutput) throws IOException {
	anOutput.write("<?XML version=\"1.0\"?>");
	writeState(anOutput);
    }

    public void writeState(Writer anOutput) throws IOException {
	anOutput.write("<");
	anOutput.write(getName());
	writeIntrinsicState(anOutput);
	if ( writeExtrinsicState(anOutput) ) {
	    anOutput.write("</");
	    anOutput.write(getName());
	    anOutput.write(">");
	}
	else {
	    anOutput.write("/>");
	}
    }

    protected void writeIntrinsicState(Writer anOutput) throws IOException {
    }
    protected boolean writeExtrinsicState(Writer anOutput) throws IOException {
	return false;
    }
    
    private GlyphInterface _parent;
    private int _originX;
    private int _originY;
    private FormatStatusInterface _formatStatus;
    private ModificationInterface _modification;
    private ComputedValueInterface _dependent;
  
}

