//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.event.HyperlinkListenerInterface;
import figue.event.HyperlinkEvent;
import figue.path.SelectionPathInterface;
import figue.path.PathInterface;
import figue.resource.FontOwnerInterface;
import figue.resource.ColorOwnerInterface;
import figue.resource.ColorInterface;
import figue.resource.FontInterface;
import figue.resource.ColorDefered;
import figue.resource.FontDefered;
import figue.resource.ColorInitializer;
import figue.resource.ResourceInitializer;
import figue.resource.FontInitializer;
import figue.resource.ColorInitializerInterface;
import figue.resource.ResourceOwnerInterface;
import figue.geometry.Rectangle;
import figue.geometry.Dimension;
import figue.geometry.Size;
import figue.geometry.Transformer;
import figue.error.ShouldNotBeCalledError;
import figue.Constants;
import figue.internal.ContextInterface;
import figue.internal.PairGlyphPath;
import figue.internal.Cursor;
import figue.InteractiveFacade;

import java.awt.event.MouseEvent;
import java.io.Writer;
import java.io.IOException;
import figue.io.Utility;

/**
 * [PENDING: update the comments] Hyperlink.
 */

public final class Hyperlink extends DecoratedGlyphAbstract implements FontOwnerInterface, ColorOwnerInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/Hyperlink.java,v $ $revision$ $Date: 1998/11/02 15:54:41 $ Copyright 1998 INRIA.";

    public Hyperlink(Object anInfo,
		     HyperlinkHistoryInterface anHistory,
		     ColorInterface anActiveColor,
		     ColorInterface aVisitedColor,
		     ColorInterface aSelectedColor,
		     FontInterface anActiveFont) {
	super();
	_clickingFlag = false;
	if ( anActiveColor != null ) {
	    _activeColor = anActiveColor;
	}
	else {
	    _activeColor = new ColorDefered(Constants.ACTIVE_LINK_COLOR,
					    new ColorInitializer(this));
	}
	if ( _visitedColor != null ) {
	    _visitedColor = aVisitedColor;
	}
	else {
	    _visitedColor = new ColorDefered(Constants.VISITED_LINK_COLOR,
					     new VisitedColorInitializer(this));
	}
	if ( _selectedColor != null ) {
	    _selectedColor = aSelectedColor;
	}
	else {
	    _selectedColor = new ColorDefered(Constants.SELECTED_LINK_COLOR,
					      new SelectedColorInitializer(this));
	}
	if ( _activeFont != null ) {
	    _activeFont = anActiveFont;
	}
	else {
	    _activeFont = new FontDefered(Constants.ACTIVE_LINK_FONT,
					  new FontInitializer(this));
	}
	_component = new ChangeGraphicalContextAnonymous(_activeFont,_activeColor,null);
	_component.setParent(this);
	_component.addChild(new Horizontal());
	_cursor = Cursor.HAND;
	_info = anInfo;
	_history = anHistory; 
    }

    public Hyperlink(Object anInfo,
		     HyperlinkHistoryInterface anHistory,
		     String anActiveColor,
		     String aVisitedColor,
		     String aSelectedColor,
		     String anActiveFont) {
	super();
	_clickingFlag = false;
	if ( anActiveColor != null ) {
	    _activeColor = new ColorDefered(anActiveColor,new ColorInitializer(this));
	}
	else {
	    _activeColor = new ColorDefered(Constants.ACTIVE_LINK_COLOR,
					    new ColorInitializer(this));
	}
	if ( _visitedColor != null ) {
	    _visitedColor = new ColorDefered(aVisitedColor,new ColorInitializer(this));
	}
	else {
	    _visitedColor = new ColorDefered(Constants.VISITED_LINK_COLOR,
					     new VisitedColorInitializer(this));
	}
	if ( _selectedColor != null ) {
	    _selectedColor = new ColorDefered(aSelectedColor,new ColorInitializer(this));
	}
	else {
	    _selectedColor = new ColorDefered(Constants.SELECTED_LINK_COLOR,
					      new SelectedColorInitializer(this));
	}
	if ( _activeFont != null ) {
	    _activeFont = new FontDefered(anActiveFont,new FontInitializer(this));
	}
	else {
	    _activeFont = new FontDefered(Constants.ACTIVE_LINK_FONT,
					  new FontInitializer(this));
	}
	_component = new ChangeGraphicalContextAnonymous(_activeFont,_activeColor,null);
	_component.setParent(this);
	_component.addChild(new Horizontal());
	_cursor = Cursor.HAND;
	_info = anInfo;
	_history = anHistory; 
    }

    public Hyperlink(Object anInfo, HyperlinkHistoryInterface anHistory) {
	this(anInfo,anHistory,(ColorInterface)null,null,null,null);
    }

    public Object clone() throws CloneNotSupportedException {
 	final Hyperlink theResult = (Hyperlink) super.clone();
	theResult._component = (ChangeGraphicalContextAnonymous)_component.clone();
	theResult._activeColor = new ColorDefered(_activeColor.getName(),new ColorInitializer(theResult));
	theResult._visitedColor =  new ColorDefered(_visitedColor.getName(),new VisitedColorInitializer(theResult));
	theResult._selectedColor = new ColorDefered(_selectedColor.getName(),new SelectedColorInitializer(theResult));
	theResult._activeFont = new FontDefered(_activeFont.getName(),new FontInitializer(theResult));
	theResult._component.setParent(theResult);
 	return theResult;
    }

    public boolean draw2D(ContextInterface aContext,
			  Transformer aTransformer,
			  SelectionPathInterface aSelectionPath,
			  Rectangle aRefreshZone,
			  Rectangle aBoundingBox) {
	checkColors();
	return super.draw2D(aContext,aTransformer,aSelectionPath,aRefreshZone,aBoundingBox);
    }

    protected void draw2DClean(ContextInterface aContext,
			       Transformer aTransformer,
			       SelectionPathInterface aSelectionPath,
			       Rectangle aRefreshZone,
			       Rectangle aBoundingBox) {
	checkColors();
	super.draw2DClean(aContext,aTransformer,aSelectionPath,aRefreshZone,aBoundingBox);
    }

    protected boolean draw2D(ContextInterface aContext,
			     Transformer aTransformer,
			     Rectangle aRefreshZone,
			     Rectangle aBoundingBox) {
	checkColors();
	return super.draw2D(aContext,aTransformer,aRefreshZone,aBoundingBox);
    }

    protected void draw2DClean(ContextInterface aContext,
			       Transformer aTransformer,
			       Rectangle aRefreshZone,
			       Rectangle aBoundingBox) {
	checkColors();
	super.draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
    }

    private final void checkColors() {
	if ( _clickingFlag ) {
	    _component.setColor(_selectedColor);
	}
	else {
	    if ( isVisited() ) {
		_component.setColor(_visitedColor);
	    }
	    else {
		_component.setColor(_activeColor);
	    }
	}
    }
	   
    public final Object getInfo() {
	return _info;
    }

    protected final GlyphInterface getComponent() {
	return  _component;
    }

    protected final void setComponent(GlyphInterface aGlyph) {
	reportError(new ShouldNotBeCalledError());
    }

    public final boolean isVisited() {
	return _history.isVisited(this);
    }

    public final void setVisited() {
	if ( ! isVisited() ) {
	    _history.setVisited(this);
	    refresh();
	}
    }

    private final void setClicking(boolean aFlag) {
	if ( _clickingFlag != aFlag ) {
	    _clickingFlag = aFlag;
	    refresh();
	}
    }

    protected final void refresh() { /// MOVE TO TOP GLYPH
	GlyphInterface theGlyph = this;
	final Dimension theSize = getSizeValue();
	Rectangle _cacheForRefreshZone = new Rectangle();
	_cacheForRefreshZone.setBounds(getOriginX()- theSize.getUpperLeftX(),
				       getOriginY()- theSize.getUpperLeftY(),
				       theSize.getWidth(),
				       theSize.getHeight());
	while ( theGlyph != null ) {
	    theGlyph.undoTranslate(_cacheForRefreshZone);
	    theGlyph = theGlyph.getParent();
	}
	getFacade().refresh(_cacheForRefreshZone);
    }

    public final GlyphInterface getChild(int aRank) {
	return getComponent().getChild(0).getChild(aRank);
    }

    public final void insertChild(int aRank, GlyphInterface aGlyph) {
	getComponent().getChild(0).insertChild(aRank,aGlyph);
    }

    public final void replaceChild(int aRank, GlyphInterface aGlyph) {
	getComponent().getChild(0).replaceChild(aRank,aGlyph);
    }

    public final void removeChild(int aRank) {
	getComponent().getChild(0).removeChild(aRank);
    }
  
    public final int numberOfChildren() {
	return getComponent().getChild(0).numberOfChildren();
    }

    public final boolean redispatchEvent(InteractiveFacade aFacade,int aMouseEventType,MouseEvent anEvent) {
	final HyperlinkListenerInterface theListeners = aFacade.getHyperlinkListeners();
	switch ( aMouseEventType ) {
	case MouseEvent.MOUSE_CLICKED: {
	    setClicking(false);
	    if ( theListeners != null ) {
		theListeners.linkClicked(new HyperlinkEvent(aFacade,this));
	    }
	    break;
	}
	case MouseEvent.MOUSE_PRESSED: {
	    setClicking(true);
	    break;
	}
	case MouseEvent.MOUSE_RELEASED: {
	    setClicking(false);
	    break;
	}
	case MouseEvent.MOUSE_ENTERED: {
	    if ( theListeners != null ) {
		theListeners.linkEntered(new HyperlinkEvent(aFacade,this));
	    }
	    break;
	}
	case MouseEvent.MOUSE_EXITED: {
	    if ( theListeners != null ) {
		theListeners.linkExited(new HyperlinkEvent(aFacade,this));
	    }
	    break;
	}
	default: {
	    break;
	}
	}
	return true;
    }

    public final Cursor getCursor() {
	return _cursor;
    }

    public final PairGlyphPath getDeepestContains(int x,
						  int y, 
						  Rectangle aBoundingBox,
						  ContextInterface aContext) {
	PairGlyphPath theResult = super.getDeepestContains(x,y,aBoundingBox,aContext);
	if ( theResult != null ) {
	    final PathInterface thePath = theResult.getPath();
	    if ( thePath == null ) {
		theResult.setGlyph(this);
	    }
	    else {
		theResult.setGlyph(this);
		theResult.setPath(null);
	    }
	}
	return theResult;
    }

    public final void setColor(ColorInterface aColor) {
	_activeColor = aColor;
    }

    public final void setFont(FontInterface aFont) {
	_activeFont = aFont;
	_component.setFont(_activeFont);
    }

    public final void setVisitedColor(ColorInterface aColor) {
	_visitedColor = aColor;
	if ( ( ! _clickingFlag ) && isVisited() ) {
	    _component.setColor(_visitedColor);
	}
    }

    public final void setSelectedColor(ColorInterface aColor) {
	_selectedColor = aColor;
	if ( _clickingFlag ) {
	    _component.setColor(_selectedColor);
	}
    }

    public final String toString() {
	StringBuffer theResult = new StringBuffer("Hyperlink<");
	theResult.append(super.toString()).append(">");
	return theResult.toString();
    }

    protected String getName() {
	return "Hyperlink";
    }
    
    protected final void writeIntrinsicState(Writer anOutput) throws IOException {
	super.writeIntrinsicState(anOutput);
	anOutput.write(" Color=\"");
	Utility.writeStringAsXmlValue(anOutput,_activeColor.getName());
	anOutput.write("\" VisitedColor=\"");
	Utility.writeStringAsXmlValue(anOutput,_visitedColor.getName());
	anOutput.write("\" SelectedColor=\"");
	Utility.writeStringAsXmlValue(anOutput,_selectedColor.getName());
	anOutput.write("\" Font=\"");
	Utility.writeStringAsXmlValue(anOutput,_activeFont.getName());
	anOutput.write("\"");
    }

    private ChangeGraphicalContextAnonymous  _component;
    private boolean _clickingFlag;
    private ColorInterface _activeColor;
    private ColorInterface _visitedColor;
    private ColorInterface _selectedColor;
    private FontInterface _activeFont;
    private Cursor _cursor;
    private Object _info;
    private HyperlinkHistoryInterface _history;
  
}

abstract class HyperlinkInitializer extends ResourceInitializer implements ColorInitializerInterface {
  
    protected HyperlinkInitializer(Hyperlink anOwner) {
	super();
	_owner = anOwner;
    }

    public final ResourceOwnerInterface getOwner() {
	return _owner;
}
  
public final String getName() {
    return "HyperlinkInitializer";
}

protected Hyperlink _owner;
}

final class VisitedColorInitializer extends HyperlinkInitializer {
  
    protected VisitedColorInitializer(Hyperlink anOwner) {
	super(anOwner);
    }

    public final ColorInterface initResource(String aName) {
	ColorInterface theColor = newColor(_owner,aName);
	_owner.setVisitedColor(theColor);
	return theColor;
}
  
}

final class SelectedColorInitializer extends HyperlinkInitializer {
  
    protected SelectedColorInitializer(Hyperlink anOwner) {
	super(anOwner);
    }

    public final ColorInterface initResource(String aName) {
	ColorInterface theColor = newColor(_owner,aName);
	_owner.setSelectedColor(theColor);
	return theColor;
}
  
}

