//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.path.SelectionPathInterface;
import figue.path.PathInterface;
import figue.path.LinearPath;
import figue.path.TouchGlyph;
import figue.resource.FontMetricInterface;
import figue.geometry.Rectangle;
import figue.geometry.Dimension;
import figue.geometry.Transformer;
import figue.format.DistanceInterface;
import figue.format.DecalageInterface;
import figue.format.AlternativeInterface;
import figue.format.ModificationInterface;
import figue.format.SonModificationStatus;
import figue.format.StrategyReadyToFormat;
import figue.format.StrategyImproveSizeEstimation;
import figue.format.StrategySetOrigin;
import figue.format.StrategySetSize;
import figue.format.StrategyComputeSizeTill;
import figue.format.StopFormattingSonsException;
import figue.format.ActionNewLine;
import figue.error.ShouldNotBeCalledError;
import figue.internal.Globals;
import figue.internal.ContextInterface;
import figue.internal.ContextMementoInterface;
import figue.internal.PairGlyphPath;
import figue.internal.Cursor;
import figue.CommandInterface;
import figue.InteractiveFacade;

import java.awt.event.MouseEvent;
import java.io.IOException;
import java.io.Writer;

/**
 * [PENDING: update the comments] LightweightComponentAdapter.
 */

public final class LightweightComponentAdapter extends Glyph {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/LightweightComponentAdapter.java,v $ $revision$ $Date: 1998/11/02 15:54:42 $ Copyright 1998 INRIA.";
  
    public LightweightComponentAdapter(LightweightInterface aLightweight) {
	super();
	_lightweightComponent = aLightweight;
	/// REPORT ERROR IF BAD DOWN CAST
	_AWTComponent = (java.awt.Component)_lightweightComponent; /// BEURK
	_size = null;
	_cacheForRefreshZone = new Rectangle();
	aLightweight.setParent(this);
    }

    public final Object clone() throws CloneNotSupportedException {
	final LightweightComponentAdapter theResult = (LightweightComponentAdapter) super.clone();
	theResult._lightweightComponent = (LightweightInterface)_lightweightComponent.clone();
	theResult._AWTComponent = (java.awt.Component)theResult._lightweightComponent;
	theResult._cacheForRefreshZone = new Rectangle();
	theResult._lightweightComponent.setParent(theResult);
	return theResult;
    }

    public final Dimension getSizeValue() {
	return _size;
    }

    public final void setSize(Dimension aSize) {
	_size = aSize;
	_lightweightComponent.setSize(_size.getWidth(),_size.getHeight());
	_lightweightComponent.validate();
    }

    public final void resetSize(Dimension aSize, ContextInterface aContext) {
	if ( ( _size != null ) && ( ! _size.sameShape(aSize) ) ) {
	    ModificationInterface theModification = getModification();
	    if ( ! theModification.isModified() ) {
		theModification = aContext.newModification();
		setModification(theModification);
	    }
	    theModification.setSize(getSizeValue());
	}
	_size = aSize;
	_lightweightComponent.setSize(_size.getWidth(),_size.getHeight());
	_lightweightComponent.validate();
    }

    public final void format2D(ContextInterface aContext,
			       FontMetricInterface aFontMetric,
			       AlternativeInterface anAlternative,
			       DistanceInterface aDistance,
			       StrategyReadyToFormat aReadyToFormatStrategy,
			       StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
			       
			       StrategySetOrigin aSetOriginStrategy,
			       StrategySetSize aSetSizeStrategy,
			       boolean aMustFormatFlag)
	throws StopFormattingSonsException {
	if ( aContext.isComplientWith(this) ) {
	    aContext.applyFontToLightweightComponent(aFontMetric.getFont(),_lightweightComponent);
	    final java.awt.Dimension thePreferredSize = _lightweightComponent.getPreferredSize();
	    final float theAlignmentX = _lightweightComponent.getAlignmentX();
	    final float theAlignmentY = _lightweightComponent.getAlignmentY();
	    final int theUpperLeftY = Math.round(theAlignmentY * thePreferredSize.height);
	    final int theBottomRightY = thePreferredSize.height - theUpperLeftY;
	    final int theUpperLeftX = Math.round(theAlignmentX * thePreferredSize.width);
	    final int theBottomRightX = thePreferredSize.width - theUpperLeftX;
	    aSetSizeStrategy.doAlgorithm(this,new Dimension(theUpperLeftX,theUpperLeftY,theBottomRightX,theBottomRightY));
	    aDistance.checkAfter(this,anAlternative);
	}
	else {
	    aSetSizeStrategy.doAlgorithm(this,new Dimension());
	}
    }

    public final boolean draw2D(ContextInterface aContext,
				Transformer aTransformer,
				SelectionPathInterface aSelectionPath,
				Rectangle aRefreshZone,
				Rectangle aBoundingBox) {
	if ( isClean() ) {
	    draw2DClean(aContext,aTransformer,aSelectionPath,aRefreshZone,aBoundingBox);
	    return true;
	}
	else {
	    return false;
	}
    }

    protected final void draw2DClean(ContextInterface aContext,
				     Transformer aTransformer,
				     SelectionPathInterface aSelectionPath,
				     Rectangle aRefreshZone,
				     Rectangle aBoundingBox) {
	ContextMementoInterface theMemento = null;
	theMemento = draw2DSelections(aContext,aTransformer,aSelectionPath);
	final Dimension theSize = getSizeValue();
	drawLightweightComponent(aContext,
				 aTransformer.applyToX(getOriginX() - theSize.getUpperLeftX()),
				 aTransformer.applyToY(getOriginY() - theSize.getUpperLeftY()));
	if ( theMemento != null ) {
	    aContext.setState(theMemento);
	    aContext.disposeMemento(theMemento);
	}
    }

    protected final boolean draw2D(ContextInterface aContext,
				   Transformer aTransformer,
				   Rectangle aRefreshZone,
				   Rectangle aBoundingBox) {
	if ( isClean() ) {
	    draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
	    return true;
	}
	else {
	    return false;
	}
    }

    protected final void draw2DClean(ContextInterface aContext,
				     Transformer aTransformer,
				     Rectangle aRefreshZone,
				     Rectangle aBoundingBox) {
	final Dimension theSize = getSizeValue();
	drawLightweightComponent(aContext,
				 aTransformer.applyToX(getOriginX() - theSize.getUpperLeftX()),
				 aTransformer.applyToY(getOriginY() - theSize.getUpperLeftY()));
    }

    private final void drawLightweightComponent(ContextInterface aContext,
						int x, int y) {
	ContextMementoInterface theMemento = aContext.saveState();
	aContext.drawLightweightComponent(_lightweightComponent,x,y);
	if ( theMemento != null ) {
	    aContext.setState(theMemento);
	    aContext.disposeMemento(theMemento);
	}
    }

    public final void refresh() {
	GlyphInterface theGlyph = this;
	final Dimension theSize = getSizeValue();
	_cacheForRefreshZone.setBounds(0,0,theSize.getWidth(),theSize.getHeight());
	while ( theGlyph != null ) {
	    theGlyph.undoTranslate(_cacheForRefreshZone);
	    theGlyph = theGlyph.getParent();
	}
	final GlyphInterface theParent = getParent();
	if ( theParent != null ) {
	    theParent.getFacade().refresh(_cacheForRefreshZone);
	}
    }

    private final PathInterface internalComputePathTo() {
	return getParent().computePathTo(this,null);
    }

    public final void invalidate() {
	getFacade().executeWhenInactive(new CommandInterface() {
	    public boolean execute() {
		PathInterface thePath = new LinearPath();
		thePath.addChild(internalComputePathTo());
		PathInterface theCurrentPath = thePath;
		while ( theCurrentPath.numberOfChildren() != 0 ) {
		    theCurrentPath = theCurrentPath.getChild(0);
		}
		theCurrentPath.addOperation(TouchGlyph.INSTANCE);
		getFacade().updateAndRedisplay(thePath);
		return true;
	    } 
	    public String getName() {
		return "Invalidate";
	    }
	}
					);
    }

    public final boolean redispatchEvent(InteractiveFacade aFacade,int aMouseEventType,MouseEvent anEvent) {
	/// SHOULD TRANSLATE EVENT COORDINATES
	_lightweightComponent.dispatchEvent(new MouseEvent(_AWTComponent,aMouseEventType,anEvent.getWhen(),anEvent.getModifiers(),anEvent.getX(),anEvent.getY(),anEvent.getClickCount(),anEvent.isPopupTrigger()));
	return true;
    }
  
    public final PairGlyphPath getDeepestContains(int x, 
						  int y,
						  Rectangle aBoundingBox,
						  ContextInterface aContext) { 
	computeBoundingBox(aBoundingBox);
	if ( aBoundingBox.contains(x,y) ) {
	    return aContext.newPairGlyphPath(this);
	}
	return null;
    }

    public Cursor getCursor() {
	return Cursor.newCursor(_AWTComponent.getCursor());
    }
  
    protected void setCursor(Cursor aCursor) {
	_AWTComponent.setCursor(aCursor.getAwtCursor());
    }

    public final void update() {
	;
    }
    
    public final void updateEager() {
	;
    }

    protected Dimension computeSize(ContextInterface aContext,
				    StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
				  
				    StrategySetOrigin aSetOriginStrategy,
				    StrategyComputeSizeTill aComputeSizeTillStrategy) {
	reportError(new ShouldNotBeCalledError());
	return null;
    }

    public final int computeDecalage2D(GlyphInterface aGlyph,
				       DecalageInterface aDecalage) {
	return 0;
    }

    public final int computeInitialDecalage2D(DecalageInterface aDecalage) {
	return 0;
    }
  
    public final int computeDistance2D(DistanceInterface aDistance,
				       DecalageInterface aDecalage) {
	return 0;
    }

    public final void improveSizeEstimationAndUpdateParents(ContextInterface aContext,
							    int aRank) {
	reportError(new ShouldNotBeCalledError());
    }

    public final void tryAlternative(AlternativeInterface anAlternative) {
	return;
    }

    public final void tryAlternativeForInitial(AlternativeInterface anAlternative) {
	return;
    }
  
    public final void tryAlternativeForDistance(DistanceInterface aDistance,
						AlternativeInterface anAlternative) {
	return;
    }
  
    public final void tryAlternativeForDecalage(DecalageInterface aDecalage,
						AlternativeInterface anAlternative) {
	return;
    }

    protected final void doTranslate(Rectangle aRectangle) {
	final Dimension theSize = getSizeValue();
	aRectangle.translate(- getOriginX() + theSize.getUpperLeftX(),- getOriginY() + theSize.getUpperLeftY());
    }
  
    protected final void undoTranslate(Rectangle aRectangle) {
	final Dimension theSize = getSizeValue();
	aRectangle.translate(getOriginX() - theSize.getUpperLeftX(),getOriginY() - theSize.getUpperLeftY());
    }

    public final void doTranslatesZones(Rectangle aRefreshZone,
					Rectangle aVisibleZone,
					int aRank) {
	reportError(new ShouldNotBeCalledError());
    }
    public final void undoTranslateZones(Rectangle aRefreshZone,
					 Rectangle aVisibleZone,
					 int aRank) {
	reportError(new ShouldNotBeCalledError());
    }

    protected final boolean sonModified(GlyphInterface aGlyph,
					SonModificationStatus aStatus,
					ContextInterface aContext) {
	reportError(new ShouldNotBeCalledError());
	return true;
    }

    public final void computeCuttingCosts(AlternativeInterface anAlternative,
					  Rectangle aRectangle,
					  Transformer aTransformer) {
	computeBoundingBox(aRectangle,aTransformer);
	final int theStart = aRectangle.getY() + 1;
	final int theEnd = aRectangle.getY() + aRectangle.getHeight() - 1;
	anAlternative.preComputeActionCostNewLine(new ActionNewLine(theStart - 1,Globals.GOOD));
	anAlternative.preComputeActionCostNewLine(new ActionNewLine(theEnd,Globals.BEST));
	for ( int theRank = theStart; theRank < theEnd; theRank++ ) {
	    anAlternative.preComputeActionCostNewLine(new ActionNewLine(theRank,Globals.FORBIDDEN));
	}
    }

    public final String toString() {
	StringBuffer theResult = new StringBuffer("LightweightComponentAdapter<");
	theResult.append("`").append(_lightweightComponent).append("'")
	    .append(super.toString()).append(">");
	return theResult.toString();
    }

    public final void writeState(Writer anOutput) throws IOException {
    } /// TEMPO

    protected String getName() {
	return null;
    }

    private LightweightInterface _lightweightComponent;
    private java.awt.Component _AWTComponent; /// BEURK
    private Dimension _size;
    private Rectangle _cacheForRefreshZone;

}

