//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.box;

import figue.path.SelectionPathInterface;
import figue.path.PathInterface;
import figue.path.LinearPath;
import figue.resource.FontMetricInterface;
import figue.geometry.PositionToFather;
import figue.geometry.PositionToBrother;
import figue.geometry.Point;
import figue.geometry.Rectangle;
import figue.geometry.Dimension;
import figue.geometry.Size;
import figue.geometry.Transformer;
import figue.format.DistanceInterface;
import figue.format.DecalageInterface;
import figue.format.AlternativeInterface;
import figue.format.SonModificationStatus;
import figue.format.StrategyImproveSizeEstimation;
import figue.format.StrategySetOrigin;
import figue.format.StrategyComputeSizeTill;
import figue.error.ImplementationError;
import figue.internal.ContextInterface;
import figue.internal.ContextMementoInterface;
import figue.internal.Direction;
import figue.internal.PairGlyphPath;
import figue.MeasureInterface;
import figue.UnitInterface;
import figue.Measure;

import java.util.Vector;
import java.io.Writer;
import java.io.IOException;
import figue.io.Utility;

/**
 * [PENDING: update the comments] VerticalAbstract.
 * @level advanced
 */

public abstract class VerticalAbstract extends Combinator {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/box/VerticalAbstract.java,v $ $revision$ $Date: 1998/11/02 15:54:44 $ Copyright 1998 INRIA.";

    protected VerticalAbstract(MeasureInterface anIndentation,
			       MeasureInterface anInterline) {
	_interline = anInterline;
	_indent = anIndentation;
    }

    public Object clone() throws CloneNotSupportedException {
	final VerticalAbstract theResult = (VerticalAbstract) super.clone();
	theResult._interline = (MeasureInterface)_interline.clone();
	theResult._indent = (MeasureInterface)_indent.clone();
	return theResult;
    }

    public final void setInterline(MeasureInterface anInterline) {
	_interline = anInterline;
    }

    public final void setInterline(int anInterline) {
	setInterline(new Measure(anInterline,UnitInterface.BLANK_TENTH));
    }

    public final void setIndentation(MeasureInterface anIndentation) {
	_indent = anIndentation;
    }

    public final void setIndentation(int anIndentation) {
	setIndentation(new Measure(anIndentation,UnitInterface.BLANK_TENTH));
    }
  
    protected final void preFormat2D(ContextInterface aContext,
				     FontMetricInterface aFontMetric) {
	_indent.setFontMetric(aContext,aFontMetric);
	_interline.setFontMetric(aContext,aFontMetric);
    }

    protected final Dimension computeSize(ContextInterface aContext,
					  StrategyImproveSizeEstimation anImproveSizeEstimationStrategy,
					
					  StrategySetOrigin aSetOriginStrategy,
					  StrategyComputeSizeTill aComputeSizeTillStrategy) {
	if ( numberOfChildren() == 0 ) {
	    return new Size();
	}
	Point theFatherPoint = aContext.newPoint();
	Point thePoint = aContext.newPoint();
	Size theResult = new Size();
	GlyphInterface theCurrentSon = null;
	Dimension theCurrentSonSize = null;
	Dimension thePreviousSonSize = null;
	boolean theFirstFlag = true;
	boolean theMoreThanOneSonFlag = false;
	final int theIndentInPixel = _indent.getValueInPixel();
	int theDecalage = theIndentInPixel;
	PositionToFather thePositionToFather = aContext.newPositionToFather();
	PositionToBrother thePositionToBrother = null;
	int theExitX = 0;
	int theExitY = 0;
	int theRank = 0;
	final int theInterlineInPixel = _interline.getValueInPixel();
	while ( aComputeSizeTillStrategy.doAlgorithm(this,theRank) ) {
	    theCurrentSon = getChild(theRank);
	    theCurrentSonSize = aComputeSizeTillStrategy.getExactOrEstimatedSize(theCurrentSon,theRank);
	    if ( theFirstFlag ) {
		thePositionToFather.movePointToPosition(thePoint,theFatherPoint);
		thePositionToBrother = aContext.newPositionToBrother(thePositionToFather);
		theExitX = theCurrentSonSize.getUpperLeftX();
		theExitY = theCurrentSonSize.getBottomRightY() + theInterlineInPixel;
		theFirstFlag = false;
	    }
	    else {
		theMoreThanOneSonFlag = true;
		thePositionToBrother.setDx(theCurrentSonSize.getUpperLeftX() +
					   theDecalage - 
					   thePreviousSonSize.getUpperLeftX());
		thePositionToBrother.setDy(theCurrentSonSize.getUpperLeftY() +
					   theInterlineInPixel +
					   thePreviousSonSize.getBottomRightY());
		thePositionToBrother.movePointToPosition(thePoint);
		theDecalage = 0;
		theExitY += theCurrentSonSize.getUpperLeftY() +
		    theCurrentSonSize.getBottomRightY() + theInterlineInPixel;
	    }
	    aSetOriginStrategy.doAlgorithm(theCurrentSon,thePoint.getX(),thePoint.getY());
	    aComputeSizeTillStrategy.setFormatStatus(theCurrentSon,theRank);
	    theResult.growToIncludeSizeAt(theCurrentSonSize,thePoint);
	    thePreviousSonSize = theCurrentSonSize;
	    theRank++;
	}
	if ( theMoreThanOneSonFlag ) {
	    theResult.setExitPointX(theIndentInPixel +
				    theCurrentSonSize.getUpperLeftX() +
				    theCurrentSonSize.getExitPointX() -
				    theExitX);
	}
	else {
	    theResult.setExitPointX(theCurrentSonSize.getUpperLeftX() +
				    theCurrentSonSize.getExitPointX() -
				    theExitX);
	}
	theResult.setExitPointY(theCurrentSonSize.getExitPointY() +
				theExitY -
				theCurrentSonSize.getBottomRightY() -
				theInterlineInPixel);
	aContext.disposePoint(thePoint);
	aContext.disposePoint(theFatherPoint);
	aContext.disposePositionToFather(thePositionToFather);
	if ( thePositionToBrother != null ) {
	    aContext.disposePositionToBrother(thePositionToBrother);
	}
	return theResult;
    }

    public final int computeDecalage2D(GlyphInterface aGlyph,
				       DecalageInterface aDecalage) {
	return _indent.getValueInPixel();
    }

    public final int computeInitialDecalage2D(DecalageInterface aDecalage) {
	return 0;
    }
  
    public final int computeDistance2D(DistanceInterface aDistance,
				       DecalageInterface aDecalage) {
	if ( aDecalage.hasBrother() ) {
	    return ( aDistance.getValue() - _indent.getValueInPixel() );
	}
	else {
	    return aDistance.getValue();
	}
    }

    public void tryAlternativeForDistance(DistanceInterface aDistance,
					  AlternativeInterface anAlternative) {
	aDistance.getDistance().tryAlternative(anAlternative);
    }
  
    public void tryAlternativeForDecalage(DecalageInterface aDecalage,
					  AlternativeInterface anAlternative) {
    }

    public final boolean draw2D(ContextInterface aContext,
				Transformer aTransformer,
				SelectionPathInterface aSelectionPath,
				Rectangle aRefreshZone,
				Rectangle aBoundingBox) {
	if ( isClean() ) {
	    draw2DClean(aContext,aTransformer,aSelectionPath,aRefreshZone,aBoundingBox);
	    return true;
	}
	else {
	    if ( isProceed() ) {
		draw2DProceed(aContext,aTransformer,aSelectionPath,aRefreshZone,aBoundingBox);
	    }
	    return false;
	}
    }

    protected final void draw2DClean(ContextInterface aContext,
				     Transformer aTransformer,
				     SelectionPathInterface aSelectionPath,
				     Rectangle aRefreshZone,
				     Rectangle aBoundingBox) {
	int theMaxRank = numberOfChildren() - 1;
	if ( theMaxRank == -1 ) {
	    return;
	}
	int theMinRank = 0;
	int theRank;
	SelectionPathInterface theSelectionSon = null;
	ContextMementoInterface theMemento = null;
	aRefreshZone.translate(-getOriginX(),-getOriginY());
	theMemento = draw2DSelections(aContext,aTransformer,aSelectionPath); 
	final int theSize = numberOfChildren();
	GlyphInterface theCurrentSon;
	Direction theDirection;
	aTransformer.translate(getOriginX(),getOriginY()); 
	while ( true ) {
	    theRank = (theMaxRank + theMinRank)/ 2;
	    theCurrentSon = getChild(theRank);
	    theDirection = theCurrentSon.checkRefreshZone(aRefreshZone,
							  aBoundingBox);
	    if ( ( theDirection == Direction.CENTER ) ||
		 ( theDirection == Direction.EAST_WEST ) ) {
		theSelectionSon = zoomInSelection(aSelectionPath,theRank);
		if ( theSelectionSon == null ) {
		    theCurrentSon.draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
		}
		else {
		    theCurrentSon.draw2DClean(aContext,
					      aTransformer,
					      theSelectionSon,
					      aRefreshZone,
					      aBoundingBox);
		}
		for ( int i = theRank + 1; i < theSize; i++ ) {
		    theCurrentSon = getChild(i);
		    theDirection = theCurrentSon.checkRefreshZone(aRefreshZone,
								  aBoundingBox);
		    if ( ( theDirection == Direction.CENTER ) ||
			 ( theDirection == Direction.EAST_WEST ) ) {
			theSelectionSon = zoomInSelection(aSelectionPath,i);
			if ( theSelectionSon == null ) {
			    theCurrentSon.draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
			}
			else {
			    theCurrentSon.draw2DClean(aContext,
						      aTransformer,
						      theSelectionSon,
						      aRefreshZone,
						      aBoundingBox);
			}
		    }
		    else {
			break;
		    }
		}
		for ( int j = theRank - 1; j >= 0; j-- ) {
		    theCurrentSon = getChild(j);
		    theDirection = theCurrentSon.checkRefreshZone(aRefreshZone,
								  aBoundingBox);
		    if ( ( theDirection == Direction.CENTER ) ||
			 ( theDirection == Direction.EAST_WEST ) ) {
			theSelectionSon = zoomInSelection(aSelectionPath,j);
			if ( theSelectionSon == null ) {
			    theCurrentSon.draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
			}
			else {
			    theCurrentSon.draw2DClean(aContext,
						      aTransformer,
						      theSelectionSon,
						      aRefreshZone,
						      aBoundingBox);
			}
		    }
		    else {
			break;
		    }
		}
		break;
	    }
	    else {
		if ( theDirection == Direction.NORTH ) {
		    if ( theMinRank != theRank ) {
			theMinRank = theRank;
		    }
		    else {
			if ( theMinRank == theMaxRank ) {
			    break;
			}
			else {
			    theMinRank = theMaxRank;
			}
		    }
		}
		else {
		    if ( theDirection == Direction.SOUTH ) {
			if ( theMaxRank != theRank ) {
			    theMaxRank = theRank;
			}
			else {
			    if ( theMaxRank == theMinRank ) {
				break;
			    }
			    else {
				theMaxRank = theMinRank;
			    }
			}
		    }
		    else {
			throw new ImplementationError("Unexpected");
		    }
		}
	    }
	}
	aTransformer.translate(-getOriginX(),-getOriginY());
	aRefreshZone.translate(getOriginX(),getOriginY());
	if ( theMemento != null ) {
	    aContext.setState(theMemento);
	    aContext.disposeMemento(theMemento);
	}
    }

    private final void draw2DProceed(ContextInterface aContext,
				     Transformer aTransformer,
				     SelectionPathInterface aSelectionPath,
				     Rectangle aRefreshZone,
				     Rectangle aBoundingBox) {
	int theMaxRank = numberOfChildren() - 1;
	if ( theMaxRank == -1 ) {
 	    return;
	}
	int theMinRank = 0;
	int theRank;
	SelectionPathInterface theSelectionSon = null;
	ContextMementoInterface theMemento = null;
	aRefreshZone.translate(-getOriginX(),-getOriginY());
	theMemento = draw2DSelectionsWhenCleanOrEnoughProceed(aContext,
							      aTransformer,
							      aSelectionPath,
							      aRefreshZone); 
	GlyphInterface theCurrentSon;
	Direction theDirection;
	aTransformer.translate(getOriginX(),getOriginY()); 
	boolean theCurrentSonIsCleanFlag = false; 
				     main_loop: while ( true ) {
					 theRank = (theMaxRank + theMinRank)/ 2;
					 theCurrentSon = getChild(theRank);
					 while ( true ) { 
					     while ( true ) {
						 if ( theCurrentSon.isClean() ) {
						     theCurrentSonIsCleanFlag = true;
						     break;
						 }
						 else {
						     if ( theCurrentSon.isProceed() ) { 
							 theCurrentSonIsCleanFlag = false;
							 break; 
						     }
						     else {
							 if ( theRank == theMinRank ) {
							     break main_loop;
							 }
							 else {
							     theCurrentSon = getChild(--theRank);
							 }
						     }
						 }
					     }
					     theDirection 
						 = theCurrentSon.checkRefreshZoneAgainstEstimation(aRefreshZone, 
												   aBoundingBox);
					     if ( ( theDirection == Direction.CENTER ) ||
						  ( theDirection == Direction.EAST_WEST ) ) {
						 theSelectionSon = zoomInSelection(aSelectionPath,theRank);
						 if ( theSelectionSon == null ) {
						     if ( theCurrentSonIsCleanFlag ) {
							 theCurrentSon.draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
						     }
						     else {
							 theCurrentSon.draw2D(aContext,aTransformer,aRefreshZone,aBoundingBox);
						     }
						 }
						 else {
						     if ( theCurrentSonIsCleanFlag ) {
							 theCurrentSon.draw2DClean(aContext,
										   aTransformer,
										   theSelectionSon,
										   aRefreshZone,
										   aBoundingBox);
						     }
						     else {
							 theCurrentSon.draw2D(aContext,
									      aTransformer,
									      theSelectionSon,
									      aRefreshZone,
									      aBoundingBox);
						     }
						 }
						 for ( int j = theRank - 1; j >= 0; j-- ) {
						     theCurrentSon = getChild(j);
						     theDirection = theCurrentSon.checkRefreshZone(aRefreshZone,
												   aBoundingBox);
						     if ( ( theDirection == Direction.CENTER ) ||
							  ( theDirection == Direction.EAST_WEST ) ) {
							 theSelectionSon = zoomInSelection(aSelectionPath,j);
							 if ( theSelectionSon == null ) {
							     theCurrentSon.draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
							 }
							 else {
							     theCurrentSon.draw2DClean(aContext,
										       aTransformer,
										       theSelectionSon,
										       aRefreshZone,
										       aBoundingBox);
							 }
						     }
						     else {
							 break;
						     }
						 }
						 int i = theRank + 1;
						  inside_loop: while ( i < numberOfChildren() ) {
						      theCurrentSon = getChild(i);
						      if ( theCurrentSon.isClean() ) {
							  theCurrentSonIsCleanFlag = true;
						      }
						      else {
							  if ( theCurrentSon.isProceed() ) {
							      theCurrentSonIsCleanFlag = false;
							  }
							  else {
							      break main_loop;
							  }
						      }
						      while ( true ) { 
							  theDirection 
							      = theCurrentSon.checkRefreshZoneAgainstEstimation(aRefreshZone,
														aBoundingBox); 
							  if ( ( theDirection == Direction.CENTER ) ||
							       ( theDirection == Direction.EAST_WEST ) ) {
							      theSelectionSon = zoomInSelection(aSelectionPath,i);
							      if ( theSelectionSon == null ) {
								  theCurrentSon.draw2D(aContext,aTransformer,aRefreshZone,aBoundingBox);
							      }
							      else {
								  theCurrentSon.draw2D(aContext,
										       aTransformer,
										       theSelectionSon,
										       aRefreshZone,
										       aBoundingBox);
							      }
							      break; 
							  }
							  else {
							      if ( ! theCurrentSonIsCleanFlag ) { 
								  break main_loop;
							      } 
							      else {
								  break inside_loop; 
							      }
							  }
						      }
						      i++;
						  }
						 break main_loop;
					     }
					     else {
						 if ( theDirection == Direction.NORTH ) {
						     if ( theMinRank != theRank ) {
							 theMinRank = theRank;
							 continue main_loop; 
						     }
						     else {
							 if ( theMinRank == theMaxRank ) {
							     break main_loop;
							 }
							 else {
							     theMinRank = theMaxRank;
							     continue main_loop; 
							 }
						     }
						 }
						 else {
						     if ( theDirection == Direction.SOUTH ) {
							 if ( ! theCurrentSonIsCleanFlag ) { 
							     break main_loop;
							 } 
							 else {
							     if ( theMaxRank != theRank ) {
								 theMaxRank = theRank;
								 continue main_loop; 
							     }
							     else {
								 if ( theMaxRank == theMinRank ) {
								     break main_loop;
								 }
								 else {
								     theMaxRank = theMinRank;
								     continue main_loop; 
								 }
							     }
							 }
						     }
						     else {
							 throw new ImplementationError("Unexpected");
						     }
						 }
					     }
					 }
				     }
	aTransformer.translate(-getOriginX(),-getOriginY());
	aRefreshZone.translate(getOriginX(),getOriginY());
	if ( theMemento != null ) {
	    aContext.setState(theMemento);
	    aContext.disposeMemento(theMemento);
	}
    }

    protected final boolean draw2D(ContextInterface aContext,
				   Transformer aTransformer,
				   Rectangle aRefreshZone,
				   Rectangle aBoundingBox) {
	if ( isClean() ) {
	    draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
	    return true;
	}
	else {
	    if ( isProceed() ) {
		draw2DProceed(aContext,aTransformer,aRefreshZone,aBoundingBox);
	    }
	    return false;
	}
    }

    protected final void draw2DClean(ContextInterface aContext,
				     Transformer aTransformer,
				     Rectangle aRefreshZone,
				     Rectangle aBoundingBox) {
	final int theSize = numberOfChildren();
	if ( theSize == 0 ) {
	    return;
	}
	aRefreshZone.translate(-getOriginX(),-getOriginY());
	aTransformer.translate(getOriginX(),getOriginY()); 
	GlyphInterface theCurrentSon;
	Direction theDirection;
	int theMinRank = 0;
	int theMaxRank = numberOfChildren() - 1;
	int theRank;
	while ( true ) {
	    theRank = (theMaxRank + theMinRank)/ 2;
	    theCurrentSon = getChild(theRank);
	    theDirection = theCurrentSon.checkRefreshZone(aRefreshZone,
							  aBoundingBox);
	    if ( ( theDirection == Direction.CENTER ) ||
		 ( theDirection == Direction.EAST_WEST ) ) {
		theCurrentSon.draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
		for ( int i = theRank + 1; i < theSize; i++ ) {
		    theCurrentSon = getChild(i);
		    theDirection = theCurrentSon.checkRefreshZone(aRefreshZone,
								  aBoundingBox);
		    if ( ( theDirection == Direction.CENTER ) ||
			 ( theDirection == Direction.EAST_WEST ) ) {
			theCurrentSon.draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
		    }
		    else {
			break;
		    }
		}
		for ( int j = theRank - 1; j >= 0; j-- ) {
		    theCurrentSon = getChild(j);
		    theDirection = theCurrentSon.checkRefreshZone(aRefreshZone,
								  aBoundingBox);
		    if ( ( theDirection == Direction.CENTER ) ||
			 ( theDirection == Direction.EAST_WEST ) ) {
			theCurrentSon.draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
		    }
		    else {
			break;
		    }
		}
		break;
	    }
	    else {
		if ( theDirection == Direction.NORTH ) {
		    if ( theMinRank != theRank ) {
			theMinRank = theRank;
		    }
		    else {
			if ( theMinRank == theMaxRank ) {
			    break;
			}
			else {
			    theMinRank = theMaxRank;
			}
		    }
		}
		else {
		    if ( theDirection == Direction.SOUTH ) {
			if ( theMaxRank != theRank ) {
			    theMaxRank = theRank;
			}
			else {
			    if ( theMaxRank == theMinRank ) {
				break;
			    }
			    else {
				theMaxRank = theMinRank;
			    }
			}
		    }
		    else {
			throw new ImplementationError("Unexpected");
		    }
		}
	    }
	}
	aTransformer.translate(-getOriginX(),-getOriginY()); 
	aRefreshZone.translate(getOriginX(),getOriginY());
    }

    private final void draw2DProceed(ContextInterface aContext,
				     Transformer aTransformer,
				     Rectangle aRefreshZone,
				     Rectangle aBoundingBox) {
	int theMaxRank = numberOfChildren() - 1;
	if ( theMaxRank == -1 ) {
	    return;
	}
	int theMinRank = 0;
	int theRank;
	aRefreshZone.translate(-getOriginX(),-getOriginY());
	aTransformer.translate(getOriginX(),getOriginY()); 
	GlyphInterface theCurrentSon;
	Direction theDirection;
	boolean theCurrentSonIsCleanFlag = false; 
				     main_loop: while ( true ) {
					 theRank = (theMaxRank + theMinRank)/ 2;
					 theCurrentSon = getChild(theRank);
					 while ( true ) { 
					     while ( true ) {
						 if ( theCurrentSon.isClean() ) {
						     theCurrentSonIsCleanFlag = true; 
						     break;
						 }
						 else {
						     if ( theCurrentSon.isProceed() ) { 
							 theCurrentSonIsCleanFlag = false; 
							 break; 
						     }
						     else {
							 if ( theRank == theMinRank ) {
							     break main_loop;
							 }
							 else {
							     theCurrentSon = getChild(--theRank);
							 }
						     }
						 }
					     }
					     theDirection 
						 = theCurrentSon.checkRefreshZoneAgainstEstimation(aRefreshZone,
												   aBoundingBox); 
					     if ( ( theDirection == Direction.CENTER ) ||
						  ( theDirection == Direction.EAST_WEST ) ) {
						 if ( theCurrentSonIsCleanFlag ) {
						     theCurrentSon.draw2DClean(aContext,
									       aTransformer,
									       aRefreshZone,
									       aBoundingBox);
						 }
						 else {
						     theCurrentSon.draw2D(aContext,
									  aTransformer,
									  aRefreshZone,
									  aBoundingBox);
						 }
						 for ( int j = theRank - 1; j >= 0; j-- ) {
						     theCurrentSon = getChild(j);
						     theDirection = theCurrentSon.checkRefreshZone(aRefreshZone,
												   aBoundingBox);
						     if ( ( theDirection == Direction.CENTER ) ||
							  ( theDirection == Direction.EAST_WEST ) ) {
							 theCurrentSon.draw2DClean(aContext,aTransformer,aRefreshZone,aBoundingBox);
						     }
						     else {
							 break;
						     }
						 }
						 int i = theRank + 1;
						  inside_loop: while ( i < numberOfChildren() ) {
						      theCurrentSon = getChild(i);
						      if ( theCurrentSon.isClean() ) {
							  theCurrentSonIsCleanFlag = true;
						      }
						      else {
							  if ( theCurrentSon.isProceed() ) {
							      theCurrentSonIsCleanFlag = false;
							  }
							  else {
							      break main_loop;
							  }
						      }
						      while ( true ) { 
							  theDirection 
							      = theCurrentSon.checkRefreshZoneAgainstEstimation(aRefreshZone,
														aBoundingBox); 
							  if ( ( theDirection == Direction.CENTER ) ||
							       ( theDirection == Direction.EAST_WEST ) ) {
							      theCurrentSon.draw2D(aContext,aTransformer,aRefreshZone,aBoundingBox);
							      break; 
							  }
							  else {
							      if ( ! theCurrentSonIsCleanFlag) { 
								  break main_loop;
							      } 
							      else {
								  break inside_loop; 
							      }
							  }
						      }
						      i++;
						  }
						 break main_loop;
					     }
					     else {
						 if ( theDirection == Direction.NORTH ) {
						     if ( theMinRank != theRank ) {
							 theMinRank = theRank;
							 continue main_loop; 
						     }
						     else {
							 if ( theMinRank == theMaxRank ) {
							     break main_loop;
							 }
							 else {
							     theMinRank = theMaxRank;
							     continue main_loop; 
							 }
						     }
						 }
						 else {
						     if ( theDirection == Direction.SOUTH ) {
							 if ( ! theCurrentSonIsCleanFlag ) { 
							     break main_loop;
							 }
							 else {
							     if ( theMaxRank != theRank ) {
								 theMaxRank = theRank;
								 continue main_loop; 
							     }
							     else {
								 if ( theMaxRank == theMinRank ) {
								     break main_loop;
								 }
								 else {
								     theMaxRank = theMinRank;
								     continue main_loop; 
								 }
							     }
							 }
						     }
						     else {
							 throw new ImplementationError("Unexpected");
						     }
						 }
					     }
					 } 
				     }
	aTransformer.translate(-getOriginX(),-getOriginY()); 
	aRefreshZone.translate(getOriginX(),getOriginY());
    }

    public final PairGlyphPath getDeepestContains(int x, 
						  int y, 
						  Rectangle aBoundingBox,
						  ContextInterface aContext) { 
	computeBoundingBox(aBoundingBox);
	if ( ! aBoundingBox.contains(x,y) ) {
	    return null;
	}
	final int theX = x - getOriginX();
	final int theY = y - getOriginY();
	GlyphInterface theCurrentSon;
	int theMinRank = 0;
	int theMaxRank = numberOfChildren() - 1;
	int theRank;
	while ( true ) {
	    theRank = (theMaxRank + theMinRank)/ 2;
	    theCurrentSon = getChild(theRank);
	    theCurrentSon.computeBoundingBox(aBoundingBox);
	    if ( aBoundingBox.contains(theX,theY) ) {
		PairGlyphPath theResult 
		    = theCurrentSon.getDeepestContains(theX,theY,aBoundingBox,aContext);
		if ( theResult != null ) {
		    PathInterface theExtendedPath = new LinearPath(theRank);
		    if ( theResult.getPath() != null ) {
			theExtendedPath.addChild(theResult.getPath());
		    }
		    theResult.setPath(theExtendedPath);
		    return theResult;
		}
	    }
	    else {
		if ( theY <= aBoundingBox.getY() ) {
		    if ( theMaxRank != theRank ) {
			theMaxRank = theRank;
		    }
		    else {
			if ( theMaxRank == theMinRank ) {
			    break;
			}
			else {
			    theMaxRank = theMinRank;
			}
		    }
		}
		else {
		    if ( theY >= ( aBoundingBox.getY() + aBoundingBox.getHeight() ) ) {
			if ( theMinRank != theRank ) {
			    theMinRank = theRank;
			}
			else {
			    if ( theMaxRank == theMinRank ) {
				break;
			    }
			    else {
				theMinRank = theMaxRank;
			    }
			}
		    }
		    else {
			break;
		    }
		}
	    }
	}
	return aContext.newPairGlyphPath(this);
    }

    protected boolean isFinalTarget() {
	return false;
    }

    protected boolean sonModified(GlyphInterface aGlyph,
				  SonModificationStatus aStatus,
				  ContextInterface aContext) {
	return true;
    }

    public final String toString() {
	StringBuffer theResult = new StringBuffer("Verti<");
	theResult.append("interline:").append(_interline)
	    .append(";indent:").append(_indent)
	    .append(";children:").append(numberOfChildren())
	    .append(super.toString()).append(">");
	return theResult.toString();
    }

    protected String getName() {
	return "V";
    }

    protected final void writeIntrinsicState(Writer anOutput) throws IOException {
	super.writeIntrinsicState(anOutput);
	anOutput.write(" Interline=\"");
	anOutput.write(Integer.toString(_interline.getValue()));
	anOutput.write('"');
	Utility.writeUnitAsXml(anOutput,_interline.getUnit());
	anOutput.write(" Indent=\"");
	anOutput.write(Integer.toString(_indent.getValue()));
	anOutput.write('"');
	Utility.writeUnitAsXml(anOutput,_indent.getUnit());
    }

    private MeasureInterface _interline;
    private MeasureInterface _indent;
  
}
