//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.doclet;

import java.util.Hashtable;
import java.util.Enumeration;
import com.sun.javadoc.*;
import java.io.Writer;
import java.io.IOException;
import java.util.Stack;

public class ClassWriter extends HtmlWriter {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/doclet/ClassWriter.java,v $ $revision$ $Date: 1998/11/02 15:54:47 $ Copyright 1998 INRIA.";

    public ClassWriter(String aType,
		       ClassDoc aClass,
		       String anOutputDirectory,
		       String aTemplateDirectory,
		       Hashtable aKnownTypeList,
		       ConstructorDoc[] aConstructorList,
		       int aConstructorSize,
		       MethodDoc[] aMethodList,
		       int aMethodSize,
		       FieldDoc[] aFieldList,
		       int aFieldSize,
		       Hashtable aSubsumptionList,
		       Stack anInheritancePath) {
	super(anOutputDirectory,aTemplateDirectory,"class.html",aKnownTypeList);
	_type = aType;
	_class = aClass;
    	_fileName = aClass.qualifiedName() + ".html";
	_constructors = aConstructorList;
	_constructorSize = aConstructorSize;
	_methods = aMethodList;
	_methodSize = aMethodSize;
	_fields = aFieldList;
	_fieldSize = aFieldSize;
	_subsumers = aSubsumptionList;
	_inheritancePath = anInheritancePath;
    }

    protected void document() throws IOException {
	final Writer theStream = getOutputStream();
	copyTemplateHeader(theStream);
	final String thePackageName = _class.containingPackage().name();
	title(_type + " <A HREF=\"Package-" + thePackageName + ".html\">" + 
	      thePackageName + "</A>." + _class.name(),theStream);
	theStream.write("<TR><TD></TD><TD>\n");
	inheritance(theStream);
	summary(theStream);
	details(theStream);
	theStream.write("</TD></TR>\n");
	copyTemplateFooter(theStream);
    }

    private void summary(Writer aStream) throws IOException {
	if ( _constructorSize > 0 ) {
	    Utility.quickSort(_constructors,0,_constructorSize-1);
	    section(aStream,"Constructor Summary",null);
	    aStream.write("<UL>\n");
	    for ( int theRank = 0; theRank < _constructorSize; theRank++ ) {
		aStream.write("<LI>");
		scrollDownToConstructor(aStream,_constructors[theRank]);
		aStream.write('\n');
	    }
	    aStream.write("</UL>\n");
	    endSection(aStream);
	}
	if ( _methodSize > 0 ) {
	    Utility.quickSort(_methods,0,_methodSize-1);
	    section(aStream,"Method Summary",null);
	    aStream.write("<UL>\n");
	    for ( int theRank = 0; theRank < _methodSize; theRank++ ) {
		aStream.write("<LI>");
		scrollDownToMethod(aStream,_methods[theRank]);
		aStream.write('\n');
	    }
	    aStream.write("</UL>\n");
	    endSection(aStream);
	}
	if ( _fieldSize > 0 ) {
	    Utility.quickSort(_fields,0,_fieldSize-1);
	    section(aStream,"Field Summary",null);
	    aStream.write("<UL>\n");
	    for ( int theRank = 0; theRank < _fieldSize; theRank++ ) {
		aStream.write("<LI>");
		scrollDownToVariable(aStream,_fields[theRank]);
		aStream.write('\n');
	    }
	    aStream.write("</UL>\n");
	    endSection(aStream);
	}
    }

    private void details(Writer aStream) throws IOException {
	if ( _constructorSize > 0 ) {
	    aStream.write("<TR><TD></TD><TD>\n");
	    section(aStream,"Constructor Detail",null);
	    aStream.write("<UL>\n");
	    for ( int theRank = 0; theRank < _constructorSize; theRank++ ) {
		aStream.write("<A NAME=\"" + _constructors[theRank].name() + _constructors[theRank].signature() + "\">\n<LI>");
		detailConstructor(aStream,_constructors[theRank]);
		aStream.write('\n');
	    }
	    aStream.write("</UL>\n");
	    endSection(aStream);
	    aStream.write("</TD></TR>\n");
	}
	if ( _methodSize > 0 ) {
	    aStream.write("<TR><TD></TD><TD>\n");
	    section(aStream,"Method Detail",null);
	    aStream.write("<UL>\n");
	    for ( int theRank = 0; theRank < _methodSize; theRank++ ) {
		aStream.write("<A NAME=\"" + _methods[theRank].name() + _methods[theRank].signature() + "\">\n<LI>");
		detailMethod(aStream,_methods[theRank]);
		aStream.write('\n');
	    }
	    aStream.write("</UL>\n");
	    endSection(aStream);
	    aStream.write("</TD></TR>\n");
	}
	if ( _fieldSize > 0 ) {
	    aStream.write("<TR><TD></TD><TD>\n");
	    section(aStream,"Field Detail",null);
	    aStream.write("<UL>\n");
	    for ( int theRank = 0; theRank < _fieldSize; theRank++ ) {
		aStream.write("<A NAME=\"" + _fields[theRank].name() + "\">\n<LI>");
		detailVariable(aStream,_fields[theRank]);
		aStream.write('\n');
	    }
	    aStream.write("</UL>\n");
	    endSection(aStream);
	    aStream.write("</TD></TR>\n");
	}
    }

    private void detailConstructor(Writer aStream, ConstructorDoc aConstructor)  throws IOException {
	highlightName(aStream,aConstructor.name());
	aStream.write("<PRE>\n");
	int theOffset = modifiers(aStream,aConstructor);
	theOffset += aConstructor.name().length();
	if ( theOffset + largestParameter(aConstructor) > Constants.MAX_OFFSET ) {
	    theOffset = aConstructor.name().length();
	    aStream.write("<BR>");
	}
	aStream.write("<STRONG>" + aConstructor.name() + "</STRONG>");
	theOffset++;
	signature(theOffset,aStream,aConstructor);
	aStream.write("</PRE>\n");
	comments(aStream,aConstructor);
	boolean theTagFlag = parameterTags(aStream,aConstructor);
	theTagFlag = ( throwsTag(aStream,aConstructor) ||theTagFlag );
	theTagFlag = ( seeTags(aStream,aConstructor) || theTagFlag );

	if ( theTagFlag ) {
	    aStream.write("<BR>\n");
	}
    }

    private int largestParameter(ExecutableMemberDoc aMember) {
	int theResult = 0;
	Parameter[] theParameters = aMember.parameters();
	final int theSize = theParameters.length;
	Parameter theCurrentParameter;
	for ( int theRank = 0; theRank < theSize; theRank++ ) {
	    theCurrentParameter = theParameters[theRank];
	    theResult = Math.max(theResult,
				 theCurrentParameter.name().length() +
				 theCurrentParameter.type().typeName().length() + 1);
	}
	return (theResult + 2);
    }

    private void detailVariable(Writer aStream,FieldDoc aVariable) throws IOException {
	highlightName(aStream,aVariable.name());
	modifiers(aStream,aVariable);
	linkToType(aStream,aVariable.type());
	aStream.write(" <STRONG>" + aVariable.name() + "</STRONG>");
	comments(aStream,aVariable);
	
    }

    private void detailMethod(Writer aStream,MethodDoc aMethod) throws IOException {
	highlightName(aStream,aMethod.name());
	aStream.write("<PRE>\n");
	int theOffset = modifiers(aStream,aMethod);
	if ( aMethod.isSynchronized() ) {
	    aStream.write("synchronized ");
	    theOffset += "synchronized ".length();
	}
	theOffset += linkToType(aStream,aMethod.returnType());
	theOffset++;
	theOffset += aMethod.name().length();
	if ( ( theOffset + largestParameter(aMethod) ) > Constants.MAX_OFFSET ) {
	    theOffset = aMethod.name().length();
	    aStream.write("<BR>");
	}
	else {
	    aStream.write(' ');
	}
	aStream.write("<STRONG>" + aMethod.name() + "</STRONG>");
	theOffset++;
	signature(theOffset,aStream,aMethod);
	aStream.write("</PRE>\n");
	comments(aStream,aMethod);
	boolean theTagFlag = parameterTags(aStream,aMethod);
	theTagFlag = ( returnTag(aStream,aMethod) || theTagFlag );
	theTagFlag = ( throwsTag(aStream,aMethod) || theTagFlag );
	theTagFlag = ( seeTags(aStream,aMethod) || theTagFlag);
	if ( theTagFlag ) {
	    aStream.write("<BR>\n");
	}
    }

    private void inheritance(Writer aStream) throws IOException {
	Enumeration theSubsumers = _subsumers.keys();
	if ( theSubsumers.hasMoreElements() ) {
	    aStream.write("<DL><DT><STRONG>Subsumers: </STRONG><DD>");
	    boolean theFirstFlag = true;
	    while ( theSubsumers.hasMoreElements() ) {
		if ( theFirstFlag ) {
		    theFirstFlag = false;
		}
		else {
		    aStream.write(", ");
		}
		linkToType(aStream,(ClassDoc)(theSubsumers.nextElement()));
	    }
	    aStream.write("</DL>\n");
	}
	final ClassDoc[] theInterfaces = _class.interfaces();
	final int theInterfaceSize = theInterfaces.length;
	final String theName;
	final String theRelationShip;
	if ( _class.isInterface() ) {
	    theName = "interface";
	    theRelationShip = "extends";
	}
	else {
	    theName = "class";
	    theRelationShip = "implements";
	    ClassDoc theSuper = _class;
	    final Stack theInheritancePath = new Stack();
	    while ( theSuper != null ) {
		theInheritancePath.push(theSuper);
		theSuper = theSuper.superclass();
	    }
	    int theDepth = 0;
	    aStream.write("<PRE>\n");
	    ClassDoc theCurrentClass;
	    while ( ! theInheritancePath.empty() ) {
		if ( theDepth > 0 ) {
		    for ( int theRank = 0; theRank < theDepth; theRank++ ) {
			aStream.write("  ");
		    }
		    aStream.write("|\n");
		    for ( int theRank = 0; theRank < theDepth; theRank++ ) {
			aStream.write("  ");
		    }
		    aStream.write("+-");
		}
		theDepth++;
		theCurrentClass = (ClassDoc)(theInheritancePath.pop());
		if ( theCurrentClass == _class ) {
		    aStream.write("<STRONG>" + theCurrentClass.name() + "</STRONG>");
		}
		else {
		    linkToStructure(aStream,theCurrentClass);
		}
		aStream.write('\n');
	    }
	    aStream.write("</PRE>\n");
	}
	aStream.write("<HR>\n<BR>\n");
	modifiers(aStream,_class);
	aStream.write(" " + theName + 
		      " <STRONG>" + _class.name() + "</STRONG>");
	if ( theInterfaceSize > 0 ) {
	    aStream.write("<BR>" + theRelationShip + " ");
	    Utility.quickSort(theInterfaces,0,theInterfaceSize-1);
	    for ( int theRank = 0; theRank < theInterfaceSize; theRank++ ) {
		if ( theRank > 0 ) {
		    aStream.write(", ");
		}
		linkToStructure(aStream,theInterfaces[theRank]);
	    }
	    aStream.write('\n');
	}
	comments(aStream,_class);
	seeTags(aStream,_class);
	aStream.write("<HR><BR>\n");
	theSubsumers = _subsumers.keys(); 
	boolean theFirstFlag = true;
	ClassDoc theCurrentClass;
	String theText;
	if ( theSubsumers.hasMoreElements() ) {
	    while ( theSubsumers.hasMoreElements() ) {
		theCurrentClass = (ClassDoc)(theSubsumers.nextElement());
		if ( isKnownType(theCurrentClass) ) {
		    theText = getInheritedMethods(theCurrentClass);
		    if ( theText.length() > 0 ) {
			if ( theFirstFlag ) {
			    theFirstFlag = false;
			    aStream.write("<DL><DT><STRONG>Inherited methods: </STRONG><DD>\n<TABLE>\n");
			}
			aStream.write("<TR VALIGN=\"TOP\"><TD>");
			linkToStructure(aStream,theCurrentClass);
			aStream.write(":</TD><TD>" + 
				      theText + 
				      "</TD></TR>\n");
		    }
		}
	    }
	}
	if ( ! _class.isInterface() ) {
	    theCurrentClass = _class.superclass();
	    while ( theCurrentClass != null ) {
		if ( isKnownType(theCurrentClass) ) {
		    theText = getInheritedMethods(theCurrentClass);
		    if ( theText.length() > 0 ) {
			if ( theFirstFlag ) {
			    theFirstFlag = false;
			    aStream.write("<DL><DT><STRONG>Inherited methods: </STRONG><DD>\n<TABLE>\n");
			}
			aStream.write("<TR VALIGN=\"TOP\"><TD>");
			linkToStructure(aStream,theCurrentClass);
			aStream.write(":</TD><TD>" + 
				      theText +
				      "</TD></TR>\n");
		    }
		}
		theCurrentClass = theCurrentClass.superclass();
	    }
	}
	if ( ! theFirstFlag ) {
	    aStream.write("</TABLE></DL>\n");
	}
    }

    public String getOutputFileName() {
	return _fileName;
    }

    private void signature(int anOffset,
			   Writer aStream,
			   ExecutableMemberDoc aMember) throws IOException {
	final Parameter[] theParameters = aMember.parameters();
	final int theSize = theParameters.length;
	if ( theSize > 0 ) {
	    for ( int theRank = 0; theRank < theSize; theRank++ ) {
		if ( theRank > 0 ) {
		    aStream.write(",\n");
		    for ( int theOtherRank = 0; theOtherRank < anOffset; theOtherRank++ ) {
			aStream.write(' ');
		    }
		}
		else {
		    aStream.write("(");
		}
		linkToType(aStream,theParameters[theRank].type());
		aStream.write(" " + theParameters[theRank].name());
	    }
	    aStream.write(")\n");
	}
	else {
	    aStream.write("()\n");
	}
    }

    private boolean parameterTags(Writer aStream, ExecutableMemberDoc aMember) throws IOException {
	final ParamTag[] theTags = aMember.paramTags();
	final int theSize = theTags.length;
	if ( theSize > 0 ) {
	    aStream.write("<DL><DT><STRONG>Parameters: </STRONG><DD><DL>\n");
	    ParamTag theCurrentTag;
	    for ( int theRank = 0; theRank < theSize; theRank++ ) {
		theCurrentTag = theTags[theRank];
		aStream.write("<DT>" + theCurrentTag.parameterName() +
			      " - <DD>" +  theCurrentTag.parameterComment() + "\n");
	    }
	    aStream.write("</DL></DL>\n");
	    return true;
	}
	else {
	    return false;
	}
    }

    private boolean throwsTag(Writer aStream,
			      ExecutableMemberDoc aMember) throws IOException {
	ThrowsTag[] theTags = aMember.throwsTags();
	final int theSize = theTags.length;
	if ( theSize > 0 ) {
	    aStream.write("<DL><DT><STRONG>Throws:</STRONG><DD><DL>\n");
	    ThrowsTag theCurrentTag;
	    for ( int theRank = 0; theRank < theSize; theRank++ ) {
		theCurrentTag = theTags[theRank];
		aStream.write("<DT>");
		linkToType(aStream,theCurrentTag.exception());
		aStream.write(" - <DD>" +  theCurrentTag.exceptionComment() + "\n");
	    }
	    aStream.write("</DL></DL>\n");
	    return true;
	}
	else {
	    return false;
	}
    }

    private boolean returnTag(Writer aStream, MethodDoc aMethod) throws IOException {
	final Tag[] theReturnTag = aMethod.tags("return");
	if ( theReturnTag.length > 0 ) {
	    aStream.write("<DL><DT><STRONG>Return:</STRONG><DD><DL>\n" +
			  theReturnTag[0].text() + "</DL></DL>\n");
	    return true;
	}
	else {
	    return false;
	}
    }

    private void scrollDownToConstructor(Writer aStream,ConstructorDoc aConstructor) throws IOException {
	scrollDownTo(aStream,
		     aConstructor.name() + aConstructor.signature(),
		     aConstructor.name());
	aStream.write(aConstructor.flatSignature());
    }

    private void scrollDownToMethod(Writer aStream, MethodDoc aMethod ) throws IOException {
	scrollDownTo(aStream,
		     aMethod.name() + aMethod.signature(),
		     aMethod.name());
	aStream.write(aMethod.flatSignature());
    }
    
    private void scrollDownToVariable(Writer aStream,FieldDoc aVariable) throws IOException {
	scrollDownTo(aStream,aVariable.name(),aVariable.name());
    }

    private void scrollDownTo(Writer aStream,String aTag, String aName) throws IOException {
	aStream.write("<A HREF=\"#" + aTag + "\">" + aName +
		      "<IMG WIDTH=9 HEIGHT=11 BORDER=0 ALIGN=TOP SRC=\"../../images/qbullets/qbullets/scrolldo.gif\"></A>");
    }

    private String _type;
    private ClassDoc _class;
    private String _fileName;
    private ConstructorDoc[] _constructors;
    private int _constructorSize;
    private MethodDoc[] _methods;
    private int _methodSize;
    private FieldDoc[] _fields;
    private int _fieldSize;
    private Hashtable _subsumers;
    private Stack _inheritancePath;
    
}
