//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.doclet;

import com.sun.javadoc.*;
import java.util.Stack;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;

public final class Doclet {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/doclet/Doclet.java,v $ $revision$ $Date: 1998/11/02 15:54:47 $ Copyright 1998 INRIA.";

    public static boolean validOptions(String[][] anOptionList,
				       DocErrorReporter reporter) {
	final int theSize = anOptionList.length;
	for ( int theRank = 0; theRank < theSize; theRank++ ) {
	    String[] theLine = anOptionList[theRank];
	    final int theOtherSize = theLine.length;
	    if ( theLine[0].equals("-d") ) {
		OUTPUT = theLine[1];
	    }
	    else {
		if ( theLine[0].equals("-template") ) {
		    TEMPLATE = theLine[1];
		}
		else {
		    if ( theLine[0].equals("-level") ) {
			LEVEL = Utility.parseLevel(theLine[1]);
		    }
		}
	    }
	}
	return true;
    }

    public static int optionLength(String anOption) {
	if ( anOption.equals("-d") ) {
	    return 2;
	}
	else {
	    if ( anOption.equals("-template") ) {
		return 2;
	    }
	    else {
		if ( anOption.equals("-level") ) {
		    return 2;
		}
		else {
		    return com.sun.javadoc.Doclet.optionLength(anOption);
		}
	    }
	}
    }

    public final static boolean start(RootDoc aRoot) {
	try {
	    final Hashtable theKnownTypes = knownTypes(LEVEL,aRoot);
	    final Vector theIndex = new Vector(1000,500);
	    final Hashtable theEntryList = new Hashtable(33);
	    final PackageDoc[] thePackages = aRoot.specifiedPackages();
	    final int thePackageSize = thePackages.length - Utility.filter(LEVEL,thePackages);
	    if ( thePackageSize > 0 ) {
		final PackagesWriter thePackagesWriter = new PackagesWriter(OUTPUT,TEMPLATE,theKnownTypes,thePackages,thePackageSize);
		thePackagesWriter.go();
		for ( int theRank = 0; theRank < thePackageSize; theRank++ ) {
		    documentPackage(theKnownTypes,LEVEL,theIndex,theEntryList,thePackages[theRank]);
		}
		index(theIndex,theEntryList,theKnownTypes);
	    }
	}
	catch ( IOException anException ) {
	    anException.printStackTrace();
	    System.exit(-1);
	}
	System.out.println('.');
	return true;
    }

    private final static void documentPackage(Hashtable aKnownTypeList,
					      int aLevel,
					      Vector anIndex,
					      Hashtable anEntryList,
					      PackageDoc aPackage) throws IOException {
	System.out.print('.');
	anIndex.addElement(aPackage);
	final ClassDoc[] theInterfaces = aPackage.interfaces();
	final int theInterfaceSize = theInterfaces.length - Utility.filter(aLevel,theInterfaces);
	final ClassDoc[] theClasses = aPackage.ordinaryClasses();
	final int theClassSize = theClasses.length - Utility.filter(aLevel,theClasses);
	final ClassDoc[] theExceptions = aPackage.exceptions();
	final int theExceptionSize = theExceptions.length - Utility.filter(aLevel,theExceptions);
	final ClassDoc[] theErrors = aPackage.errors();
	final int theErrorSize = theErrors.length - Utility.filter(aLevel,theErrors);
	final PackageWriter thePackageWriter = new PackageWriter(aPackage,OUTPUT,TEMPLATE,aKnownTypeList,theInterfaces,theInterfaceSize,theClasses,theClassSize,theExceptions,theExceptionSize,theErrors,theErrorSize);
	thePackageWriter.go();
	final String theName = aPackage.name();
	for ( int theRank = 0; theRank < theInterfaceSize; theRank++ ) {
	    documentClass("Interface",aKnownTypeList,aLevel,anIndex,anEntryList,theName,theInterfaces[theRank]);
	}
	for ( int theRank = 0; theRank < theClassSize; theRank++ ) {
	    documentClass("Class",aKnownTypeList,aLevel,anIndex,anEntryList,theName,theClasses[theRank]);
	}
	for ( int theRank = 0; theRank < theExceptionSize; theRank++ ) {
	    documentClass("Exception",aKnownTypeList,aLevel,anIndex,anEntryList,theName,theExceptions[theRank]);
	}
	for ( int theRank = 0; theRank < theErrorSize; theRank++ ) {
	    documentClass("Error",aKnownTypeList,aLevel,anIndex,anEntryList,theName,theErrors[theRank]);
	}
    }

    private final static void documentClass(String aType,
					    Hashtable aKnownTypeList,
					    int aLevel,
					    Vector anIndex,
					    Hashtable anEntryList,
					    String aPackage,
					    ClassDoc aStructure) throws IOException {
	System.out.print('.');
	anIndex.addElement(aStructure);
	final String theName = aStructure.name();
	updateEntryList(aKnownTypeList,theName);
	final InfoShunk theInfo = (InfoShunk)(aKnownTypeList.get(aStructure));
	final ConstructorDoc[] theConstructors = theInfo._constructors;
	final MethodDoc[] theMethods = theInfo._methods;
	final FieldDoc[] theFields = theInfo._fields;
	final Hashtable theSubsumers = theInfo._subsumers;
	final Stack theInheritancePath = theInfo._inheritance;
	final int theConstructorSize = theInfo._constructorSize;
	final int theMethodSize = theInfo._methodSize;
	final int theFieldSize = theInfo._fieldSize;
	final ClassWriter theClassWriter = new ClassWriter(aType,aStructure,OUTPUT,TEMPLATE,aKnownTypeList,theConstructors,theConstructorSize,theMethods,theMethodSize,theFields,theFieldSize,theSubsumers,theInheritancePath);
	theClassWriter.go();
	for ( int theRank = 0; theRank < theConstructorSize; theRank++ ) {
	    anIndex.addElement(theConstructors[theRank]);
	    updateEntryList(anEntryList,theConstructors[theRank].name());
	}
	MethodDoc theCurrentMethod;
	String theCurrentName;
	String theCurrentSignature;
	final MethodDoc[] theOriginalMethods = aStructure.methods();
	final int theOriginalSize = theOriginalMethods.length;
	for ( int theRank = 0; theRank < theMethodSize; theRank++ ) {
	    theCurrentMethod = theMethods[theRank];
	    if ( aStructure != theCurrentMethod.containingClass() ) {
		theCurrentName = theCurrentMethod.name();
		theCurrentSignature = theCurrentMethod.signature();
		for ( int theOtherRank = 0; theOtherRank < theOriginalSize; theOtherRank++ ) {
		    theCurrentMethod = theOriginalMethods[theOtherRank];
		    if ( ( theCurrentName.equals(theCurrentMethod.name()) ) && 
			 ( theCurrentSignature.equals(theCurrentMethod.signature())) ) {
			break;
		    }
		}
	    }
	    anIndex.addElement(theCurrentMethod);
	    updateEntryList(anEntryList,theMethods[theRank].name());
	}
	for ( int theRank = 0; theRank < theFieldSize; theRank++ ) {
	    anIndex.addElement(theFields[theRank]);
	    updateEntryList(anEntryList,theFields[theRank].name());
	}
    }

    private final static void updateEntryList(Hashtable anEntryList,String aName) {
	final String theFirstChar = aName.substring(0,1).toUpperCase();
	if ( ! anEntryList.containsKey(theFirstChar) ) {
	    anEntryList.put(theFirstChar,theFirstChar);
	}
    }

    private final static Hashtable computeSubsumption(Hashtable theSubsumers,
						      ClassDoc aStructure) { 
	final ClassDoc theSuperClass = aStructure.superclass();
	if ( ( theSuperClass != null )  && 
	     ( ! theSubsumers.containsKey(theSuperClass)) ) {
	    if ( theSuperClass.isInterface() ) {
		theSubsumers.put(theSuperClass,"x");
	    }
	    computeSubsumption(theSubsumers,theSuperClass);
	}
	final ClassDoc[] theInterfaces = aStructure.interfaces();
	for ( int theRank = 0; theRank < theInterfaces.length; theRank++ ) {
	    if ( ! theSubsumers.containsKey(theInterfaces[theRank]) ) {
		theSubsumers.put(theInterfaces[theRank],"x");
		computeSubsumption(theSubsumers,theInterfaces[theRank]);
	    }
	}
	return theSubsumers;
    }

    private final static void index(Vector anIndex, 
				    Hashtable anEntryList,
				    Hashtable aKnownTypeList) throws IOException{
	final IndexWriter theIndexWriter = new IndexWriter(OUTPUT,TEMPLATE,aKnownTypeList,anEntryList,anIndex);
	theIndexWriter.go();
    }

    private final static Hashtable knownTypes(int aLevel,RootDoc aRoot) {
	return precomputeClassInfo(aRoot.specifiedPackages(),aLevel);
    }

    private final static Hashtable precomputeClassInfo(PackageDoc[] aPackageList, int aLevel) {
	final Hashtable theDico = new Hashtable();
	final int thePackageSize = aPackageList.length;
	PackageDoc theCurrentPackage;
	for ( int theRank = 0; theRank < thePackageSize; theRank++ ) {
	    theCurrentPackage = aPackageList[theRank];
	    precomputeClassInfo(theDico,theCurrentPackage.interfaces());
	    precomputeClassInfo(theDico,theCurrentPackage.ordinaryClasses());
	    precomputeClassInfo(theDico,theCurrentPackage.exceptions());
	    precomputeClassInfo(theDico,theCurrentPackage.errors());
	}
	InfoShunk theInfo;
	Enumeration theClasses = theDico.keys();
	while ( theClasses.hasMoreElements() ) {
	    theInfo = (InfoShunk)(theDico.get((ClassDoc)(theClasses.nextElement())));
	    if ( ! theInfo.isUpToDate() ) {
		theInfo.update(theDico);
	    }
	}
	theClasses = theDico.keys();
	while ( theClasses.hasMoreElements() ) {
	    theInfo = (InfoShunk)(theDico.get((ClassDoc)(theClasses.nextElement())));
	    theInfo.filter(aLevel);
	}
	return theDico;
    }

    private final static void precomputeClassInfo(Hashtable aDico,
						  ClassDoc[] aClassList) {
	final int theSize = aClassList.length;
	ClassDoc theCurrentClass;
	for ( int theRank = 0; theRank < theSize; theRank++ ) {
	    theCurrentClass = aClassList[theRank];
	    aDico.put(theCurrentClass,new InfoShunk(theCurrentClass));
	}
    }

    private static int LEVEL = Constants.LEVEL_END_USER;
    private static String OUTPUT = ".";
    private static String TEMPLATE = ".";
    
}

