//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.format;

import figue.box.GlyphInterface;
import figue.geometry.Dimension;
import figue.internal.ContextInterface;
import figue.internal.Globals;

public abstract class Distance extends Value implements DistanceInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/format/Distance.java,v $ $revision$ $Date: 1998/11/02 15:55:00 $ Copyright 1998 INRIA.";

    //// WHY EXTENDS VALUE SINCE THERE IS A DEPENDANT ONLY FOR IMMEDIATE DISTANCE ?
  
    protected Distance(DecalageInterface aDecalage) {
	super();
	_decalage = aDecalage;
    }
  
    protected void init (DecalageInterface aDecalage) {
	super.init();
	_decalage = aDecalage;
    }

    public final DecalageInterface getDecalage() {
	return _decalage;
    }

    public void setDecalage(DecalageInterface aDecalage) {
	_decalage = aDecalage;
    }

    public final boolean isSufficientFor(Dimension aSize) {
	return ( getValue() >= aSize.getWidth() );
    }
  
    public final void checkBefore(AlternativeInterface anAlternative) 
	throws StopFormattingSonsException {
	try {
	    while ( getValue() < 0 ) {
		if ( Globals.DEBUG_FORMAT ) {
		    System.out.println("checkBefore : " + getValue());
		}
		tryAnotherReformat(anAlternative);
	    }
	}
	catch ( TooSmallPageException anException ) {
	    if ( Globals.DEBUG_FORMAT_HISTORY ) {
		System.out.println("not enought space ... before ");
	    }
	}
    }
  
    public final void checkAfter(GlyphInterface aGlyph,
				 AlternativeInterface anAlternative)
	throws StopFormattingSonsException {
	try {
	    while ( ! isSufficientFor(aGlyph.getSizeValue()) ) { /// getSize()
		if ( Globals.DEBUG_FORMAT ) {
		    System.out.println("checkAfter : " + getValue() + " " + aGlyph.toString());
		}
		tryAnotherReformat(anAlternative);
		aGlyph.update();
	    }
	}
	catch ( TooSmallPageException anException ) {
	    if ( Globals.DEBUG_FORMAT_HISTORY ) {
		System.out.println("not enought space ... after " + aGlyph);
	    }
	}
    }
  
    public DistanceInterface lowerLevel(ContextInterface aContext,
					GlyphInterface aGlyph) {
	if ( Globals.DEBUG_FORMAT ) {
	    System.out.println("-> lowerLevel " + aGlyph.toString());
	}
	DecalageInterface theDecalage = aContext.newLazyDecalage(aGlyph);
	DistanceInterface theDistance = aContext.newLazyDistance(aGlyph,this,theDecalage);
	theDecalage.addDependent((ComputedValueInterface)theDistance);
	aGlyph.addDependent(theDecalage);
	return theDistance;
    }

    private final void tryAnotherReformat(AlternativeInterface anAlternative) 
	throws TooSmallPageException, StopFormattingSonsException {
	anAlternative.setAction(ActionNone.INSTANCE);
	tryAlternative(anAlternative);
	anAlternative.execute();
    }
  
    public void dispose(ContextInterface aContext) {
	_decalage.dispose(aContext);
    }
  
    private DecalageInterface _decalage;
  
}
