//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.format;

import figue.box.GlyphInterface;
import figue.internal.Globals;
import figue.geometry.Dimension;
import figue.geometry.Size;

public final class FormatStatusProceed extends FormatStatus {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/format/FormatStatusProceed.java,v $ $revision$ $Date: 1998/11/02 15:55:02 $ Copyright 1998 INRIA.";
  
    public  FormatStatusProceed() {
	super();
	_currentSize = new Size();
	_previousAwakeSize = new Size();
	_estimatedSize = new Size();
	_estimatedHeight = 0;
	_previousAwakeHeight = 0;
    }
  
    public final boolean isProceed() {
	return true;
    }

    public final Size getCurrentSize() {
	return _currentSize;
    }

    public final boolean setCurrentSizeAndCheckAwake(Size aSize) {
	final boolean theAwakeningFlag 
	    = ( aSize.getHeight() - _previousAwakeSize.getHeight() ) >=
	    Globals.AWAKE_DRAW_WHEN_FORMAT_HEIGHT;
	_currentSize.setBounds(aSize);
	if ( theAwakeningFlag ) {
	    _previousAwakeSize.setBounds(_currentSize);
	}
	return theAwakeningFlag;
    }

    public final boolean improveEstimatedHeightAndCheckAwake(int anHeight) {
	_estimatedHeight+= anHeight;
	final boolean theAwakeningFlag =
	    ( ( _estimatedHeight - _previousAwakeHeight ) > Globals.AWAKE_DRAW_WHEN_FORMAT_HEIGHT );
	if ( theAwakeningFlag ) {
	    _previousAwakeHeight = _estimatedHeight;
	}
	return theAwakeningFlag;
    }

    public final Dimension getEstimatedSize(GlyphInterface aGlyph) { 
	return _estimatedSize;
    }

    public final void setEstimatedSize(Dimension aSize) {
	_estimatedSize.setBounds(aSize);
    }

    public final Dimension expandEstimatedSize(int aWidth, int aHeight) { 
	_estimatedSize.setWidth(aWidth);
	_estimatedSize.setHeight(aHeight);
	_previousAwakeSize.setBounds(_estimatedSize);
	return _estimatedSize;
    }

    public final String toString() {
	StringBuffer theResult = new StringBuffer("Status<Proceed:");
	theResult.append(_currentSize);
	theResult.append(">");
	return theResult.toString();
    }

    Size _currentSize;
    Size _previousAwakeSize;
    Dimension _estimatedSize;
    int _estimatedHeight;
    int _previousAwakeHeight;
}

