//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.format;

import figue.box.GlyphInterface;
import figue.internal.ContextInterface;
import figue.internal.Globals;

public final class LazyDistance extends Distance implements ComputedValueInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/format/LazyDistance.java,v $ $revision$ $Date: 1998/11/02 15:55:04 $ Copyright 1998 INRIA.";
  
    public LazyDistance(GlyphInterface aGlyph,
			DistanceInterface aDistance,
			DecalageInterface aDecalage) {
	super(aDecalage);
	_glyph = aGlyph;
	_lastValue = -1;
	_distance = aDistance;
    }
    
    public void init(GlyphInterface aGlyph,
		     DistanceInterface aDistance,
		     DecalageInterface aDecalage) {
	super.init(aDecalage);
	_glyph = aGlyph;
	_lastValue = -1;
	_distance = aDistance;
    }

    public final DistanceInterface getDistance() {
	return _distance;
    }
  
    public final int getValue() {
	if ( _lastValue == -1 ) {
	    update();
	}
	return _lastValue;
    }

    public final int getEagerValue() {
	updateEager();
	return _lastValue;
    }
  
    public void update() {
	if ( _glyph != null ) {
	    _lastValue = _glyph.computeDistance2D(_distance,getDecalage());
	    notifyChange();
	}
    }

    public void updateEager() {
	if ( _glyph != null ) {
	    _lastValue = _glyph.computeDistance2D(_distance,getDecalage());
	    notifyEagerChange();
	}
    }

    public final void upperLevel(ContextInterface aContext) {
	if ( Globals.DEBUG_FORMAT ) {
	    System.out.println("-> upperLevel");
	}
	_distance.clearDependent();
	disposeDecalage(aContext,getDecalage());
	dispose(aContext);
    }

    public final void newBrother(ContextInterface aContext,
				 GlyphInterface aGlyph) {
	if ( Globals.DEBUG_FORMAT ) {
	    System.out.println("-> newBrother " + aGlyph.toString());
	}
	_lastValue = -1;
	LazyDecalageToBrother theDecalage = aContext.newLazyDecalageToBrother(_glyph,aGlyph,getDecalage());
	theDecalage.addDependent(this);
	getDecalage().addDependent(theDecalage);
	setDecalage(theDecalage);
	aGlyph.addDependent(theDecalage);
    }

    public final void tryAlternative(AlternativeInterface anAlternative) {
	if ( Globals.DEBUG_FORMAT_HISTORY ) {
	    System.out.println("LazyDistance.tryAlternative " + _glyph.toString()); 
	}
	_glyph.tryAlternativeForDistance(this,anAlternative);
    }

    public final void clearDependent() {
	_distance.removeDependent();
    }

    private final void disposeDecalage(ContextInterface aContext,
				       DecalageInterface aDecalage) {
	DecalageInterface theDecalage = aDecalage;
	DecalageInterface theOtherDecalage;
	while ( theDecalage != null ) {
	    theOtherDecalage = theDecalage.getDecalage();
	    theDecalage.dispose(aContext);
	    theDecalage = theOtherDecalage;
	}
    }

    public final void dispose(ContextInterface aContext) {
 	_glyph = null;
 	_lastValue = -1;
 	_distance = null;
	aContext.disposeLazyDistance(this);
    }

    public final String toString() {
	StringBuffer theResult = new StringBuffer("LazyDistance<");
	theResult.append(_lastValue);
	if ( _glyph != null ) {
	    theResult.append(", ").append(_glyph.toString());
	}
	if ( _distance != null ) {
	    theResult.append(", ").append(_distance);
	}
	if ( getDecalage() != null ) {
	    theResult.append(", ").append(getDecalage());
	}
	theResult.append(">");
	return theResult.toString();
    }
  
    private GlyphInterface _glyph;
    private int _lastValue;
    private DistanceInterface _distance;

}
