//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.format;

import figue.internal.ContextInterface;
import figue.geometry.Point;
import figue.geometry.Dimension;

public final class Modification implements ModificationInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/format/Modification.java,v $ $revision$ $Date: 1998/11/02 15:55:04 $ Copyright 1998 INRIA.";
  
    public Modification(ModificationStatus aStatus) {
	_origin = null;
	_size = null;
	_status = aStatus;
	_sonRank = NO_RANK_SPECIFIED;
	_sonCleanFlag = false;
    }

    public final void init(ModificationStatus aStatus) {
	_origin = null;
	_size = null;
	_status = aStatus;
	_sonRank = NO_RANK_SPECIFIED;
	_sonCleanFlag = false;	
    }

    public final void dispose(ContextInterface aContext) {
	aContext.disposeModification(this);
    }
  
    public final boolean isModified() {
	return _status.isModified();
    }
  
    public final boolean isOriginModified() {
	return _status.isOriginModified();
    }
  
    public final boolean isSizeModified() {
	return _status.isSizeModified();
    }

    public final boolean isSonModified() {
	return _status.isSonModified();
    }

    public final int getSonModifiedRank() {
	return _sonRank;
    }

    public final void setSonModifiedRank(int aRank) {
	_sonRank = aRank;
    }

    public final boolean isSonClean() {
	return _sonCleanFlag;
    }

    public final void setSonClean(boolean aFlag) {
	_sonCleanFlag = aFlag;
    }
  
    public final Point getOrigin() {
	return _origin;
    }

    public final void setOrigin(int x, int y) {
	if ( _origin == null ) {
	    _origin = new Point(x,y);
	}
	else {
	    _origin.setLocation(x,y);
	}
	_status = _status.originModified();
    }
  
    public final Dimension getSize() {
	return _size;
    }

    public final void setSize(Dimension aDimension) {
	if ( _size == null ) {
	    _size = new Dimension(aDimension);
	}
	else {
	    _size.setBounds(aDimension);
	}
	_status = _status.sizeModified();
    }

    public final String toString() {
	StringBuffer theResult = new StringBuffer("Modification<");
	boolean theFlag = false;
	if ( isOriginModified() ) {
	    theResult.append(_origin);
	    theFlag = true;
	}
	if ( isSizeModified() ) {
	    if ( theFlag ) {
		theResult.append(", ");
	    }
	    else {
		theFlag = true;
	    }
	    theResult.append(_size);
	}
	if ( isSonModified() ) {
	    if ( theFlag ) {
		theResult.append(", ");
	    }
	    theResult.append("son");
	    if ( getSonModifiedRank() != NO_RANK_SPECIFIED ) {
		theResult.append(" ").append(getSonModifiedRank());
		if ( ! isSonClean() ) {
		    theResult.append(" not clean");
		}
	    }
	}
	theResult.append(">");
	return theResult.toString();
    }

    private Point _origin;
    private Dimension _size;
    private ModificationStatus _status;
    private int _sonRank;
    private boolean _sonCleanFlag;

    public final static int NO_RANK_SPECIFIED = -2;
  
}
