//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.geometry;

/**
 * [PENDING: update the comments] Rectangle.
 */

public class Rectangle {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/geometry/Rectangle.java,v $ $revision$ $Date: 1998/11/02 15:55:10 $ Copyright 1998 INRIA.";
    
    public Rectangle(int x,int y,int width,int height) {
	super();
	_x = x;
	_y = y;
	_width = width;
	_height = height;
    }
    
    public Rectangle(int width,int height) {
	this(0,0,width,height);
    }
    
    public Rectangle() {
	this(0,0,0,0);
    }
    
    public Rectangle(Rectangle aRectangle) {
	this(aRectangle.getX(),aRectangle.getY(),aRectangle.getWidth(),aRectangle.getHeight());
    }

    public final int getX() {
	return _x;
    }
    
    public final void setX(int x) { 
	_x = x;
    }
    
    public final int getY() {
	return _y;
    }
    
    public final void setY(int y) { 
	_y = y;
    }
    
    public final int getWidth() { 
	return _width;
    }
    
    public final void setWidth(int width) { 
	_width = width;
    }
    
    public final int getHeight() {
	return _height;
    }
    
    public final void setHeight(int height) { 
	_height = height;
    }
    
    public final void setLocation(Point aPoint) {
	setLocation(aPoint.getX(),aPoint.getY());
    }
    
    public final void setLocation(int x, int y) {
	_x = x;
	_y = y;
    }
    
    public final void translate(Point aPoint) { 
	translate(aPoint.getX(),aPoint.getY());
    }
    
    public final void translate(int x, int y) {
	_x += x;
	_y += y;
    }
    
    public final void setSize(int width, int height) {
	_width = width;
	_height = height;
    }
    
    public final void setBounds(Rectangle aRectangle) {
	setBounds(aRectangle.getX(),
		  aRectangle.getY(),
		  aRectangle.getWidth(),
		  aRectangle.getHeight());
    }
    
    public final void setBounds(int x, int y, int width, int height) {
	_x = x;
	_y = y;
	_width = width;
	_height = height;
    }
    
    public final boolean contains(Point aPoint) {
	return contains(aPoint.getX(),aPoint.getY());
    }
    
    public final boolean contains(int x, int y) {
	return ( ( x >= getX() ) && ( ( getX() + getWidth() ) >= x ) &&
		 ( y >= getY() ) && ( ( getY() + getHeight() ) >= y ) );
    }
    
    public final boolean contains(Rectangle aRectangle) { 
	return union(aRectangle).equals(this);
    }
    
    public final boolean sameShape(Rectangle aRectangle) {
	return ( ( getWidth() == aRectangle.getWidth() ) &&
		 ( getHeight() == aRectangle.getHeight() ) );
    }
    
    public final boolean isEmpty() {
	return ( ( getWidth() == 0 ) && ( getHeight() == 0 ) );
    }

    public final Rectangle union(Rectangle r) {
	final int x1 = Math.min(_x, r.getX());
	final int x2 = Math.max(_x + _width, r.getX() + r.getWidth());
	final int y1 = Math.min(_y, r.getY());
	final int y2 = Math.max(_y + _height, r.getY() + r.getHeight());
	return new Rectangle(x1, y1, x2 - x1, y2 - y1);
    }

    public final Rectangle intersection(Rectangle r) {
	final int x1 = Math.max(_x, r.getX());
	final int x2 = Math.min(_x + _width, r.getX() + r.getWidth());
	final int y1 = Math.max(_y, r.getY());
	final int y2 = Math.min(_y + _height, r.getY() + r.getHeight());
	return new Rectangle(x1, y1, x2 - x1, y2 - y1);
    }

    public final boolean intersects(Rectangle r) {
	return ( !((r.getX() + r.getWidth() <= _x) ||
		   (r.getY() + r.getHeight() <= _y) ||
		   (r.getX() >= _x + getWidth()) ||
		   (r.getY() >= _y + getHeight())) );
    }
    
    public String toString() {
	StringBuffer theResult = new StringBuffer("Rectangle<");
	internalToString(theResult);
	theResult.append(">");
	return theResult.toString();
    }
    void internalToString(StringBuffer theResult) {
	theResult.append(getX()).append("/").append(getY())
	    .append(";").append(getWidth()).append("x").append(getHeight());
    }

    private int _x;
    private int _y;
    private int _width;
    private int _height;
    
}

