//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.internal;

import figue.box.GlyphInterface;
import figue.path.SelectionPathInterface;
import figue.path.PathInterface;
import figue.path.InsertGlyph;
import figue.path.RemoveGlyph;
import figue.path.ReplaceGlyph;
import figue.path.MarkStable;
import figue.path.MarkUnstable;
import figue.FacadeInterface;

public final class PathBuildingVisitor extends PathVisitor {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/internal/PathBuildingVisitor.java,v $ $revision$ $Date: 1998/11/02 15:55:15 $ Copyright 1998 INRIA.";
    
    public PathBuildingVisitor(FacadeInterface aFacade) {
	super(aFacade);
    }
    
    public final void markStable(MarkStable aMarkStableOperation,
				 GlyphInterface aGlyph,
				 SelectionPathInterface aSelectionPath) {
	aGlyph.setDirty();
    }
    
    public final void markStable(MarkStable aMarkStableOperation,
				 GlyphInterface aGlyph) {
	aGlyph.setDirty();
    }
    
    public final void markUnstable(MarkUnstable aMarkUnstableOperation,
				   GlyphInterface aGlyph,
				   SelectionPathInterface aSelectionPath) {
	aGlyph.setUnstable();
    }
    
    public final void markUnstable(MarkUnstable aMarkUnstableOperation,
				   GlyphInterface aGlyph) {
	aGlyph.setUnstable();
    }
    
    public final void insertGlyph(InsertGlyph anInsertOperation,
				  GlyphInterface aGlyph,
				  SelectionPathInterface aSelectionPath) {
	aGlyph.insertChild(anInsertOperation.getRank(),
			   anInsertOperation.getGlyph());
    }
    
    public final void insertGlyph(InsertGlyph anInsertOperation,
				  GlyphInterface aGlyph) {
	aGlyph.insertChild(anInsertOperation.getRank(),
			   anInsertOperation.getGlyph());
    }
    
  public final void replaceGlyph(ReplaceGlyph aReplaceOperation,
				 GlyphInterface aGlyph,
				 SelectionPathInterface aSelectionPath) {
      removeSelections(aGlyph,aSelectionPath);
      GlyphInterface aFather = aGlyph.getParent();
      aFather.replaceChild(aFather.indexOf(aGlyph),
			   aReplaceOperation.getGlyph());
  }
    
    public final void replaceGlyph(ReplaceGlyph aReplaceOperation,
				   GlyphInterface aGlyph) {
	GlyphInterface aFather = aGlyph.getParent();
	aFather.replaceChild(aFather.indexOf(aGlyph),
			     aReplaceOperation.getGlyph());
    }
    
    public final void removeGlyph(RemoveGlyph aRemoveOperation,
				  GlyphInterface aGlyph,
				  SelectionPathInterface aSelectionPath) {
	removeSelections(aGlyph,aSelectionPath);
	aGlyph.removeChild(aRemoveOperation.getRank());
    }
    
    public final void removeGlyph(RemoveGlyph aRemoveOperation,
				  GlyphInterface aGlyph) {
	aGlyph.removeChild(aRemoveOperation.getRank());
    }
    
    public final void down(GlyphInterface aGlyph,
			   SelectionPathInterface aSelectionPath,
			   PathInterface aPath) {
	final int thePathRank = aPath.getRank();
	int theRank = aSelectionPath.indexOf(thePathRank);
    if ( theRank == -1 ) {
	aPath.accept(this,
		     aGlyph.getChild(thePathRank));
    }
    else {
      aPath.accept(this,
		   aGlyph.getChild(thePathRank),
		   aSelectionPath.getChild(theRank));
    }
    }
    
    public final void down(GlyphInterface aGlyph,
			   PathInterface aPath) {
	aPath.accept(this,aGlyph.getChild(aPath.getRank()));
    }
    
}
