//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.internal;

import figue.geometry.Rectangle;
import figue.path.SelectionPathInterface;
import figue.path.PathInterface;
import figue.path.MarkGlyph;
import figue.box.GlyphInterface;
import figue.error.ShouldNotBeCalledError;
import figue.FacadeInterface;

public final class PathCollectAreaVisitor extends PathVisitor {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/internal/PathCollectAreaVisitor.java,v $ $revision$ $Date: 1998/11/02 15:55:16 $ Copyright 1998 INRIA.";
    
    public PathCollectAreaVisitor(FacadeInterface aFacade) {
	super(aFacade);
	_zone = new Rectangle();
	_zoneNotInitializedFlag = true;
	_rectangleCacheForBoundingBox = new Rectangle();
	_visibleZone = new Rectangle();
    }
    
    public final void markGlyph(MarkGlyph aMarkOperation,
				GlyphInterface aGlyph) {
	updateZone(aGlyph);
    }
    
    public final void down(GlyphInterface aGlyph,
			   SelectionPathInterface aSelectionPath,
			   PathInterface aPath) {
	reportError(new ShouldNotBeCalledError());
    }
    
    public final void down(GlyphInterface aGlyph,
			   PathInterface aPath) {
	final int thePathRank = aPath.getRank();
	aGlyph.doTranslatesZones(_zone,_visibleZone,thePathRank);
	aPath.accept(this,aGlyph.getChild(thePathRank));
	aGlyph.undoTranslateZones(_zone,_visibleZone,thePathRank);
    }
    
    public final Rectangle getZone() {
	return _zone;
    }
    
    public final void reinit() {
	_zone.setBounds(0,0,0,0);
	_zoneNotInitializedFlag = true;
    }
    
    private final void updateZone(GlyphInterface aGlyph) {
	aGlyph.computeBoundingBox(_rectangleCacheForBoundingBox);
	if ( _zoneNotInitializedFlag ) {
	    _zoneNotInitializedFlag = false;
	    _zone.setBounds(_rectangleCacheForBoundingBox);
	}
	else {
	    _zone.setBounds(_zone.union(_rectangleCacheForBoundingBox));
	}
    }
    
    private Rectangle _zone;
    private boolean _zoneNotInitializedFlag;
    private Rectangle _rectangleCacheForBoundingBox;
    private Rectangle _visibleZone;
    
}
