//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.internal;

import figue.box.GlyphInterface;
import figue.path.SelectionPathInterface;
import figue.path.PathInterface;
import figue.path.TouchGlyph;
import figue.path.InsertGlyph;
import figue.path.RemoveGlyph;
import figue.path.ReplaceGlyph;
import figue.format.SonModificationStatus;
import figue.FacadeInterface;

public final class PathModificationVisitor extends PathVisitorIncremental {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/internal/PathModificationVisitor.java,v $ $revision$ $Date: 1998/11/02 15:55:16 $ Copyright 1998 INRIA.";
    
    public PathModificationVisitor(FacadeInterface aFacade) {
	super(aFacade);
    }
    
    public final void insertGlyph(InsertGlyph anInsertOperation,
				  GlyphInterface aGlyph,
				  SelectionPathInterface aSelectionPath) {
	GlyphInterface theSon = anInsertOperation.getGlyph();
	aGlyph.insertChild(anInsertOperation.getRank(),theSon);
	theSon.markAllBranchDirty(SonModificationStatus.EXPAND,
				  getContext());
    }
    
    public final void insertGlyph(InsertGlyph anInsertOperation,
				  GlyphInterface aGlyph) {
	GlyphInterface theSon = anInsertOperation.getGlyph();
	aGlyph.insertChild(anInsertOperation.getRank(),theSon);
	theSon.markAllBranchDirty(SonModificationStatus.EXPAND,
				  getContext());
    }
    
    public final void replaceGlyph(ReplaceGlyph aReplaceOperation,
				   GlyphInterface aGlyph,
				   SelectionPathInterface aSelectionPath) {
	updateRefreshZone(aGlyph);
	removeSelections(aGlyph,aSelectionPath);
	GlyphInterface aFather = aGlyph.getParent();
	GlyphInterface theSon = aReplaceOperation.getGlyph();
	aFather.replaceChild(aFather.indexOf(aGlyph),theSon);
	theSon.markAllBranchDirty(SonModificationStatus.SHRINK,
				  getContext());
    }
    
    public final void replaceGlyph(ReplaceGlyph aReplaceOperation,
				   GlyphInterface aGlyph) {
	updateRefreshZone(aGlyph);
	GlyphInterface aFather = aGlyph.getParent();
	final int theRank = aFather.indexOf(aGlyph);
	GlyphInterface theSon = aReplaceOperation.getGlyph();
	aFather.replaceChild(theRank,theSon);
	theSon.markAllBranchDirty(SonModificationStatus.SHRINK,
				  getContext());
  }
    
    public final void touchGlyph(TouchGlyph aTouchOperation,
				 GlyphInterface aGlyph,
				 SelectionPathInterface aSelectionPath) {
	updateRefreshZone(aGlyph);
	aGlyph.markAllBranchDirty(SonModificationStatus.SHRINK,
				  getContext());
    }
    
    public final void touchGlyph(TouchGlyph aTouchOperation,
				 GlyphInterface aGlyph) {
    updateRefreshZone(aGlyph);
    aGlyph.markAllBranchDirty(SonModificationStatus.SHRINK,
			      getContext());
    }
    
    public final void removeGlyph(RemoveGlyph aRemoveOperation,
				  GlyphInterface aGlyph,
				  SelectionPathInterface aSelectionPath) {
	final int theRank = aRemoveOperation.getRank();
	doTranslateZones(aGlyph,theRank);
	updateRefreshZone(aGlyph.getChild(theRank));
	undoTranslateZones(aGlyph,theRank);
	removeSelections(aGlyph,aSelectionPath);
	aGlyph.getChild(theRank).markAllBranchDirty(SonModificationStatus.KILL,
						    getContext());
	aGlyph.removeChild(theRank);
    }
    
    public final void removeGlyph(RemoveGlyph aRemoveOperation,
				  GlyphInterface aGlyph) {
	final int theRank = aRemoveOperation.getRank();
	doTranslateZones(aGlyph,theRank);
	updateRefreshZone(aGlyph.getChild(theRank));
	undoTranslateZones(aGlyph,theRank);
	aGlyph.getChild(theRank).markAllBranchDirty(SonModificationStatus.KILL,
						    getContext());
	aGlyph.removeChild(theRank);
    }
    
    public final void down(GlyphInterface aGlyph,
			   SelectionPathInterface aSelectionPath,
			   PathInterface aPath) {
	final int thePathRank = aPath.getRank();
	doTranslateZones(aGlyph,thePathRank);
	int theRank = aSelectionPath.indexOf(thePathRank);
	if ( theRank == -1 ) {
	    aPath.accept(this,aGlyph.getChild(thePathRank));
	}
	else {
	    aPath.accept(this,
			 aGlyph.getChild(thePathRank),
			 aSelectionPath.getChild(theRank));
	}
	undoTranslateZones(aGlyph,thePathRank);
    }
    
    public final void down(GlyphInterface aGlyph,
			   PathInterface aPath) {
	final int thePathRank = aPath.getRank();
	doTranslateZones(aGlyph,thePathRank);
	aPath.accept(this,aGlyph.getChild(thePathRank));
	undoTranslateZones(aGlyph,thePathRank);
    }

}
