//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.internal;

import figue.geometry.Rectangle;
import figue.geometry.Dimension;
import figue.box.GlyphInterface;
import figue.FacadeInterface;

public abstract class PathVisitorIncremental extends PathVisitor {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/internal/PathVisitorIncremental.java,v $ $revision$ $Date: 1998/11/02 15:55:17 $ Copyright 1998 INRIA.";
    
    protected PathVisitorIncremental(FacadeInterface aFacade) {
	super(aFacade);
	_refreshZone = new Rectangle();
	_visibleZone = new Rectangle();
	_rectangleCacheForBoundingBox = new Rectangle();
	_refreshZoneNotInitializedFlag = true;
    }
    
    public final void updateRefreshZone(GlyphInterface aGlyph) {
	aGlyph.computeBoundingBox(_rectangleCacheForBoundingBox);
	_rectangleCacheForBoundingBox.setBounds(_rectangleCacheForBoundingBox.intersection(_visibleZone));
	if ( _refreshZoneNotInitializedFlag ) {
	    _refreshZoneNotInitializedFlag = false;
	    _refreshZone.setBounds(_rectangleCacheForBoundingBox);
	}
	else {
	    _refreshZone.setBounds(_refreshZone.union(_rectangleCacheForBoundingBox));
	}
    }
    
    public final void updateRefreshZone(GlyphInterface aGlyph,
					int anOldX,
					int anOldY,
					Dimension anOldSize) {
	updateRefreshZone(aGlyph);
	aGlyph.computeOldBoundingBox(_rectangleCacheForBoundingBox,anOldX,anOldY,anOldSize);
	_rectangleCacheForBoundingBox.setBounds(_rectangleCacheForBoundingBox.intersection(_visibleZone));
	_refreshZone.setBounds(_refreshZone.union(_rectangleCacheForBoundingBox));
    }
    
    public final Rectangle getRefreshZone() {
	return _refreshZone;
    }
    
    public final void setVisibleZone(Rectangle aZone) {
	_visibleZone.setBounds(aZone);
	_refreshZone.setBounds(0,0,0,0);
	_refreshZoneNotInitializedFlag = true;
    }
    
    protected final void doTranslateZones(GlyphInterface aGlyph, int aRank) {
	aGlyph.doTranslatesZones(_refreshZone,_visibleZone,aRank);
    }
    
    protected final void undoTranslateZones(GlyphInterface aGlyph, int aRank) {
	aGlyph.undoTranslateZones(_refreshZone,_visibleZone,aRank);
    }
    
    public void dispose() {
	_refreshZone = null;
	_visibleZone = null;
	_rectangleCacheForBoundingBox = null;
	super.dispose();
    }
    
    private Rectangle _refreshZone;
    private Rectangle _visibleZone;
    private Rectangle _rectangleCacheForBoundingBox;
    private boolean _refreshZoneNotInitializedFlag;
    
}
