//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.io.xml;

import figue.*;
import figue.box.*;
import figue.path.*;
import figue.resource.*;
import figue.selection.*;
import figue.internal.*;
import figue.FacadeInterface;

import java.io.Writer;
import java.io.IOException;
import java.io.StringReader;
import java.util.Enumeration;
import antlr.collections.AST;
import antlr.ASTFactory;
import java.util.Hashtable;

import figue.*;

/**
 * [PENDING: update the comments] Utility.
 * @level advanced
 */

public final class Utility {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/io/xml/Utility.java,v $ $revision$ $Date: 1998/11/02 15:55:21 $ Copyright 1998 INRIA.";
 
    private Utility() {
    }

    final static int parseInt(String aString) {
	try {
	    if ( aString != null ) {
		return Integer.parseInt(aString);
	    }
	}
	catch ( NumberFormatException anException ) {
	}
	return -1;
    }

    final static boolean isSpace(String aString) {
	final int theLength = aString.length();
	for ( int theRank = 0; theRank < theLength; theRank++ ) {
	    switch ( aString.charAt(theRank) ) {
	    case ' ':
	    case '\t':
	    case '\n':
	    case '\r': {
		break;
	    }
	    default: {
		return false;
	    }
	    }
	}
	return true;
    }

    static AST decodeFigueTypeFromXml(ASTFactory aFactory,
				      String aText) {
	final int theType;
	String theName = aText.toUpperCase();
	if ( TAGS.containsKey(theName) ) {
	    theType = ((Integer)(TAGS.get(theName))).intValue();
	}
	else {
	    theType = figueTokensTokenTypes.GLYPH;
	}
	
	return aFactory.create(theType,aText);
    }

    final static AST decodeFigueAttributesFromXml(ASTFactory aFactory,
						  TokenWithAttributes aToken) {
	if ( ! aToken.hasAttributes() ) {
	    return null;
	}
	else {
	    final Attributes theAttributes = aToken.getAttributes();
	    Enumeration theNames = aToken.getAttributeNames();
	    String theCurrentName;
	    final AST theList = aFactory.create(figueTokensTokenTypes.ATTRIBUTE_LIST,"A:");
	    AST[] thePair = { null, null };
	    while ( theNames.hasMoreElements() ) {
		theCurrentName = (String)(theNames.nextElement());
		theList.addChild(decodeAttribute(aFactory,theCurrentName,theAttributes.get(theCurrentName)));
	    }
	    aToken.clear();
	    return theList;
	}
    }
    
    private final static AST decodeAttribute(ASTFactory aFactory,
					     String aName, String aValue) {
	final String theName = aName.toUpperCase();
	if ( ATTRIBUTES.containsKey(theName) ) {
	    return aFactory.create(((Integer)(ATTRIBUTES.get(theName))).intValue(),aValue);
	}
	else {
	    PAIR[0] = aFactory.create(figueTokensTokenTypes.STRING,aName);
	    PAIR[1] = aFactory.create(figueTokensTokenTypes.STRING,aValue);
	    return aFactory.make(PAIR);
	}
    }

    final static boolean newFont(FacadeInterface aFacade,
				 String aName,String aFamily,String aStyle,String aSize, String anUnderlinedFlag) {
	try {
	    final boolean theFlag;
	    if ( anUnderlinedFlag.equalsIgnoreCase("true") ) {
		theFlag = true;
	    }
	    else {
		if ( anUnderlinedFlag.equalsIgnoreCase("false") ) {
		    theFlag = false;
		}
		else {
		    return false;
		}
	    }
	    final FontFamily theFamily = FontFamily.valueOf(aFamily);
	    final FontStyle theStyle = FontStyle.valueOf(aStyle);
	    if ( ( aName != null ) && ( theFamily != null ) && ( theStyle != null ) ) {
		if ( ! aFacade.isFontDefined(aName) ) {
		    aFacade.doGetFont(aName,theFamily,theStyle,Integer.parseInt(aSize),theFlag);
		}
		return true;
	    }
	}
	catch ( NumberFormatException anException ) {
	}
	catch ( WrongResourceException anException ) {
	}
	return false;
    }

    final static boolean newColor(FacadeInterface aFacade,
				  String aName,String aRed,String aGreen,String aBlue) {
	try {
	    if ( aName != null ) {
		if ( ! aFacade.isColorDefined(aName) ) {
		    aFacade.doGetColor(aName,Integer.parseInt(aRed),Integer.parseInt(aGreen),Integer.parseInt(aBlue));
		}
		return true;
	    }
	}
	catch ( NumberFormatException anException ) {
	}
	catch ( WrongResourceException anException ) {
	}
	return false;
    }

    final static boolean newStyle(FacadeInterface aFacade,
				  String aName,String aFont,String aColor,String aBackground) {
	try {
	    if ( aName != null ) {
		if ( ! aFacade.isStyleDefined(aName) ) {
		    final FontInterface theFont;
		    if ( aFont != null ) {
			theFont = aFacade.getFont(aFont);
		    }
		    else {
			theFont = null;
		    }
		    final ColorInterface theColor;
		    if ( aColor != null ) {
			theColor = aFacade.getColor(aColor);
		    }
		    else {
			theColor = null;
		    }
		    final ColorInterface theBackground;
		    if ( aBackground != null ) {
			theBackground = aFacade.getColor(aBackground);
		    }
		    else {
			theBackground = null;
		    }
		    aFacade.doGetStyle(aName,theFont,theColor,theBackground);
		}
		return true;
	    }
	}
	catch ( WrongResourceException anException ) {
	    anException.printStackTrace();
	}
	return false;
    }

    final static boolean newSelection(FacadeInterface aFacade,
				      String aName, String aPriority, String aColor, String aBackground, String aVisibleFlag) {
	try {
	    if ( aName != null ) {
		final boolean theFlag;
		if ( aVisibleFlag.equalsIgnoreCase("true") ) {
		    theFlag = true;
		}
		else {
		    if ( aVisibleFlag.equalsIgnoreCase("false") ) {
			theFlag = false;
		    }
		    else {
			return false;
		    }
		}
		final ColorInterface theColor;
		if ( aColor != null ) {
		    theColor = aFacade.getColor(aColor);
		}
		else {
		    theColor = null;
		}
		final ColorInterface theBackground;
		if ( aBackground != null ) {
		    theBackground = aFacade.getColor(aBackground);
		}
		else {
		    theBackground = null;
		}
		final Priority thePriority = Priority.valueOf(aPriority);
		if ( thePriority != null ) {
		    if ( ! aFacade.isSelectionDefined(aName) ) {
			aFacade.doGetSelection(aName,thePriority,theColor,theBackground).setVisible(theFlag);
			return true;
		    }
		}
	    }
	}
	catch ( WrongResourceException anException ) {
	}
	catch ( WrongSelectionException anException ) {
	}
        return false;
    }

    final static GlyphInterface newHorizontal(VectorGlyph aList, String aValue, String anUnit) {
	try {
	    final GlyphInterface theResult = new Horizontal(new Measure(decodeValue(aValue,10),decodeUnit(anUnit,UnitInterface.BLANK_TENTH)));
	    addChildren(theResult,aList);
	    return theResult;
	}
	catch ( NumberFormatException anException ) {
	}
	catch ( Throwable anException) {
	}
	return null;
    }

    final static GlyphInterface newVertical(VectorGlyph aList, String aValue1, String anUnit1, String aValue2, String anUnit2) {
	try {
	    final GlyphInterface theResult = new Vertical(new Measure(decodeValue(aValue1,0),decodeUnit(anUnit1,UnitInterface.BLANK_TENTH)),new Measure(decodeValue(aValue2,0),decodeUnit(anUnit2,UnitInterface.BLANK_TENTH)));
	    addChildren(theResult,aList);
	    return theResult;
	}
	catch ( NumberFormatException anException ) {
	}
	catch ( Throwable anException) {
	}
	return null;
    }

    final static GlyphInterface newParagraph(VectorGlyph aList, String aValue0, String anUnit0, String aValue1, String anUnit1, String aValue2, String anUnit2) {
	try {
	    final GlyphInterface theResult = new Paragraph(new Measure(decodeValue(aValue0,10),decodeUnit(anUnit0,UnitInterface.BLANK_TENTH)),new Measure(decodeValue(aValue1,0),decodeUnit(anUnit1,UnitInterface.BLANK_TENTH)),new Measure(decodeValue(aValue2,0),decodeUnit(anUnit2,UnitInterface.BLANK_TENTH)));
	    addChildren(theResult,aList);
	    return theResult;
	}
	catch ( NumberFormatException anException ) {
	}
	catch ( Throwable anException) {
	}
	return null;
    }

    final static GlyphInterface newChangeContext(GlyphInterface aGlyph, String aName) {
	final GlyphInterface theResult = new ChangeGraphicalContextNamed(aName);
	addChild(theResult,aGlyph);
	return theResult;
    }

    final static GlyphInterface newChangeContext(GlyphInterface aGlyph, String aName1, String aName2, String aName3) {
	final GlyphInterface theResult = new ChangeGraphicalContextAnonymous(aName1,aName2,aName3);
	addChild(theResult,aGlyph);
	return theResult;
    }

    final static GlyphInterface newInsets(GlyphInterface aGlyph, String aValue0, String anUnit0, String aValue1, String anUnit1, String aValue2, String anUnit2, String aValue3, String anUnit3) {
	try {
	    final GlyphInterface theResult = new Insets(new Measure(decodeValue(aValue0,0),decodeUnit(anUnit0,UnitInterface.BLANK_TENTH)),new Measure(decodeValue(aValue1,0),decodeUnit(anUnit1,UnitInterface.BLANK_TENTH)),new Measure(decodeValue(aValue2,0),decodeUnit(anUnit2,UnitInterface.BLANK_TENTH)),new Measure(decodeValue(aValue3,0),decodeUnit(anUnit3,UnitInterface.BLANK_TENTH)));
	    addChild(theResult,aGlyph);
	    return theResult;
	}
	catch ( NumberFormatException anException ) {
	}
	catch ( Throwable anException) {
	}
	return null;
    }

    final static GlyphInterface newKeyWord(String aText, String aName) {
	return Atom.newAtom(aText,aName);
    }

    final static GlyphInterface newHyperlink(GlyphInterface aGlyph, String aValue0, String aValue1, String aValue2, String aValue3) {
	final GlyphInterface theResult = new Hyperlink(null,null,aValue0,aValue1,aValue2,aValue3); // TEMPO INVALID HISTORY AND INFO 
	addChild(theResult,aGlyph);
	return theResult;
    }

    private final static UnitInterface decodeUnit(String aName,UnitInterface aDefault) throws Throwable {
	if ( aName == null ) {
	    return aDefault;
	}
	else {
	    if ( aName.equalsIgnoreCase(UnitInterface.PIXEL.getName()) ) {
		return UnitInterface.PIXEL;
	    }
	    else {
		if ( aName.equalsIgnoreCase(UnitInterface.BLANK_TENTH.getName()) ) {
		    return UnitInterface.BLANK_TENTH;
		}
		else {
		    if ( aName.equalsIgnoreCase(UnitInterface.LINE_TENTH.getName()) ) {
			return UnitInterface.LINE_TENTH;
		    }
		    else {
			throw new Throwable();
		    }
		}
	    }
	}
    }

    final static String getEntity(String aName) {
	return (String)(ENTITY.get(aName.toUpperCase()));
    }

    final static String charReference(String aValue,int aRadix) {
	try {
	    final char r = (char)(Integer.parseInt(aValue,aRadix));
	    return String.valueOf(r);
	}
	catch ( NumberFormatException  anException ) {
	}
	return String.valueOf('\u0000');
    }

    private final static int decodeValue(String aValue,int aDefault) throws NumberFormatException {
	if ( aValue == null ) {
	    return aDefault;
	}
	else {
	    return Integer.parseInt(aValue);
	}
    }

    private final static void addChild(GlyphInterface aGlyph, GlyphInterface aChild) {
	if ( aChild != null ) {
	    aGlyph.addChild(aChild);
	}
    }

    private final static void addChildren(GlyphInterface aGlyph, VectorGlyph aList) {
	if ( aList != null ) {
	    final int theSize = aList.size();
	    for ( int theRank = 0; theRank < theSize; theRank++ ) {
		aGlyph.addChild(aList.elementAt(theRank));
	    }
	}
    }

    private final static AST[] PAIR = { null, null }; /// TEMPO NO STATIC
    private final static Hashtable ENTITY = new Hashtable();
    private final static Hashtable TAGS = new Hashtable();
    private final static Hashtable ATTRIBUTES = new Hashtable();
    
    static {
	ENTITY.put("LT","<");
	ENTITY.put("GT",">");
	ENTITY.put("AMP","&");
	ENTITY.put("APOS","'");
	ENTITY.put("QUOT","\"");
	ENTITY.put("WS"," ");
	ENTITY.put("LN","\n");
	ENTITY.put("TAB","\t");
	TAGS.put("FACADE",new Integer(figueTokensTokenTypes.FACADE));
	TAGS.put("SELECTIONS",new Integer(figueTokensTokenTypes.SELECTIONS));
	TAGS.put("SELECTION",new Integer(figueTokensTokenTypes.SELECTION));
	TAGS.put("RESOURCES",new Integer(figueTokensTokenTypes.RESOURCES));
	TAGS.put("COLORS",new Integer(figueTokensTokenTypes.COLORS));
	TAGS.put("COLOR",new Integer(figueTokensTokenTypes.COLOR));
	TAGS.put("FONTS",new Integer(figueTokensTokenTypes.FONTS));
	TAGS.put("FONT",new Integer(figueTokensTokenTypes.FONT));
	TAGS.put("STYLES",new Integer(figueTokensTokenTypes.STYLES));
	TAGS.put("STYLE",new Integer(figueTokensTokenTypes.STYLE));
	TAGS.put("ROOT",new Integer(figueTokensTokenTypes.ROOT));
	TAGS.put("V",new Integer(figueTokensTokenTypes.VERTICAL));
	TAGS.put("H",new Integer(figueTokensTokenTypes.HORIZONTAL));
	TAGS.put("P",new Integer(figueTokensTokenTypes.PARAGRAPH));
	TAGS.put("CONTEXT",new Integer(figueTokensTokenTypes.CHANGE_CONTEXT));
	TAGS.put("INSETS",new Integer(figueTokensTokenTypes.INSETS));
	TAGS.put("KEYWORD",new Integer(figueTokensTokenTypes.KEYWORD));
	TAGS.put("HYPERLINK",new Integer(figueTokensTokenTypes.HYPERLINK));
	TAGS.put("PATH",new Integer(figueTokensTokenTypes.PATH));
	TAGS.put("INSERTGLYPH",new Integer(figueTokensTokenTypes.INSERT_GLYPH));
	TAGS.put("REMOVEGLYPH",new Integer(figueTokensTokenTypes.REMOVE_GLYPH));
	TAGS.put("REPLACEGLYPH",new Integer(figueTokensTokenTypes.REPLACE_GLYPH));
	TAGS.put("TOUCHGLYPH",new Integer(figueTokensTokenTypes.TOUCH_GLYPH));
	TAGS.put("MARKGLYPH",new Integer(figueTokensTokenTypes.MARK_GLYPH));
	TAGS.put("MARKSTABLE",new Integer(figueTokensTokenTypes.MARK_STABLE));
	TAGS.put("MARKUNSTABLE",new Integer(figueTokensTokenTypes.MARK_UNSTABLE));
	TAGS.put("EXTENDSELECTION",new Integer(figueTokensTokenTypes.EXTEND_SELECTION));
	TAGS.put("INVERSESELECTION",new Integer(figueTokensTokenTypes.INVERSE_SELECTION));
	TAGS.put("SHRINKSELECTION",new Integer(figueTokensTokenTypes.SHRINK_SELECTION));
	ATTRIBUTES.put("NAME",new Integer(figueTokensTokenTypes.A_NAME));
	ATTRIBUTES.put("RED",new Integer(figueTokensTokenTypes.A_RED));
	ATTRIBUTES.put("GREEN",new Integer(figueTokensTokenTypes.A_GREEN));
	ATTRIBUTES.put("BLUE",new Integer(figueTokensTokenTypes.A_BLUE));
	ATTRIBUTES.put("SIZE",new Integer(figueTokensTokenTypes.A_SIZE));
	ATTRIBUTES.put("STYLE",new Integer(figueTokensTokenTypes.A_STYLE));
	ATTRIBUTES.put("FAMILY",new Integer(figueTokensTokenTypes.A_FAMILY));
	ATTRIBUTES.put("UNDERLINED",new Integer(figueTokensTokenTypes.A_UNDERLINED));
	ATTRIBUTES.put("FONT",new Integer(figueTokensTokenTypes.A_FONT));
	ATTRIBUTES.put("FOREGROUND",new Integer(figueTokensTokenTypes.A_FOREGROUND));
	ATTRIBUTES.put("BACKGROUND",new Integer(figueTokensTokenTypes.A_BACKGROUND));
	ATTRIBUTES.put("UNIT",new Integer(figueTokensTokenTypes.A_UNIT));
	ATTRIBUTES.put("INDENT",new Integer(figueTokensTokenTypes.A_INDENT));
	ATTRIBUTES.put("INTERWORD",new Integer(figueTokensTokenTypes.A_INTERWORD));
	ATTRIBUTES.put("INTERLINE",new Integer(figueTokensTokenTypes.A_INTERLINE));
	ATTRIBUTES.put("VISIBLE",new Integer(figueTokensTokenTypes.A_VISIBLE));
	ATTRIBUTES.put("PRIORITY",new Integer(figueTokensTokenTypes.A_PRIORITY));
	ATTRIBUTES.put("SPACETOLEFT",new Integer(figueTokensTokenTypes.A_SPACE_TO_LEFT));
	ATTRIBUTES.put("SPACETORIGHT",new Integer(figueTokensTokenTypes.A_SPACE_TO_RIGHT));
	ATTRIBUTES.put("SPACEBEFORE",new Integer(figueTokensTokenTypes.A_SPACE_BEFORE));
	ATTRIBUTES.put("SPACEAFTER",new Integer(figueTokensTokenTypes.A_SPACE_AFTER));
	ATTRIBUTES.put("TEXT",new Integer(figueTokensTokenTypes.A_TEXT));
	ATTRIBUTES.put("ACTIVECOLOR",new Integer(figueTokensTokenTypes.A_ACTIVE_COLOR));
	ATTRIBUTES.put("VISITEDCOLOR",new Integer(figueTokensTokenTypes.A_VISITED_COLOR));
	ATTRIBUTES.put("SELECTEDCOLOR",new Integer(figueTokensTokenTypes.A_SELECTED_COLOR));
	ATTRIBUTES.put("RANK",new Integer(figueTokensTokenTypes.A_RANK));
    }
    
}
