//*****************************************************************************
//   Copyright (c) 1998 Bernard Sufrin, Oxford.
//   http://www.comlab.ox.ac.uk/oucl/people/bernard.sufrin.html
//*****************************************************************************

package figue.jape;

import  java.awt.image.*;
import  java.awt.Color;
import  java.awt.Component;
/**
        Utility for calculating indexed color models for
        use in rendering scaled bitmap fonts.
        
*/
public class ColorModel { 

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/jape/ColorModel.java,v $ $revision$ $Date: 1998/11/02 15:55:22 $ Copyright 1998, Bernard Sufrin, Oxford.";
   
    /**
       Generate either a linearly interpolated
       indexed color model, or a bitmap.
    */
        
    public static IndexColorModel range(int bits, int scale, Color fgd, Color bgd )
    { if (bits!=8) 
	{
            byte fgRed   = (byte)fgd.getRed( );
            byte fgGreen = (byte)fgd.getGreen( );
            byte fgBlue  = (byte)fgd.getBlue( );

            byte bgRed   = (byte)bgd.getRed( );
            byte bgGreen = (byte)bgd.getGreen( );
            byte bgBlue  = (byte)bgd.getBlue( );
            
            byte red  [ ] = new byte[ ] { bgRed,   fgRed   };
            byte green[ ] = new byte[ ] { bgGreen, fgGreen };
            byte blue [ ] = new byte[ ] { bgBlue,  fgBlue  };
            return new IndexColorModel(1, 2, red, green, blue, 0);
	}
    else return range(scale, fgd, bgd);
    }
        
    /**
       Generate a linearly interpolated (between
       <TT>fg</TT> and <TT>bg</TT>) indexed color
       model suitable for use with a linear
       sampling scale of <tt>scale</tt>.
    */
    public static IndexColorModel range(int scale, Color fgd, Color bgd ) 
    {
	int range = scale*scale;
	int size  = range + 1;
	int mid   = range / 2;

	byte red  [ ] = new byte[ size ];
	byte green[ ] = new byte[ size ];
	byte blue [ ] = new byte[ size ];

	int fgRed   = fgd.getRed( );
	int fgGreen = fgd.getGreen( );
	int fgBlue  = fgd.getBlue( );

	int bgRed   = bgd.getRed( );
	int bgGreen = bgd.getGreen( );
	int bgBlue  = bgd.getBlue( );

	for( int fg = 0; fg < size; ++ fg ) 
	    {
		int bg = range - fg;

		red  [ fg ] = ( byte )(( fg * fgRed   + bg * bgRed   + mid ) / range );
		green[ fg ] = ( byte )(( fg * fgGreen + bg * bgGreen + mid ) / range );
		blue [ fg ] = ( byte )(( fg * fgBlue  + bg * bgBlue  + mid ) / range );
	    }

	return new IndexColorModel(8, size, red, green, blue, 0);
    }
    
    public static Color interpolate(Color fgd, Color bgd, float ratio)
    {    float s = (1.0F-ratio)/256.0F;
    float r = ratio/256.0F;
         
    int fR = fgd.getRed( );
    int fG = fgd.getGreen( );
    int fB = fgd.getBlue( );

    int bR = bgd.getRed( );
    int bG = bgd.getGreen( );
    int bB = bgd.getBlue( );
         
    Color res = new Color((float)(fR*r+bR*s), (float)(fG*r+bG*s), (float)(fB*r+bB*s));
    return res;
    }
        
    public static ColorScheme invert(final ColorScheme g)
    { 
	return new ColorScheme()
	    { public Color getForeground() { return g.getBackground(); }
	    public Color getBackground() { return g.getForeground(); }
	    };
    }
    
    public static ColorScheme disable(final ColorScheme g)
    { // only give a nontrivial answer when the base scheme is nontrivial
	return new ColorScheme()
	    { final public Color getForeground() 
		{ if (g.getForeground()==null || g.getBackground()==null) 
		    return null;
		else 
		    return interpolate(g.getForeground(), g.getBackground(), 0.4F); 
		}
	    final public Color getBackground() 
		{ return g.getBackground(); }
	    };
    }
    
    public static ColorScheme from(final ColorScheme g, final Color fg)
    { 
	return new ColorScheme()
	    { final public Color getForeground() { return fg; }
	    final public Color getBackground() { return g.getBackground(); }
	    };
    }
    
    public static ColorScheme from(final Color fg, final Color bg)
    { 
	return new ColorScheme()
	    { final public Color getForeground() { return fg; }
	    final public Color getBackground() { return bg; }
	    };
    }
    
    public static ColorScheme from(final Component c)
    { 
	return new ColorScheme()
	    { final public Color getForeground() { return c.getForeground(); }
	    final public Color getBackground() { return c.getBackground(); }
	    };
    }
}










