//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.multithread;

final class ThreadState implements ThreadStateInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/multithread/ThreadState.java,v $ $revision$ $Date: 1998/11/02 15:55:27 $ Copyright 1998 INRIA.";
    
    ThreadState(String aName,
		boolean aQuitFlag,
		boolean anInactiveFlag,
		boolean aFormattableFlag,
		boolean aFormattingFlag,
		boolean aDrawableFlag,
		boolean aDrawingFlag,
		boolean anUpdatingGlyphFlag,
		boolean aBuildingFlag,
		boolean aBuildingPhaseFlag) {
	_name = aName;
	_quitFlag = aQuitFlag;
	_inactiveFlag = anInactiveFlag;
	_formattableFlag = aFormattableFlag;
	_formattingFlag = aFormattingFlag;
	_drawableFlag = aDrawableFlag;
	_drawingFlag = aDrawingFlag;
	_updatingGlyphFlag = anUpdatingGlyphFlag;
	_buildingFlag = aBuildingFlag;
	_buildingPhaseFlag = aBuildingPhaseFlag;
    }

    public final boolean isDone() {
	return _quitFlag;
    }

    public final boolean isInactive() throws StoppedFacadeException {
	return _inactiveFlag;
    }

    public final boolean isFormattable() throws StoppedFacadeException {
	return _formattableFlag;
    }
  
    public final boolean isFormatting() throws StoppedFacadeException {
	return _formattingFlag;
    }
  
    public final void startFormat(NotifyingThreadStateInterface aNotifyingThreadState) {
	aNotifyingThreadState.changeState(_stateAfterStartFormat);
    }
  
    public final void endFormat(NotifyingThreadStateInterface aNotifyingThreadState) {
	aNotifyingThreadState.setState(_stateAfterEndFormat);
    }

    public final void suspendFormat(NotifyingThreadStateInterface aNotifyingThreadState) {
	aNotifyingThreadState.setState(_stateAfterSuspendFormat);
    }

    public final boolean isDrawable() throws StoppedFacadeException {
	return _drawableFlag;
    }

    public final boolean isDrawing() throws StoppedFacadeException {
	return _drawingFlag;
    }

    public final void startDraw(NotifyingThreadStateInterface aNotifyingThreadState) {
	aNotifyingThreadState.changeState(_stateAfterStartDraw);
    }
  
    public final void endDraw(NotifyingThreadStateInterface aNotifyingThreadState) {
	aNotifyingThreadState.setState(_stateAfterEndDraw);
    }

    public final boolean isUpdatingGlyph() throws StoppedFacadeException {
	return _updatingGlyphFlag;
    }

    public final void startUpdateGlyph(NotifyingThreadStateInterface aNotifyingThreadState) {
	aNotifyingThreadState.changeState(_stateAfterStartUpdateGlyph);
    }

    public final void endUpdateGlyph(NotifyingThreadStateInterface aNotifyingThreadState) {
	aNotifyingThreadState.setState(_stateAfterEndUpdateGlyph);
    }

    public final boolean isBuilding() throws StoppedFacadeException {
	return _buildingFlag;
    }

    public final boolean isBuildingPhase() throws StoppedFacadeException {
	return _buildingPhaseFlag;
    }

    public final void startBuild(NotifyingThreadStateInterface aNotifyingThreadState) {
	aNotifyingThreadState.changeState(_stateAfterStartBuild);
    }

    public final void endBuild(NotifyingThreadStateInterface aNotifyingThreadState) {
	aNotifyingThreadState.setState(_stateAfterEndBuild);
    }

    public final void quit(NotifyingThreadStateInterface aNotifyingThreadState) {
	aNotifyingThreadState.setState(_stateAfterQuit);
    }

    public final void init(NotifyingThreadStateInterface aNotifyingThreadState) {
	aNotifyingThreadState.setState(_stateAfterInit);
    }
    public final void fatalError(NotifyingThreadStateInterface aNotifyingThreadState) {
	aNotifyingThreadState.setState(_stateAfterFatalError);
    }

    private final void initStatesToInvalid() {
	_stateAfterStartFormat = InvalidThreadState.INSTANCE;
	_stateAfterEndFormat = InvalidThreadState.INSTANCE;
	_stateAfterSuspendFormat = InvalidThreadState.INSTANCE;
	_stateAfterStartDraw = InvalidThreadState.INSTANCE;
	_stateAfterEndDraw = InvalidThreadState.INSTANCE;
	_stateAfterStartUpdateGlyph = InvalidThreadState.INSTANCE;
	_stateAfterEndUpdateGlyph = InvalidThreadState.INSTANCE;
	_stateAfterStartBuild = InvalidThreadState.INSTANCE;
	_stateAfterEndBuild = InvalidThreadState.INSTANCE;
	_stateAfterQuit = InvalidThreadState.INSTANCE;
	_stateAfterInit = InvalidThreadState.INSTANCE;
	_stateAfterFatalError = InvalidThreadState.INSTANCE; /// TEMPO
    }
    
    public final String toString() {
	StringBuffer theResult = new StringBuffer("State<");
	theResult.append(_name).append(">");
	return theResult.toString();
    }

    private final String _name;
    private final boolean _inactiveFlag;
    private final boolean _formattableFlag;
    private final boolean _formattingFlag;
    private final boolean _drawableFlag;
    private final boolean _drawingFlag;
    private final boolean _updatingGlyphFlag;
    private final boolean _buildingFlag;
    private final boolean _buildingPhaseFlag;
    private final boolean _quitFlag;
    private ThreadStateInterface _stateAfterStartFormat;
    private ThreadStateInterface _stateAfterEndFormat;
    private ThreadStateInterface _stateAfterSuspendFormat;
    private ThreadStateInterface _stateAfterStartDraw;
    private ThreadStateInterface _stateAfterEndDraw;
    private ThreadStateInterface _stateAfterStartUpdateGlyph;
    private ThreadStateInterface _stateAfterEndUpdateGlyph;
    private ThreadStateInterface _stateAfterStartBuild;
    private ThreadStateInterface _stateAfterEndBuild;
    private ThreadStateInterface _stateAfterQuit;
    private ThreadStateInterface _stateAfterInit;
    private ThreadStateInterface _stateAfterFatalError;

    protected final static ThreadState START_BUILD;
    protected final static ThreadState BUILD;
    protected final static ThreadState DRAW;
    protected final static ThreadState DRAW_AND_FORMAT;
    protected final static ThreadState DRAW_AND_FORMAT_AND_BUILD;
    protected final static ThreadState FORMAT_SUSPENDED;
    protected final static ThreadState FORMAT;
    protected final static ThreadState FORMAT_AND_BUILD;
    protected final static ThreadState FORMAT_SUSPENDED_AND_BUILD;
    protected final static ThreadState INACTIVE;
    protected final static ThreadState INIT;

    static {
	START_BUILD = new ThreadState("Start building",
				      false, // done ?
				      false, // inactive ?
				      false, // formattable ?
				      false, // formatting ?
				      false, // drawable ?
				      false, // drawing ?
				      true, // updating glyph ?
				      false, // building ?
				      true); // building  phase
	START_BUILD.initStatesToInvalid();
	BUILD = new ThreadState("Building",
				false, // done ?
				false, // inactive ?
				false, // formattable ?
				false, // formatting ?
				false, // drawable ?
				false, // drawing ?
				false, // updating glyph ?
				true, // building ?
				true); // building  phase
	BUILD.initStatesToInvalid();
	DRAW = new ThreadState("Drawing",
			       false, // done ?
			       false, // inactive ?
			       false, // formattable ?
			       false, // formatting ?
			       false, // drawable ?
			       true, // drawing ?
			       false, // updating glyph ?
			       false, // building ?
			       false); // building  phase
	DRAW.initStatesToInvalid();
	DRAW_AND_FORMAT = new ThreadState("Drawing and formatting",
					  false, // done ?
					  false, // inactive ?
					  false, // formattable ?
					  false, // formatting ?
					  false, // drawable ?
					  true, // drawing ?
					  false, // updating glyph ?
					  false, // building ?
					  true); // building  phase
	DRAW_AND_FORMAT.initStatesToInvalid();
	DRAW_AND_FORMAT_AND_BUILD = new ThreadState("Drawing and formatting and building glyphs",
						    false, // done ?
						    false, // inactive ?
						    false, // formattable ?
						    false, // formatting ?
						    false, // drawable ?
						    true, // drawing ?
						    false, // updating glyph ?
						    true, // building ?
						    true); // building  phase
	DRAW_AND_FORMAT_AND_BUILD.initStatesToInvalid();
	
	
	FORMAT_SUSPENDED = new ThreadState("Formatting suspended",
					   false, // done ?
					   false, // inactive ?
					   false, // formattable ?
					   false, // formatting ?
					   true, // drawable ?
					   false, // drawing ?
					   false, // updating glyph ?
					   false, // building ?
					   true); // building  phase
	FORMAT_SUSPENDED.initStatesToInvalid();
	FORMAT_SUSPENDED_AND_BUILD = new ThreadState("Formatting suspended and building glyphs",
						     false, // done ?
						     false, // inactive ?
						     false, // formattable ?
						     false, // formatting ? 
						     true, // drawable ?
						     false, // drawing ?
						     false, // updating glyph ?
						     true, // building ?
						     true); // building  phase
	FORMAT_SUSPENDED_AND_BUILD.initStatesToInvalid();
	
	FORMAT = new ThreadState("Formatting",
				 false, // done ?
				 false, // inactive ?
				 false, // formattable ?
				 true, // formatting ?
				 false, // drawable ?
				 false, // drawing ?
				 false, // updating glyph ?
				 false, // building ?
				 true); // building  phase
	FORMAT.initStatesToInvalid();
	FORMAT_AND_BUILD = new ThreadState("Formatting and building glyphs",
					   false, // done ?
					   false, // inactive ?
					   false, // formattable ?
					   true, // formatting ?
					   false, // drawable ?
					   false, // drawing ?
					   false, // updating glyph ?
					   true, // building ?
					   true); // building  phase
	FORMAT_AND_BUILD.initStatesToInvalid();
	INACTIVE = new ThreadState("Inactive",
				   false, // done ?
				   true, // inactive ?
				   false, // formattable ?
				   false, // formatting ?
				   true, // drawable ?
				   false, // drawing ?
				   false, // updating glyph ?
				   false, // building ?
				   false); // building  phase
	INACTIVE.initStatesToInvalid();
	INIT = new ThreadState("Init",
			       false, // done ?
			       false, // inactive ?
			       false, // formattable ?
			       false, // formatting ?
			       false, // drawable ?
			       false, // drawing ?
			       false, // updating glyph ?
			       false, // building ?
			       false); // building  phase
	INIT.initStatesToInvalid();
	
	BUILD._stateAfterEndBuild = INACTIVE;
	
	DRAW._stateAfterEndDraw = INACTIVE;
	
	DRAW_AND_FORMAT._stateAfterEndDraw = FORMAT;
	
	FORMAT_SUSPENDED._stateAfterStartDraw = DRAW_AND_FORMAT;
	
	FORMAT_SUSPENDED_AND_BUILD._stateAfterStartDraw = DRAW_AND_FORMAT_AND_BUILD;
	FORMAT_SUSPENDED_AND_BUILD._stateAfterEndBuild = FORMAT_SUSPENDED;
	
	DRAW_AND_FORMAT_AND_BUILD._stateAfterEndBuild = DRAW_AND_FORMAT;
	DRAW_AND_FORMAT_AND_BUILD._stateAfterEndDraw = FORMAT_AND_BUILD;
	
	FORMAT._stateAfterEndFormat = INACTIVE;
	FORMAT._stateAfterSuspendFormat = FORMAT_SUSPENDED;
	
	FORMAT_AND_BUILD._stateAfterEndBuild = FORMAT;
	FORMAT_AND_BUILD._stateAfterEndFormat = BUILD;
	FORMAT_AND_BUILD._stateAfterSuspendFormat = FORMAT_SUSPENDED_AND_BUILD;
	
	INACTIVE._stateAfterStartDraw = DRAW;
	INACTIVE._stateAfterStartUpdateGlyph = START_BUILD;
	INACTIVE._stateAfterStartFormat = FORMAT;
	INACTIVE._stateAfterQuit = ThreadStateDone.INSTANCE;
	
	INIT._stateAfterInit = INACTIVE;
	
	START_BUILD._stateAfterEndUpdateGlyph = FORMAT_AND_BUILD;
	
    }
    
}
