//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.path;

import figue.internal.PathVisitorInterface;
import figue.box.GlyphInterface;

import java.util.Vector;
import java.io.Writer;
import java.io.IOException;

/**
 * [PENDING: update the comments] LinearPath.
 */

public final class LinearPath implements PathInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/path/LinearPath.java,v $ $revision$ $Date: 1998/11/02 15:55:29 $ Copyright 1998 INRIA.";
  
    public LinearPath(int aRank) {
	super();
	_rank = aRank;
	_operations = new Vector(0,1);
	_child = null;
    }

    public LinearPath() {
	this(0);
    }
  
    public LinearPath (int[] r) {
	this(0);
	LinearPath res = null;
	LinearPath res1 = null;
	for (int i = 0; i < r.length; i++) {
	    res1 = new LinearPath(r[i]-1);
	    if (res == null)
		addChild(res1);
	    else
		res.addChild(res1);
	    res = res1;
	}
    }

    public final int getRank() { 
	return _rank; 
    }

    public final void setRank(int aValue) {
	_rank = aValue;
    }

    public final void addOperation(PathOperationInterface anOperation) {
	_operations.addElement(anOperation);
    }
  
    public final PathOperationInterface getOperation(int aRank) {
	return (PathOperationInterface)(_operations.elementAt(aRank));
    }

    public final void addChild(PathInterface aChild) {
	_child = aChild;
    }

    public final PathInterface getChild(int aRank) {
	return _child;
    }

    public final void insertChild(PathInterface aChild, int aRank) {
	_child = aChild;
    }

    public final void accept(PathVisitorInterface aVisitor,
			     GlyphInterface aGlyph,
			     SelectionPathInterface aSelectionPath) {
	final int theSize = numberOfOperation();
	final int theOtherSize = numberOfChildren();
	for( int theRank = 0; theRank < theSize; theRank++ ) {
	    getOperation(theRank).accept(aVisitor,aGlyph,aSelectionPath);
	}
	if ( _child != null ) {
	    aVisitor.down(aGlyph,aSelectionPath,_child);
	}
    }

    public final void accept(PathVisitorInterface aVisitor,
			     GlyphInterface aGlyph) {
	final int theSize = numberOfOperation();
	for( int theRank = 0; theRank < theSize; theRank++ ) {
	    getOperation(theRank).accept(aVisitor,aGlyph);
	}
	if ( _child != null ) {
	    aVisitor.down(aGlyph,_child);
	}
    }
  
    public final int numberOfChildren() {
	if ( _child == null ) {
	    return 0;
	}
	else {
	    return 1;
	}
    }

    public final int numberOfOperation() {
	return _operations.size();
    }


    public final String toString() {
	StringBuffer theResult = new StringBuffer("LinearPath<\n");
	internalToString(theResult,"");
	theResult.append(">");
	return theResult.toString();
    }

    public final void internalToString(StringBuffer aBuffer,
				       String aPath) {
	aBuffer.append("  - ").append(aPath).append(".").append(_rank)
	    .append(": {");
	final int theSize = numberOfOperation();
	boolean theFirstFlag = true;
	for( int theRank = 0; theRank < theSize; theRank++ ) {
	    if ( theFirstFlag ) {
		theFirstFlag = false;
	    }
	    else {
		aBuffer.append(", ");
	    }
	    aBuffer.append(getOperation(theRank).toString());
	}
	aBuffer.append("}\n");
	final String thePath = aPath + "." + String.valueOf(_rank);
	final int theOtherSize = numberOfChildren();
	for( int theRank = 0; theRank < theOtherSize; theRank++ ) {
	    getChild(theRank).internalToString(aBuffer,thePath);
	}
    }

    public final void writeAsXML(Writer anOutput) throws IOException {
	anOutput.write("<?XML version=\"1.0\"?>");
	writeState(anOutput);
    }

    public final void writeState(Writer anOutput) throws IOException {
	anOutput.write("<Path Rank=\"");
	anOutput.write(Integer.toString(_rank));
	anOutput.write("\">");
	final int theSize = numberOfOperation();
	if ( theSize > 0 ) {
	    for( int theRank = 0; theRank < theSize; theRank++ ) {
		getOperation(theRank).writeState(anOutput);
	    }
	}
	if ( _child != null ) {
	    _child.writeState(anOutput);
	}
	anOutput.write("</Path>");
    }
  
    private int _rank; 
    private Vector _operations;
    private PathInterface _child;

}
