//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.resource;

import figue.internal.Globals;
import figue.error.ImplementationError;
import figue.error.ErrorHandlerInterface;
import figue.custom.ConfigurationInterface;

import java.util.Hashtable;
import java.awt.Frame;
import java.net.URL;

/**
 * [PENDING: update the comments] AWTResourceFactory.
 * @level advanced
 */

public final class AWTResourceFactory extends ResourceFactory {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/resource/AWTResourceFactory.java,v $ $revision$ $Date: 1998/11/02 15:55:34 $ Copyright 1998 INRIA.";
  
    public AWTResourceFactory(ErrorHandlerInterface anErrorHandler,
			      URL aPortableFontUrl) {
	super(anErrorHandler);
	_colorRegistry = new Hashtable(Globals.AVERAGE_COLOR_NUMBER,
				       Globals.VERY_FAST_ACCESS);
	_fontRegistry = new Hashtable(Globals.AVERAGE_FONT_NUMBER,
				      Globals.FAST_ACCESS);
	_styleRegistry = new Hashtable(Globals.AVERAGE_STYLE_NUMBER,
				       Globals.VERY_FAST_ACCESS);
	_defaultStyle = null;
	_awtColorRegistry = new Hashtable(Globals.AVERAGE_COLOR_NUMBER,
					  Globals.VERY_FAST_ACCESS);
	_awtFontRegistry = new Hashtable(Globals.AVERAGE_FONT_NUMBER,
					 Globals.FAST_ACCESS);
	_portableFontRegistry =  new Hashtable(Globals.AVERAGE_FONT_NUMBER,
					       Globals.FAST_ACCESS);
	_portableFontUrl = aPortableFontUrl;
	_componentForResolution = null;
    }

    public final ColorInterface createColor(String aMnemonic,
					    BasicColor aColor)
	throws WrongResourceException {
	final java.awt.Color theColor;
	switch ( aColor.getEnum() ) {
	case BasicColor.ENUM_BLACK:{
	    theColor = java.awt.Color.black;
	    break;
	}
	case BasicColor.ENUM_WHITE:{
	    theColor = java.awt.Color.white;
	    break;
	}
	case BasicColor.ENUM_RED:{
	    theColor = java.awt.Color.red;
	    break;
	}
	case BasicColor.ENUM_GREEN:{
	    theColor = java.awt.Color.green;
	    break;
	}
	case BasicColor.ENUM_BLUE:{
	    theColor = java.awt.Color.blue;
	    break;
	}
	case BasicColor.ENUM_GRAY:{
	    theColor = java.awt.Color.gray;
	    break;
	}
	case BasicColor.ENUM_LIGHT_GRAY:{
	    theColor = java.awt.Color.lightGray;
	    break;
	}
	case BasicColor.ENUM_DARK_GRAY:{
	    theColor = java.awt.Color.darkGray;
	    break;
	}
	case BasicColor.ENUM_CYAN:{
	    theColor = java.awt.Color.cyan;
	    break;
	}
	case BasicColor.ENUM_MAGENTA:{
	    theColor = java.awt.Color.magenta;
	    break;
	}
	case BasicColor.ENUM_ORANGE:{
	    theColor = java.awt.Color.orange;
	    break;
	}
	case BasicColor.ENUM_PINK:{
	    theColor = java.awt.Color.pink;
	    break;
	}
	case BasicColor.ENUM_YELLOW:{
	    theColor = java.awt.Color.yellow;
	    break;
	}
	default: {
	    theColor = null;
	    throw new ImplementationError("Unknown basic color: " + aColor.toString());
	}
	}
	return new AWTColor(theColor,aMnemonic);
    }
  
    public final ColorInterface createColor(String aMnemonic,
					    int red, 
					    int green, 
					    int blue)
	throws WrongResourceException {
	final String theKey = computeKey(red,green,blue);
	ColorInterface theColor = (ColorInterface)(_awtColorRegistry.get(theKey));
	if ( theColor != null ) {
	    return theColor;
	}
	java.awt.Color theAwtColor = new java.awt.Color(red,green,blue);
	if ( theAwtColor == null ) {
	    throw new WrongResourceException("Can't allocate color: " +
					     String.valueOf(red) + " " +
					     String.valueOf(blue) + " " +
					     String.valueOf(green));
	}
	theColor = new AWTColor(theAwtColor,aMnemonic);
	_awtColorRegistry.put(theKey,theColor);
	return theColor;
    }

    public final FontInterface createFont(String aMnemonic,
					  FontFamily aFamily,
					  FontStyle aStyle,
					  int aPointSize,
					  boolean anUnderlinedFlag)
	throws WrongResourceException {
	final String theKey = computeKey(aFamily,aStyle,aPointSize,anUnderlinedFlag);
	FontInterface theFont = (FontInterface)(_awtFontRegistry.get(theKey));
	if ( theFont != null ) {
	    return theFont;
	}
	java.awt.Font theAwtFont = 
	    ConfigurationInterface.INSTANCE.newAwtFont(aFamily,decode(aStyle),aPointSize);
	if ( theAwtFont == null ) {
	    throw new WrongResourceException("Can't allocate font: " + 
					     aFamily + aStyle.toString() +
					     String.valueOf(aPointSize));
	}
	theFont = new AWTFont(theAwtFont,aMnemonic,theKey,aFamily,aStyle,aPointSize,anUnderlinedFlag);
	_awtFontRegistry.put(theKey,theFont);
	return theFont;
    }

    private final int decode(FontStyle aStyle) {
	switch ( aStyle.getEnum() ) {
	case FontStyle.ENUM_PLAIN: {
	    return java.awt.Font.PLAIN;
	}
	case FontStyle.ENUM_ITALIC: {
	    return java.awt.Font.ITALIC;
	}
	case FontStyle.ENUM_BOLD: {
	    return java.awt.Font.BOLD;
	}
	case FontStyle.ENUM_BOLD_ITALIC: {
	    return ( java.awt.Font.BOLD + java.awt.Font.ITALIC );
	}   
	default: {
	    throw new ImplementationError("Unknown font style: " + aStyle.toString());
	}
	}
    }
    
    protected final Hashtable getColorRegistry() {
	return _colorRegistry;
    }
    protected final Hashtable getFontRegistry() {
	return _fontRegistry;
    }
    protected final Hashtable getStyleRegistry() {
	return _styleRegistry;
    }
    
    protected final StyleInterface getDefaultStyle() {
	return _defaultStyle;
    }
    
    protected final void setDefaultStyle(StyleInterface aStyle) {
	_defaultStyle = aStyle;
    }

    protected final Hashtable getPortableFontRegistry() {
	return _portableFontRegistry;
    }
    
    public final PortableFontInterface createPortableFont(String aMnemonic,
							  String aName,
							  int aDpi,
							  double aScale)
	throws WrongResourceException {
	final String theKey = computeKey(aName,aDpi,aScale);
	PortableFontInterface theFont = (PortableFontInterface)(_portableFontRegistry.get(theKey));
	if ( theFont != null ) {
	    return theFont;
	}
	if ( _componentForResolution == null ) {
	    _componentForResolution = new Frame();
	    _componentForResolution.addNotify();
	}
	theFont = new PortableFont(_componentForResolution,_portableFontUrl,aName,aDpi,aScale);
	_portableFontRegistry.put(theKey,theFont);
	return theFont;
    }
    
    public final void dispose() {
	_colorRegistry.clear();
	_colorRegistry = null;
	_fontRegistry.clear();
	_fontRegistry = null;
	_styleRegistry.clear();
	_styleRegistry = null;
	_defaultStyle = null;
	_awtColorRegistry.clear();
	_awtColorRegistry = null;
	_awtFontRegistry.clear();
	_awtFontRegistry = null;
	_portableFontRegistry.clear();
	_portableFontRegistry = null;
	_portableFontUrl = null;
	if ( _componentForResolution != null ) {
	    _componentForResolution.dispose();
	    _componentForResolution = null;
	}
	super.dispose();
    }

    private Hashtable _colorRegistry;
    private Hashtable _fontRegistry;
    private Hashtable _styleRegistry;
    private StyleInterface _defaultStyle;
    private Hashtable _awtColorRegistry;
    private Hashtable _awtFontRegistry;
    private Hashtable _portableFontRegistry;
    private URL _portableFontUrl;
    private Frame _componentForResolution;

}

final class AWTColor extends Color {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/resource/AWTResourceFactory.java,v $ $revision$ $Date: 1998/11/02 15:55:34 $ Copyright 1998 INRIA.";
  
    public AWTColor(java.awt.Color aColor, String aMnemonic) { 
	super(aMnemonic);
	_color = aColor; 
    }
    
    public final int getRed() {
	return _color.getRed();
    }
    
    public final int getGreen() {
	return _color.getGreen();
    }
    
    public final int getBlue() {
	return _color.getBlue();
    }
    
    public final java.awt.Color getAwtColor() { 
	return _color; 
    }
    
    private java.awt.Color _color;
    
}

final class AWTFont extends Font {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/resource/AWTResourceFactory.java,v $ $revision$ $Date: 1998/11/02 15:55:34 $ Copyright 1998 INRIA.";

    public AWTFont(java.awt.Font aFont,
		   String aMnemonic,
		   String aKey,
		   FontFamily aFamily,
		   FontStyle aStyle,
		   int aPointSize,
		   boolean anUnderlinedFlag) {
	super(aMnemonic,aKey,aFamily,aStyle,aPointSize,anUnderlinedFlag);
	_font = aFont;
    }
    
    public final java.awt.Font getAwtFont() {
	return _font; 
    }
    
    public final boolean equals(Object anObject) {
	if ( this == anObject ) {
	    return true;
	}
	if ( getClass() != anObject.getClass() ) {
	    return false;
	}
return getKey().equals(((AWTFont)anObject).getKey());
    }
    
private java.awt.Font _font;
    
}

