//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.resource;

import figue.error.ShouldNotBeCalledError;
import figue.jape.PKFace;

import java.awt.Graphics;
import java.net.URL;
import java.awt.Component;

/**
 * [PENDING: update the comments] PortableFont;
 * @level advanced
 */

public final class PortableFont implements PortableFontInterface {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/resource/PortableFont.java,v $ $revision$ $Date: 1998/11/02 15:55:39 $ Copyright 1998 INRIA.";

    public PortableFont(Component aComponent,
			URL anUrl,
			String aName, int aDpi, double aScale) 
	throws WrongResourceException {
	_name = aName;
	_dpi = aDpi;
	_scale = aScale;
	final PKFace thePKFace = PKFace.open(aComponent,anUrl,aName,aDpi,aScale);
	if ( thePKFace == null ) {
	    _font = null;
	    throw new WrongResourceException("Can't allocate portable font: " + 
					     aName + String.valueOf(aDpi) + 
					     String.valueOf(aScale));
	}
	else {
	    _font = thePKFace.getFont();
	}
    }
    
    public final void draw(Graphics aContext, String aString, int x, int y) {
	_font.draw(aContext,aString,x,y);
    }

    public final FontMetricInterface getFontMetric() {
	if ( _metric == null ) {
	    _metric = new PortableFontMetric();
	}
	return _metric;
    }

    public final String getName() {
	return _name;
    }
    
    public final int getDpi() {
	return _dpi;
    }
    
    public final double getScale() {
	return _scale;
    }

    private String _name;
    private int _dpi;
    private double _scale;
    private figue.jape.Font _font;
    private FontMetricInterface _metric;

    final class PortableFontMetric implements FontMetricInterface {

	public PortableFontMetric() {
	}
	
	public final int getAscent() {
	    return _font.getAscent();
	}
	
	public final int getDescent() {
	    return _font.getDescent();
	}
	
	public final int bytesWidth(byte[] aByteArray) {
	    return stringWidth(new String(aByteArray));
	}
	
	public final int stringWidth(String aString) {
	    return _font.getWidth(aString);
	}
	
	public final int getHeight() {
	    return _font.getHeight();
	}
	
	public final FontInterface getFont() {
	    throw new ShouldNotBeCalledError(); /// TEMPO
	}
	
	public final boolean equals(Object anObject) {
	    if ( this == anObject ) {
		return true;
	    }
	    if ( getClass() != anObject.getClass() ) {
		return false;
	    }
	    PortableFontMetric theMetric = (PortableFontMetric)anObject;
	    return getFont().equals(theMetric.getFont());
	}
	
    }

}
