//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.utility;

import java.awt.Canvas;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Color;
import java.awt.Image;
import java.applet.Applet;
import java.awt.Toolkit;
import java.net.URL;
import java.awt.MediaTracker;
import java.net.MalformedURLException;
import java.awt.Font;

public final class AboutFigue extends Canvas {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/utility/AboutFigue.java,v $ $revision$ $Date: 1998/11/02 15:56:01 $ Copyright 1998 INRIA.";

    public AboutFigue(Applet anApplet) {
	super();
	_logo = null;
	_corner = null;
	_bottom = null;
	_right = null;
	setBackground(Color.white);
	final MediaTracker theTracker = new MediaTracker(this);
	final Toolkit theToolkit = getToolkit();
	final URL theCodeBaseUrl = anApplet.getCodeBase();
	try {
	    URL theUrl = new URL(theCodeBaseUrl,"figueLogo.gif");
	    try {
		_logo = theToolkit.getImage(theUrl);
	    }
	    catch ( Throwable aSecurityException ) {
		_logo = null;
	    }
	    theUrl = new URL(theCodeBaseUrl,"corner.gif");
	    try {
		_corner = theToolkit.getImage(theUrl);
	    }
	    catch ( Throwable aSecurityException ) {
		_corner = null;
	    }
	    theUrl = new URL(theCodeBaseUrl,"bottom.gif");
	    try {
		_bottom = theToolkit.getImage(theUrl);
	    }
	    catch ( Throwable aSecurityException ) {
		_bottom = null;
	    }
	    theUrl = new URL(theCodeBaseUrl,"right.gif");
	    try {
		_right = theToolkit.getImage(theUrl);
	    }
	    catch ( Throwable aSecurityException ) {
		_right = null;
	    }
	    if ( ( _logo != null ) && ( _right != null ) &&
		 ( _bottom != null ) && ( _corner != null ) ) {
		theTracker.addImage(_logo,1);
		theTracker.addImage(_corner,2);
		theTracker.addImage(_bottom,3);
		theTracker.addImage(_right,4);
		try {
		    theTracker.waitForID(1);
		    theTracker.waitForID(2);
		    theTracker.waitForID(3);
		    theTracker.waitForID(4);
		}
		catch ( InterruptedException  anException ) {
		    ;
		}
	    }
	}
	catch ( MalformedURLException anException ) {
	    ;
	}
	catch ( Throwable aSecurityException ) {
	    aSecurityException.printStackTrace();
	}
	if ( ( _logo != null ) && ( _right != null ) &&
		 ( _bottom != null ) && ( _corner != null ) ) {
	    show();
	}
    }
    
    public final void update(Graphics g) {
	paint(g);
    }

    public final void paint(Graphics g) {
	final int theThickness = 17;
	final Rectangle theSize = bounds();
	final int theWidth = theSize.width;
	final int theHeight = theSize.height;
	if ( ( _logo != null ) && ( _corner != null ) &&
	     ( _bottom != null ) && ( _right != null ) ) {
	    g.setFont(new Font("SansSerif",Font.ITALIC,11));
	    g.drawImage(_right,theWidth - theThickness,0,theThickness,theHeight - theThickness,Color.white,null);
	    g.drawImage(_bottom,0,theHeight - theThickness,theWidth - theThickness,theThickness,Color.white,null);
	    g.drawImage(_corner,theWidth - theThickness,theHeight - theThickness,theThickness,theThickness,Color.white,null);
	    g.drawString("Copyright  1998 INRIA. Loading ... please wait.",25,theHeight - theThickness - 10 - g.getFontMetrics().getHeight());
 	    g.drawImage(_logo,theWidth - _logo.getWidth(null) - theThickness - 25,25,Color.white,null);
	}
	else {
	    g.setColor(Color.white);
	    g.fillRect(0,0,theWidth,theHeight);
	}
    }
    
    public final void dispose() {
	if ( _logo != null ) {
	    _logo.flush();
	    _logo = null;
	}
	if ( _corner != null ) {
	    _corner.flush();
	    _corner = null;
	}
	if ( _bottom != null ) {
	    _bottom.flush();
	    _bottom = null;
	}
	if ( _right != null ) {
	    _right.flush();
	    _right = null;
	}
    }

    private Image _logo;
    private Image _corner;
    private Image _bottom;
    private Image _right;

}
