//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.utility;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Component;
import java.awt.Button;
import java.awt.FlowLayout;
import java.applet.Applet;
import java.awt.Frame;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.Panel;
import java.awt.Event;

public final class AlertBox extends Frame {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/utility/AlertBox.java,v $ $revision$ $Date: 1998/11/02 15:56:01 $ Copyright 1998 INRIA.";

    public AlertBox(Applet anApplet,
		    Component aComponent,
		    boolean anInformationFlag,
		    Color aColor) {
	super();
	setLayout(new BorderLayout());
	setBackground(aColor);
	final String theImageName;
	final int theIconWith = 36;
	final int theInterspace = 15;
	final int theInterline = 15;
	_button = new Button("Dismiss");
	if ( anInformationFlag ) {
	    _icon = new Icon(Icon.INFO,theIconWith,theIconWith,aColor,anApplet);
	}
	else {
	    _icon = new Icon(Icon.CAUTION,theIconWith,theIconWith,aColor,anApplet);
	}
	add(BorderLayout.NORTH,new Blank(theInterspace,theInterline,aColor));
	add(BorderLayout.EAST,new Blank(theInterspace,theInterline,aColor));
	add(BorderLayout.WEST,new Blank(theInterspace,theInterline,aColor));
	final Panel thePanel = new Panel();
	thePanel.setLayout(new BorderLayout());
	thePanel.add(BorderLayout.NORTH,new Blank(theInterspace,theInterline,aColor));
	final Panel theButtonPanel = new Panel();
	theButtonPanel.setLayout(new FlowLayout(FlowLayout.CENTER,0,0));
	theButtonPanel.add(_button);
	thePanel.add(BorderLayout.CENTER,theButtonPanel);
	thePanel.add(BorderLayout.SOUTH,new Blank(theInterspace,theInterline,aColor));
	add(BorderLayout.SOUTH,thePanel);
	final Panel theMainPanel = new Panel();
	theMainPanel.setLayout(new FlowLayout(FlowLayout.CENTER,0,0));
	theMainPanel.add(_icon);
	theMainPanel.add(new Blank(theInterspace,theInterline,aColor));
	theMainPanel.add(aComponent);
	add(BorderLayout.CENTER,theMainPanel);
	final Dimension theScreenSize = getToolkit().getScreenSize();
	final int theScreenWidth = theScreenSize.width;
 	final int theScreenHeight = theScreenSize.height;
	reshape(0,0,theScreenWidth / 2,theScreenHeight / 2);
	validate();
	pack();
	final Dimension theFinalSize = size();
	final int theFinalWidth = theFinalSize.width;
	final int theFinalHeight = theFinalSize.height;
	move(((theFinalWidth + theScreenWidth) / 2) - theFinalWidth,
	     ((theFinalHeight + theScreenHeight) / 2) - theFinalHeight);
	setTitle("Figue");
	setResizable(false);
	show();
	toFront();
    }

    public AlertBox(Applet anApplet,Component aComponent,boolean anInformationFlag) {
	this(anApplet,aComponent,anInformationFlag,Color.lightGray);
    }

    public AlertBox(Applet anApplet,Component aComponent) {
	this(anApplet,aComponent,Color.lightGray);
    }

    public AlertBox(Applet anApplet,Component aComponent, Color aColor) {
	this(anApplet,aComponent,true,aColor);
    }

    public final void update(Graphics g) {
	paint(g);
    }

    public final void paint(Graphics g) {
	final Rectangle theSize = bounds();
	final int theWidth = theSize.width;
	final int theHeight = theSize.height;
	g.setFont(new Font("SansSerif",Font.PLAIN,12));
	
    }

    public final boolean action(Event anEvent, Object arg) {
        final Object theTarget = anEvent.target;
         
        if ( theTarget == _button) {
	    dispose();
            return true;
        }
        return false;
    }

    public final void dispose() {
	if ( _icon != null ) {
	    _icon.dispose();
	    _icon = null;
	}
	_component = null;
	_button = null;
	hide();
    }
    
    private Icon _icon;
    private Component _component;
    private Button _button;

}

