//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.utility;

import java.net.URL;
import java.awt.MediaTracker;
import java.net.MalformedURLException;
import java.awt.Image;
import java.awt.Color;
import java.awt.Toolkit;
import java.applet.Applet;
import java.awt.Canvas;
import java.awt.Graphics;
import java.awt.Dimension;

public final class Icon extends Canvas {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/utility/Icon.java,v $ $revision$ $Date: 1998/11/02 15:56:02 $ Copyright 1998 INRIA.";

    public Icon(int anIconEnum,
		int aWidth, 
		int anHeight, 
		Color aColor,
		Applet anApplet) {
	super();
	_width = aWidth;
	_height = anHeight;
	_sign = null;
	_color = aColor;
	final MediaTracker theTracker = new MediaTracker(this);
	final Toolkit theToolkit = getToolkit();
	final URL theCodeBaseUrl = anApplet.getCodeBase();
	String theImageName;
	switch ( anIconEnum ) {
	case CAUTION: {
	    theImageName = "caution.gif";
	    break;
	}
	case INFO: {
	    theImageName = "info.gif";
	    break;
	}
	default: {
	    theImageName = "";
	    System.exit(0); // TEMPO
	}
	}
	try {
	    final URL theUrl= new URL(theCodeBaseUrl,theImageName);
	    try {
		_sign = theToolkit.getImage(theUrl);
	    }
	    catch ( Throwable aSecurityException ) {
		_sign = null;
	    }
	    if ( _sign != null ) {
		theTracker.addImage(_sign,1);
		try {
		    theTracker.waitForID(1);
		}
		catch ( InterruptedException  anException ) {
		    ;
		}
	    }
	}
	catch ( MalformedURLException anException ) {
	    ;
	}
	show();
    }

    public final void dispose() {
	if ( _sign != null ) {
	    _sign.flush();
	    _sign = null;
	} 
	_color = null;
    }

    public final void update(Graphics g) {
	paint(g);
    }

    public final void paint(Graphics g) {
	if ( _sign != null ) {
	    g.drawImage(_sign,0,0,_width,_height,_color,null);
	}
    }

    public final Dimension preferredSize() {
	return minimumSize();
    }

    public final Dimension minimumSize() {
	return new Dimension(_width,_height);
    }

    public final Dimension getMinimumSize() {
	return minimumSize();
    }

    public final Dimension getPreferredSize() {
	return preferredSize();
    }
    
    private Image _sign;
    private int _width;
    private int _height;
    private Color _color;
    
    public final static int CAUTION = 0;
    public final static int INFO = CAUTION + 1;
}
