//*****************************************************************************
//   Copyright (c) 1998 INRIA.
//   http://www.inria.fr/croap/aioli/modules/figue/web/credits.html#COPYRIGHT
//*****************************************************************************

package figue.utility;

import java.applet.AppletContext;
import java.net.URL;
import java.net.MalformedURLException;
import java.lang.Class;
import java.applet.Applet;
import java.applet.AppletStub;
import java.awt.GridLayout;
import java.awt.Graphics;
import java.awt.Color;

import figue.*;

public final class SwitchApplet extends DefaultApplet implements AppletStub {

    /**
     * For code identification with unix what command.
     * @level internal
     */
    public final static String VERSION_ID
	= "@(#) $Source: /net/croap/CVSROOT/figue/java/utility/SwitchApplet.java,v $ $revision$ $Date: 1998/11/04 15:57:21 $ Copyright 1998 INRIA.";

    public SwitchApplet() {
	super();
    }
  
    public final void init() {
    }

    private final JavaVersion getJavaVersion() {
	double theVersion = 1.0;
	boolean theSwingFlag = false;
	try {
	    Class.forName("java.awt.AWTEvent");
	    theVersion = 1.1;
	    Class.forName("import javax.swing.JComponent");
	    theSwingFlag = true;
	}
	catch ( ClassNotFoundException anException ) {
	}
	catch ( SecurityException anException ) {
	}
	return new JavaVersion(theVersion,theSwingFlag);
    }

    public final void appletResize(int aWidth, int anHeight) {
	resize(aWidth,anHeight);
    }

    public final void start() {
 	_splash = new SplashScreen(this);
	String theClassToLoad = getParameter(CLASS_TO_LOAD);
	if ( theClassToLoad == null ) {
	    reportUnexpectedFailure("Nothing to load.");
	}
	else {
	    String theMinimalJavaConfiguration = getParameter(MINIMAL_JAVA_CONFIGURATION);
	    if ( theMinimalJavaConfiguration == null) {
		theMinimalJavaConfiguration = "1.1";
	    }
	    final JavaVersion theExpectedJavaVersion;
	    if ( theMinimalJavaConfiguration.equals("1.1") ) {
		theExpectedJavaVersion = new JavaVersion(1.1);
	    }
	    else {
		if ( theMinimalJavaConfiguration.equals("1.2") ) {
		    theExpectedJavaVersion = new JavaVersion(1.2,true);
		}
		else {
		    if ( theMinimalJavaConfiguration.equals("SWING") ) {
			theExpectedJavaVersion = new JavaVersion(1.1,true);
		    }
		    else {
			theExpectedJavaVersion = new JavaVersion(1.0);
		    }
		}
	    }
	    final JavaVersion theJavaVersion = getJavaVersion();
	    if ( theJavaVersion.getVersion() < theExpectedJavaVersion.getVersion() ) {
		showAlertVersion("a Java " + theExpectedJavaVersion.getVersion() + " enabled browser",Color.lightGray,getFont());
	    }
	    else {
		if ( theExpectedJavaVersion.isSwingCompliant() && 
		     ( ! theJavaVersion.isSwingCompliant() ) ) {
		    showAlertVersion("a Swing enabled browser",Color.lightGray,getFont(),"SWING");
		}
		else {
		    try {
			_concreteApplet = (Applet)Class.forName(theClassToLoad).newInstance();
			_concreteClassLoadedFlag = true;
			setLayout(new GridLayout(1,1));
			add(_concreteApplet);
			_concreteApplet.setStub(this);
			_concreteApplet.init();
			_splash.done();
			_splash = null;
			
		    }
		    catch ( ClassNotFoundException anException ) {
			reportUnexpectedFailure("Unknown class: " + theClassToLoad);
		    }
		    catch ( IllegalAccessException anException ) {
			reportUnexpectedFailure("Can't instantiate: " + theClassToLoad);
		    }
		    catch ( InstantiationException anException ) {
			reportUnexpectedFailure("Not an applet: " + theClassToLoad);
		    }
		    catch ( ClassCastException anException ) {
			reportUnexpectedFailure("Not an applet: " + theClassToLoad);
		    }
		}
	    }
	}
	if ( _splash != null ) {
	    _splash.done();
	    _splash = null;
	}
	if ( _concreteClassLoadedFlag ) {
	    _concreteApplet.start();
	}
    }

    public final void stop() {
	if ( _concreteClassLoadedFlag ) {
	    _concreteApplet.stop();
	}
	dispose();
	super.stop();
    }

    public final void destroy() {
	if ( _concreteClassLoadedFlag ) {
	    _concreteApplet.destroy();
	}
 	dispose();
	super.destroy();
    }

    protected final void dispose() {
	_concreteClassLoadedFlag = false;
	_concreteApplet = null;
	removeAll();
	if ( _splash != null ) {
	    _splash.done();
	    _splash.dispose();
	    _splash = null;
	}
	super.dispose();
    }
    
    private final static String CLASS_TO_LOAD = "CLASS_TO_LOAD";
    private final static String MINIMAL_JAVA_CONFIGURATION = "JAVA_CONFIG";

    private boolean _concreteClassLoadedFlag = false;
    private Applet _concreteApplet = null;
    private SplashScreen _splash = null;

}
